@@ -8,7 +8,7 @@ void on_notebook1_switch_page (GtkNotebook *notebook, GtkNotebookPage *page, gui
 void on_mni_file_save_version (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_reverse (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_case_locase_activate (GtkMenuItem *menuitem, gpointer user_data);
-void on_mni_case_upcase_activate (GtkMenuItem *menuitem, gpointer user_data); 
+void on_mni_case_upcase_activate (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_new_file_activate (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_file_open_activate (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_file_save_activate (GtkMenuItem *menuitem, gpointer user_data);
@@ -76,7 +76,7 @@ void on_mni_test (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_refresh_hl (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_make_numbering (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_open_hotkeys (GtkMenuItem *menuitem, gpointer user_data);
-void tea_win_on_drag_data_lcb(GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *data, guint info, guint time); 
+void tea_win_on_drag_data_lcb(GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *data, guint info, guint time);
 void set_dnd_accept (GtkWidget *widget);
 gboolean on_tea_window_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
 void on_mni_snippet_click (GtkMenuItem *menuitem, gpointer user_data);
@@ -190,7 +190,7 @@ void on_mni_string_unquote (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_copy_filename (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_quote_string (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_file_close_all (GtkMenuItem *menuitem, gpointer user_data);
-void on_mni_markup_wiki_internal_link (GtkMenuItem *menuitem, gpointer user_data); 
+void on_mni_markup_wiki_internal_link (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_mm_change (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_imageplane (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_ins_np (GtkMenuItem *menuitem, gpointer user_data);
@@ -203,18 +203,18 @@ void on_mni_dvi_to_pdf (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_view_pdf (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_view_ps (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_split_to_tex_paragraphs (GtkMenuItem *menuitem, gpointer user_data);
-void on_mni_markup_list_item (GtkMenuItem *menuitem, gpointer user_data); 
+void on_mni_markup_list_item (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_markup_common (GtkMenuItem *menuitem, gpointer user_data);
-void on_mni_markup_tex_begend (GtkMenuItem *menuitem, gpointer user_data); 
-void on_mni_markup_col (GtkMenuItem *menuitem, gpointer user_data); 
-void on_mni_markup_row (GtkMenuItem *menuitem, gpointer user_data); 
-void on_mni_markup_table (GtkMenuItem *menuitem, gpointer user_data); 
-void on_mni_markup_itemizedlist (GtkMenuItem *menuitem, gpointer user_data); 
-void on_mni_markup_enumeratedlist (GtkMenuItem *menuitem, gpointer user_data); 
-void on_mni_markup_em (GtkMenuItem *menuitem, gpointer user_data); 
+void on_mni_markup_tex_begend (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_markup_col (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_markup_row (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_markup_table (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_markup_itemizedlist (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_markup_enumeratedlist (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_markup_em (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_calendar (GtkMenuItem *menuitem, gpointer user_data);
-void on_mni_markup_section (GtkMenuItem *menuitem, gpointer user_data); 
-void on_mni_markup_chap (GtkMenuItem *menuitem, gpointer user_data); 
+void on_mni_markup_section (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_markup_chap (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_markup_tex_common (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_pdflatex_process (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_close_current_tag (GtkMenuItem *menuitem, gpointer user_data);
@@ -259,3 +259,15 @@ void on_mni_test2 (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_find_in_files (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_mark_lines_with_fuzzy_nontrans (GtkMenuItem *menuitem, gpointer user_data);
 void on_mni_lorem_ipsum (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_rtfm_en (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_extract_at_each_line (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_refresh_ctags (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_browse_symbol_decl (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_extract_at_each_line_aft_sep (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_extract_at_each_line_bef_sep (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_mass_encode (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_self_check (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_quotes_to_facing_quotes (GtkMenuItem *menuitem, gpointer user_data);
+void tea_handle_param (const gchar *p);
+void on_mni_script_click (GtkMenuItem *menuitem, gpointer user_data);
+void on_mni_calc (GtkMenuItem *menuitem, gpointer user_data);
