@@ -2,7 +2,7 @@
                           interface.c  -  description
                              -------------------
     begin                : 2003
-    copyright            : (C)2003, 2004, 2005 by Peter 'Roxton' Semiletov
+    copyright            : (C)2003-2006 by Peter 'Roxton' Semiletov
     email                : peter.semiletov@gmail.com
  ***************************************************************************/
 
@@ -25,6 +25,7 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
+#include "tea_text_document.h"
 #include "callbacks.h"
 #include "interface.h"
 #include "tea_defs.h"
@@ -32,12 +33,7 @@
 #include "tea_funx.h"
 #include "tea_gtk_utils.h"
 #include "tea_tree_view.h"
-
-enum
-{
-  COL_TEXT = 0,
-  NUM_COLS
-};
+#include "rox_strings.h" // Needed for fill_entities_special_menu
 
 
 void ui_update (void)
@@ -66,14 +62,14 @@ void mpb_stop (void)
   g_source_remove (id_mpb);
   gtk_widget_hide (pb_status);
 }
- 
+
 
 static GtkWidget* create_hardcoded_toolbar (void)
 {
   GtkWidget *toolbar = gtk_toolbar_new ();
-  
+
   gtk_toolbar_insert_stock (toolbar, GTK_STOCK_PREFERENCES, NULL, NULL, show_options, NULL, 0);
-   
+
   tea_gtk_toolbar_prepend_space (toolbar);
   tea_gtk_toolbar_prepend_space (toolbar);
   tea_gtk_toolbar_prepend_space (toolbar);
@@ -89,9 +85,9 @@ static GtkWidget* create_hardcoded_toolbar (void)
   gtk_toolbar_insert_stock (toolbar, GTK_STOCK_OPEN, _("Open a file"), NULL, tea_file_open_dialog, NULL, 0);
   gtk_toolbar_insert_stock (toolbar, GTK_STOCK_NEW, _("Create a new file"), NULL, file_new, NULL, 0);
 
-  gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_ICONS); 
-  //gtk_toolbar_set_icon_size (toolbar, GTK_ICON_SIZE_SMALL_TOOLBAR);  
-  
+  gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_ICONS);
+  //gtk_toolbar_set_icon_size (toolbar, GTK_ICON_SIZE_SMALL_TOOLBAR);
+
   if (confile.do_show_main_toolbar)
      gtk_widget_show (toolbar);
   else
@@ -113,7 +109,7 @@ void fill_entities_special_menu (void)
   fill_entities ();
   GList *l = ht_to_glist (ht_entities, opt_ht_to_glist_keys);
   l = glist_strings_sort (l);
-  build_menu_from_glist (l, mni_insert_char_entities_menu, on_mni_entity);  
+  build_menu_from_glist (l, mni_insert_char_entities_menu, on_mni_entity);
   glist_strings_free (l);
 }
 
@@ -142,7 +138,6 @@ GtkWidget* create_tea_main_window (void)
                     tea_main_window);
 
   mainvbox = gtk_vbox_new (FALSE, UI_PACKFACTOR);
-  gtk_widget_set_name (mainvbox, "mainvbox");
   gtk_widget_show (mainvbox);
   gtk_container_add (GTK_CONTAINER (tea_main_window), mainvbox);
 
@@ -155,20 +150,18 @@ GtkWidget* create_tea_main_window (void)
                     G_CALLBACK (on_bt_close_current_doc),
                     NULL);
 
-
   menubar1 = gtk_menu_bar_new ();
 
   gtk_widget_set_name (menubar1, "menubar1");
   gtk_widget_show (menubar1);
 
-//  gtk_box_pack_start (GTK_BOX (mainvbox), menubar1, FALSE, FALSE, UI_PACKFACTOR);
   gtk_box_pack_start (GTK_BOX (hb_topbox), menubar1, FALSE, FALSE, UI_PACKFACTOR);
   gtk_box_pack_start (GTK_BOX (hb_topbox), bt_close_current_doc, FALSE, FALSE, UI_PACKFACTOR);
- 
+
   gtk_tooltips_set_tip (tooltips,
                         bt_close_current_doc,
                         _("Close a current document"),
-                        NULL);  
+                        NULL);
 
 
   mni_temp = new_menu_item (_("File"), menubar1, NULL);
@@ -179,16 +172,16 @@ GtkWidget* create_tea_main_window (void)
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_n, GDK_CONTROL_MASK,
                               GTK_ACCEL_VISIBLE);
-  
+
   mni_temp = new_menu_item (_("New Kwas (file manager)"), mni_file_menu, on_mni_new_kwas);
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_N, GDK_MOD1_MASK,
                               GTK_ACCEL_VISIBLE);
 
-  //mni_temp = new_menu_item (_("test"), mni_file_menu, on_mni_test);
-  //mni_temp = new_menu_item (_("test2"), mni_file_menu, on_mni_test2);
+//  mni_temp = new_menu_item ("test", mni_file_menu, on_mni_test);
+  //mni_temp = new_menu_item ("test2", mni_file_menu, on_mni_test2);
+
 
-    
   mni_temp = new_menu_item (_("Crapbook"), mni_file_menu, on_mni_file_crapbook);
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_M, GDK_MOD1_MASK,
@@ -207,15 +200,15 @@ GtkWidget* create_tea_main_window (void)
                               GTK_ACCEL_VISIBLE);
 
   mni_temp = new_menu_item (_("The last opened file"), mni_file_menu, on_mni_load_last_file);
-  
+
   mni_temp = new_menu_item (_("Open different"), mni_file_menu, NULL);
   mni_opendifferent_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_item (_("Open movie"), mni_opendifferent_menu, on_mni_open_open_movie);
   mni_temp = new_menu_item (_("Open snippet"), mni_opendifferent_menu, on_mni_snippet_file_open_activate);
   mni_temp = new_menu_item (_("Open template"), mni_opendifferent_menu, on_mni_templ_file_open_activate);
   mni_temp = new_menu_item (_("Open session file"), mni_opendifferent_menu, on_mni_session_file_open_activate);
-  mni_temp = new_menu_item (_("Open from the Famous text entry"), mni_opendifferent_menu, on_mni_fam_open);  
-  
+  mni_temp = new_menu_item (_("Open from the Famous text entry"), mni_opendifferent_menu, on_mni_fam_open);
+
   mni_temp = new_menu_sep (mni_file_menu);
 
   mni_recent_list = new_menu_item (_("Recent files"), mni_file_menu, NULL);
@@ -223,13 +216,13 @@ GtkWidget* create_tea_main_window (void)
 
   mni_bookmarks_root = new_menu_item (_("Bookmarks"), mni_file_menu, NULL);
   mni_bookmarks_menu = new_menu_submenu (mni_bookmarks_root);
- 
-  mni_sessions = new_menu_item (_("Sessions"), mni_file_menu, NULL); 
+
+  mni_sessions = new_menu_item (_("Sessions"), mni_file_menu, NULL);
   mni_sessions_menu = new_menu_submenu (mni_sessions);
 
-  mni_templates = new_menu_item (_("Templates"), mni_file_menu, NULL); 
+  mni_templates = new_menu_item (_("Templates"), mni_file_menu, NULL);
   mni_templates_menu = new_menu_submenu (mni_templates);
-    
+
   mni_temp = new_menu_sep (mni_file_menu);
 
   mni_temp = new_menu_item (_("Save"), mni_file_menu, on_mni_file_save_activate);
@@ -241,9 +234,9 @@ GtkWidget* create_tea_main_window (void)
 
   mni_temp = new_menu_item (_("Save different"), mni_file_menu, NULL);
   mni_savedifferent_menu = new_menu_submenu (mni_temp);
- 
+
   mni_temp = new_menu_item (_("Save session"), mni_savedifferent_menu, on_mni_file_save_session);
-  
+
   mni_temp = new_menu_item (_("Save as template"), mni_savedifferent_menu, on_mni_file_save_as_template_activate);
   mni_temp = new_menu_item (_("Save as snippet"), mni_savedifferent_menu, on_mni_file_save_as_snippet_activate);
 
@@ -256,9 +249,9 @@ GtkWidget* create_tea_main_window (void)
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_b, GDK_CONTROL_MASK,
                               GTK_ACCEL_VISIBLE);
-  
+
   mni_temp = new_menu_item (_("Save all"), mni_savedifferent_menu, on_mni_save_all);
- 
+
   mni_temp = new_menu_item (_("Revert to saved"), mni_file_menu, on_mni_file_revert_to_saved);
 
   mni_temp = new_menu_sep (mni_file_menu);
@@ -271,23 +264,23 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_item (_("Hotkeys config"), mni_file_configs_menu, on_mni_open_hotkeys);
   mni_temp = new_menu_item (_("External programs list config"), mni_file_configs_menu, on_mni_ext_programs_file_open);
   mni_temp = new_menu_item (_("Bookmarks file"), mni_file_configs_menu, on_mni_file_open_bookmark);
-  mni_temp = new_menu_item (_("Open the crashfile"), mni_file_configs_menu, on_mni_crashfile_open);  
-  
+  mni_temp = new_menu_item (_("Open the crashfile"), mni_file_configs_menu, on_mni_crashfile_open);
+
   mni_temp = new_menu_item (_("Preferences"), mni_file_menu, on_mni_show_options);
- 
+
   mni_temp = new_menu_sep (mni_file_menu);
 
   mni_temp = new_menu_item (_("Close current"), mni_file_menu, on_mni_file_close_current);
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_w, GDK_CONTROL_MASK,
                               GTK_ACCEL_VISIBLE);
-  
+
   mni_temp = new_menu_item (_("Close all"), mni_file_menu, on_mni_file_close_all);
 /*
   GtkActionGroup *ag_file = gtk_action_group_new ("file");
 
-  mni_temp = new_action_item (ag_file, 
-                            mni_file_menu, 
+  mni_temp = new_action_item (ag_file,
+                            mni_file_menu,
                             "quit",
                             "Quit",
                             NULL,
@@ -314,7 +307,7 @@ GtkWidget* create_tea_main_window (void)
 
   mni_temp = new_menu_item (_("Copy"), mni_edit_menu, on_mni_edit_copy);
   mni_temp = new_menu_item (_("Copy all"), mni_edit_menu, on_mni_edit_copy_all);
-  mni_temp = new_menu_item (_("Copy current URL"), mni_edit_menu, on_mni_edit_copy_current_url);
+  //mni_temp = new_menu_item (_("Copy current URL"), mni_edit_menu, on_mni_edit_copy_current_url);
   mni_temp = new_menu_item (_("Copy current filename"), mni_edit_menu, on_mni_copy_filename);
 
   mni_temp = new_menu_item (_("Cut!"), mni_edit_menu, on_mni_edit_cut);
@@ -326,22 +319,6 @@ GtkWidget* create_tea_main_window (void)
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group, GDK_Up, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
   mni_temp = new_menu_item (_("Move line down"), mni_edit_menu, on_mni_move_line_down);
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group, GDK_Down, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
-  
-  mni_temp = new_menu_sep (mni_edit_menu);
-
-  mni_temp = new_menu_item (_("Marking"), mni_edit_menu, NULL);
-  mni_mark_menu = new_menu_submenu (mni_temp);
-  mni_temp = new_menu_tof (mni_mark_menu);
- 
-  mni_temp = new_menu_item (_("Mark"), mni_mark_menu, on_mni_paint);
-  mni_temp = new_menu_item (_("Copy marked"), mni_mark_menu, on_mni_copy_painted);
-  mni_temp = new_menu_item (_("Unmark all"), mni_mark_menu, on_mni_unpaint_all);
-  mni_temp = new_menu_item (_("Mark each n-th line"), mni_mark_menu, on_mni_mark_each_n_line);
-  mni_temp = new_menu_item (_("Mark lines those contains a string"), mni_mark_menu, on_mni_mark_all_lines_with_string);
-  mni_temp = new_menu_item (_("Mark by regexp"), mni_mark_menu, on_mni_mark_on_regexp);
-  mni_temp = new_menu_item (_("Mark all fuzzy and empty translations"), mni_mark_menu, on_mni_mark_lines_with_fuzzy_nontrans);
-
-
 
   mni_temp = new_menu_sep (mni_edit_menu);
 
@@ -349,16 +326,16 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_item (_("Copy to new"), mni_edit_menu, on_mni_copy_to_new);
   mni_temp = new_menu_item (_("Paste to new"), mni_edit_menu, on_mni_paste_to_new);
   mni_temp = new_menu_item (_("Append to the Crapbook"), mni_edit_menu, on_mni_edit_append_to_crapbook);
-  
+
   mni_temp = new_menu_item (_("Select all"), mni_edit_menu, on_mni_edit_select_all);
-  
+
   mni_temp = new_menu_item (_("Delete current line"), mni_edit_menu, on_mni_edit_delete_current_line);
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_y, GDK_CONTROL_MASK,
                               GTK_ACCEL_VISIBLE);
 
   mni_temp = new_menu_item (_("Delete"), mni_edit_menu, on_mni_edit_delete);
-  
+
   mni_temp = new_menu_sep (mni_edit_menu);
 
   mni_edit_undo = new_menu_item (_("Undo"), mni_edit_menu, on_mni_undo);
@@ -377,6 +354,20 @@ GtkWidget* create_tea_main_window (void)
   mni_quest_menu =  new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_quest_menu);
 
+  mni_temp = new_menu_item (_("Marking"), mni_quest_menu, NULL);
+  mni_mark_menu = new_menu_submenu (mni_temp);
+  mni_temp = new_menu_tof (mni_mark_menu);
+
+  mni_temp = new_menu_item (_("Mark"), mni_mark_menu, on_mni_paint);
+  mni_temp = new_menu_item (_("Copy marked"), mni_mark_menu, on_mni_copy_painted);
+  mni_temp = new_menu_item (_("Unmark all"), mni_mark_menu, on_mni_unpaint_all);
+  mni_temp = new_menu_item (_("Mark each n-th line"), mni_mark_menu, on_mni_mark_each_n_line);
+  mni_temp = new_menu_item (_("Mark lines those contains a string"), mni_mark_menu, on_mni_mark_all_lines_with_string);
+  mni_temp = new_menu_item (_("Mark by regexp"), mni_mark_menu, on_mni_mark_on_regexp);
+  mni_temp = new_menu_item (_("Mark all fuzzy and empty translations"), mni_mark_menu, on_mni_mark_lines_with_fuzzy_nontrans);
+  mni_temp = new_menu_item (_("Find and mark"), mni_mark_menu, on_mni_search_and_paint);
+  mni_temp = new_menu_item (_("Mark lines > n characters"), mni_mark_menu, on_mni_paint_lines_bigger_than);
+
 
   mni_temp = new_menu_item (_("Find in files"), mni_quest_menu, on_mni_find_in_files);
 
@@ -386,7 +377,7 @@ GtkWidget* create_tea_main_window (void)
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_f, GDK_CONTROL_MASK,
                               GTK_ACCEL_VISIBLE);
- 
+
   mni_temp = new_menu_item (_("Find case insensitive"), mni_quest_menu, on_mni_quest_find_ncase);
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_f, GDK_CONTROL_MASK | GDK_SHIFT_MASK,
@@ -403,11 +394,7 @@ GtkWidget* create_tea_main_window (void)
                               GTK_ACCEL_VISIBLE);
 
   mni_temp = new_menu_item (_("Replace"), mni_quest_menu, on_mni_replace_in_place);
-  
 
-  mni_temp = new_menu_item (_("Find and mark"), mni_quest_menu, on_mni_search_and_paint);
-  mni_temp = new_menu_item (_("Mark lines > n characters"), mni_quest_menu, on_mni_paint_lines_bigger_than);
-  
   mni_temp = new_menu_item (_("Goto line"), mni_quest_menu, on_mni_goto_line);
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_g, GDK_MOD1_MASK,
@@ -429,17 +416,17 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_item (_("Markup"), menubar1, NULL);
   mni_markup_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_markup_menu);
- 
+
   mni_temp = new_menu_item (_("Markup mode"), mni_markup_menu, NULL);
   mni_mm_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_mm_menu);
-  
+
   mni_temp = new_menu_item ("HTML", mni_mm_menu, on_mni_mm_change);
   mni_temp = new_menu_item ("XHTML", mni_mm_menu, on_mni_mm_change);
   mni_temp = new_menu_item ("Wikipedia", mni_mm_menu, on_mni_mm_change);
   mni_temp = new_menu_item ("LaTeX", mni_mm_menu, on_mni_mm_change);
   mni_temp = new_menu_item ("Docbook", mni_mm_menu, on_mni_mm_change);
-  
+
   mni_temp = new_menu_item (_("Wikipedia elements"), mni_markup_menu, NULL);
   mni_wiki_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_wiki_menu);
@@ -503,8 +490,8 @@ GtkWidget* create_tea_main_window (void)
   mni_html_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_html_menu);
 
-  mni_temp = new_menu_item (_("Build-in template"), mni_html_menu, on_mni_html_default_template);
-  
+  mni_temp = new_menu_item (_("Built-in template"), mni_html_menu, on_mni_html_default_template);
+
   mni_temp = new_menu_item (_("Strip tags"), mni_html_menu, on_mni_html_strip_tags);
   mni_temp = new_menu_item (_("Enclose selected link into tags"), mni_html_menu, on_mni_html_enclose_link);
   mni_temp = new_menu_item (_("Convert tags to entities"), mni_html_menu, on_mni_convert_ascii_2_html);
@@ -526,16 +513,16 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_item (_("Emphasis"), mni_markup_menu, on_mni_markup_em);
 
   mni_temp = new_menu_item (_("Underline"), mni_markup_menu, on_mni_Markup_underline_activate);
-  
+
   mni_temp = new_menu_sep (mni_markup_menu);
 
   mni_temp = new_menu_item (_("Close the current tag"), mni_markup_menu, on_mni_close_current_tag);
-  
+
   mni_temp = new_menu_item (_("Comment"), mni_markup_menu, on_mni_Markup_comment);
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_slash, GDK_MOD1_MASK,
                               GTK_ACCEL_VISIBLE);
-  
+
   mni_temp = new_menu_item (_("Chapter"), mni_markup_menu, on_mni_markup_chap);
   mni_temp = new_menu_item (_("Section"), mni_markup_menu, on_mni_markup_section);
 
@@ -555,7 +542,7 @@ GtkWidget* create_tea_main_window (void)
                               GTK_ACCEL_VISIBLE);
 
   mni_temp = new_menu_item (_("Image"), mni_markup_menu, on_mni_image_activate);
- 
+
   mni_temp = new_menu_item (_("Line break"), mni_markup_menu, on_mni_Markup_br);
 
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
@@ -589,7 +576,7 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_item (_("Table"), mni_table_menu, on_mni_markup_table);
   mni_temp = new_menu_item (_("Row"), mni_table_menu, on_mni_markup_row);
   mni_temp = new_menu_item (_("Cell"), mni_table_menu, on_mni_markup_col);
-  
+
   mni_temp = new_menu_item (_("H"), mni_markup_menu, NULL);
   mni_markup_header_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_markup_header_menu);
@@ -599,7 +586,7 @@ GtkWidget* create_tea_main_window (void)
 
   for (c = 1; c <= 6; c++)
       {
-       h = g_strdup_printf ("h%d", c);                
+       h = g_strdup_printf ("h%d", c);
        mni_temp = new_menu_item (h, mni_markup_header_menu, on_mni_markup_common);
        g_free (h);
       }
@@ -622,7 +609,7 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_tof (mni_functions_menu);
 
   mni_temp = new_menu_item (_("Calendar"), mni_functions_menu, on_mni_calendar);
-  
+
   mni_temp = new_menu_item (_("Document stats"), mni_functions_menu, on_mni_utils_stats);
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_F11, 0,
@@ -633,14 +620,10 @@ GtkWidget* create_tea_main_window (void)
 
   mni_spellcheck = new_menu_item (_("Spell-checker languages"), mni_functions_menu, NULL);
   mni_spellcheck_menu = new_menu_submenu (mni_spellcheck);
-
   mni_temp = new_menu_item (_("Spell check"), mni_functions_menu, on_mni_spellcheck_def);
-
   mni_temp = new_menu_item (_("Suggest a word"), mni_functions_menu, on_mni_suggest);
 
 
-  
-
   mni_temp = new_menu_sep (mni_functions_menu);
 
   mni_latex = new_menu_item ("LaTeX", mni_functions_menu, NULL);
@@ -655,12 +638,12 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_item (_("View PDF"), mni_latex_menu, on_mni_view_pdf);
   mni_temp = new_menu_item (_("View dvi"), mni_latex_menu, on_mni_view_dvi);
   mni_temp = new_menu_item (_("View PS"), mni_latex_menu, on_mni_view_ps);
-  
+
 
   mni_ide = new_menu_item (_("IDE"), mni_functions_menu, NULL);
   mni_ide_menu = new_menu_submenu (mni_ide);
   mni_temp = new_menu_tof (mni_ide_menu);
-  
+
   mni_temp = new_menu_item (_("New project"), mni_ide_menu, on_mni_project_new);
   mni_temp = new_menu_item (_("Open project"), mni_ide_menu, on_mni_project_open);
   mni_temp = new_menu_item (_("Save project"), mni_ide_menu, on_mni_show_project_save);
@@ -668,7 +651,9 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_item (_("Project properties"), mni_ide_menu, on_mni_show_project_props);
   mni_temp = new_menu_item (_("Make"), mni_ide_menu, on_mni_project_make);
   mni_temp = new_menu_item (_("Execute"), mni_ide_menu, on_mni_show_project_run);
- 
+  mni_temp = new_menu_item (_("Refresh tags with ctags"), mni_ide_menu, on_mni_refresh_ctags);
+
+
   mni_temp = new_menu_item (_("UNIversal Text AnalyZer"), mni_functions_menu, NULL);
   mni_utils_unitaz_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_utils_unitaz_menu);
@@ -678,11 +663,19 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_item (_("Call UNITAZ with sorting by count case insensitive"), mni_utils_unitaz_menu, on_mni_tools_unitaz_sort_by_count_caseinsens);
   mni_temp = new_menu_item (_("Call UNITAZ plain"), mni_utils_unitaz_menu, on_mni_tools_unitaz_plain);
   mni_temp = new_menu_item (_("Extract words"), mni_utils_unitaz_menu, on_mni_extract_words);
-  
+
   mni_temp = new_menu_sep (mni_functions_menu);
 
+  mni_scripts = new_menu_item (_("Scripts"), mni_functions_menu, NULL);
+  mni_scripts_menu = new_menu_submenu (mni_scripts);
+
+
   mni_snippets = new_menu_item (_("Snippets"), mni_functions_menu, NULL);
   mni_snippets_menu = new_menu_submenu (mni_snippets);
+  mni_tools = new_menu_item (_("Tools"), mni_functions_menu, NULL);
+  mni_tools_menu = new_menu_submenu (mni_tools);
+
+  mni_temp = new_menu_item (_("Mass encode tool"), mni_tools_menu, on_mni_mass_encode);
 
   mni_temp = new_menu_sep (mni_functions_menu);
 
@@ -695,7 +688,7 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_tof (mni_insert_char_entities_menu);
 
   fill_entities_special_menu ();
-  
+
   mni_temp = new_menu_item (_("Insert DOCTYPE"), mni_ins_menu, NULL);
   mni_insert_doctype_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_insert_doctype_menu);
@@ -718,12 +711,12 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_item (_("Case"), mni_functions_menu, NULL);
   mni_functions_case_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_functions_case_menu);
-                        
+
   mni_temp = new_menu_item (_("UPCASE"), mni_functions_case_menu, on_mni_case_upcase_activate);
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_Up, GDK_CONTROL_MASK,
                               GTK_ACCEL_VISIBLE);
-                        
+
   mni_temp = new_menu_item (_("lowcase"), mni_functions_case_menu, on_mni_case_locase_activate);
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_Down, GDK_CONTROL_MASK,
@@ -732,32 +725,34 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_item (_("Upcase The First Letter Of Each Word"), mni_functions_case_menu, on_mni_upcase_each_first_letter_ofw);
 
   mni_temp = new_menu_item (_("cRAcKeriZE"), mni_functions_case_menu, on_mni_crackerize);
- 
+
   mni_temp = new_menu_item (_("Numbers"), mni_functions_menu, NULL);
   mni_functions_number_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_functions_number_menu);
 
+
+  mni_temp = new_menu_item (_("Calculate"), mni_functions_number_menu, on_mni_calc);
   mni_temp = new_menu_item (_("Arabian to Roman"), mni_functions_number_menu, on_mni_Functions_number_arabian2roman);
   mni_temp = new_menu_item (_("HEX to decimal"), mni_functions_number_menu, on_mni_Functions_number_hex2dec);
   mni_temp = new_menu_item (_("Binary to decimal"), mni_functions_number_menu, on_mni_Functions_number_bin2dec);
   mni_temp = new_menu_item (_("Counter"), mni_functions_number_menu, on_mni_make_numbering);
-  
+
   mni_temp = new_menu_item (_("Format"), mni_functions_menu, NULL);
   mni_format_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_format_menu);
 
   mni_temp = new_menu_item (_("Browser to text"), mni_format_menu, on_mni_copied_html_to_ftext);
-  
+
   mni_temp = new_menu_item (_("Kill formatting"), mni_format_menu, on_mni_kill_formatting);
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_D, GDK_MOD1_MASK,
                               GTK_ACCEL_VISIBLE);
 
   mni_temp = new_menu_item (_("Kill formatting on each line"), mni_format_menu, on_mni_kill_formatting_on_each_line);
-  
+
   mni_temp = new_menu_item (_("Wrap raw at position"), mni_format_menu, on_mni_wrap_raw);
   mni_temp = new_menu_item (_("Wrap on spaces at position"), mni_format_menu, on_mni_wrap_on_spaces);
-  
+
   mni_temp = new_menu_item (_("Kill leading whitespace in each line"), mni_format_menu, on_mni_strings_remove_leading_whitespaces);
   mni_temp = new_menu_item (_("Kill trailing whitespace in each line"), mni_format_menu, on_mni_strings_remove_trailing_whitespaces);
 
@@ -774,6 +769,8 @@ GtkWidget* create_tea_main_window (void)
   mni_functions_string_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_functions_string_menu);
 
+  mni_temp = new_menu_item (_("Double quotes to facing quotes"), mni_functions_string_menu, on_mni_quotes_to_facing_quotes);
+
   mni_temp = new_menu_item (_("CSV-like table to LaTeX table"), mni_functions_string_menu, on_mni_csv_to_latex_table);
   mni_temp = new_menu_item (_("Convert usual quotes to TeX quotes"), mni_functions_string_menu, on_mni_quotes_to_tex);
   mni_temp = new_menu_item (_("Escape and quote"), mni_functions_string_menu, on_mni_quote_string);
@@ -783,6 +780,7 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_item (_("Remove blank lines"), mni_functions_string_menu, on_mni_func_strings_removeblanks);
   mni_temp = new_menu_item (_("Kill duplicates"), mni_functions_string_menu, on_mni_str_kill_dups);
   mni_temp = new_menu_item (_("Numerate lines"), mni_functions_string_menu, on_mni_count_string_list);
+
   mni_temp = new_menu_item (_("Shuffle lines"), mni_functions_string_menu, on_mni_shuffle_strings);
   mni_temp = new_menu_item (_("Convert tabs to spaces"), mni_functions_string_menu, on_mni_tabs_to_spaces);
   mni_temp = new_menu_item (_("Convert spaces to tabs"), mni_functions_string_menu, on_mni_spaces_to_tabs);
@@ -799,10 +797,14 @@ GtkWidget* create_tea_main_window (void)
   mni_filter_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_filter_menu);
 
+  mni_temp = new_menu_item (_("Extract at the each line"), mni_filter_menu, on_mni_extract_at_each_line);
+  mni_temp = new_menu_item (_("Extract at the each line before a separator"), mni_filter_menu, on_mni_extract_at_each_line_bef_sep);
+  mni_temp = new_menu_item (_("Extract at the each line after a separator"), mni_filter_menu, on_mni_extract_at_each_line_aft_sep);
+
   mni_temp = new_menu_item (_("Kill lines that contain a phrase"), mni_filter_menu, on_mni_func_filter_exclude_from_list);
   mni_temp = new_menu_item (_("Kill all lines except with a phrase"), mni_filter_menu, on_mni_func_filter_antiexclude_from_list);
-  
-  mni_temp = new_menu_item (_("Kill all lines < N characters"), mni_filter_menu, on_mni_filter_kill_lesser);
+
+  mni_temp = new_menu_item (_("Kill all lines <= N characters"), mni_filter_menu, on_mni_filter_kill_lesser);
   mni_temp = new_menu_item (_("Kill all lines > N characters"), mni_filter_menu, on_mni_filter_kill_more_than);
 
   mni_temp = new_menu_item (_("End of line"), mni_functions_menu, NULL);
@@ -817,7 +819,7 @@ GtkWidget* create_tea_main_window (void)
 
   mni_temp = new_menu_item (_("Last used"), mni_functions_menu, on_mni_repeat_the_last);
 
-    
+
   mni_tabs = new_menu_item (_("Tabs"), menubar1, NULL);
   mni_tabs_menu = new_menu_submenu (mni_tabs);
 
@@ -860,11 +862,13 @@ GtkWidget* create_tea_main_window (void)
                               GTK_ACCEL_VISIBLE);
 
   mni_temp = new_menu_item (_("Edit at cursor"), mni_nav_menu, on_mni_file_edit_at_cursor);
-  
+  mni_temp = new_menu_item (_("Browse the symbol's declaration"), mni_nav_menu, on_mni_browse_symbol_decl);
+
+
   mni_temp = new_menu_item (_("View"), menubar1, NULL);
   mni_view_menu = new_menu_submenu (mni_temp);
   mni_temp = new_menu_tof (mni_view_menu);
- 
+
   mni_co = new_menu_item (_("Encoding"), mni_view_menu, NULL);
   mni_co_menu = new_menu_submenu (mni_co);
 
@@ -913,7 +917,11 @@ GtkWidget* create_tea_main_window (void)
   mni_temp = new_menu_tof (mni_what_menu);
 
   mni_temp = new_menu_item (_("About"), mni_what_menu, on_mni_about_activate);
+ // mni_temp = new_menu_item (_("Self check"), mni_what_menu, on_mni_self_check);
+
   mni_temp = new_menu_item (_("Read the fine manual"), mni_what_menu, on_mni_rtfm);
+  mni_temp = new_menu_item (_("Read the fine English manual anyway"), mni_what_menu, on_mni_rtfm_en);
+  mni_temp = new_menu_item (_("Self check"), mni_what_menu, on_mni_self_check);
 
   mni_temp = new_menu_sep (mni_what_menu);
   mni_temp = new_menu_item ("NEWS", mni_what_menu, on_mni_help_item);
@@ -924,7 +932,7 @@ GtkWidget* create_tea_main_window (void)
   tb_main_toolbar = create_hardcoded_toolbar ();
   gtk_box_pack_start (GTK_BOX (mainvbox), tb_main_toolbar, FALSE, TRUE, UI_PACKFACTOR);
 
-  
+
   vpaned1 = gtk_vpaned_new ();
   gtk_widget_show (vpaned1);
   gtk_box_pack_start (GTK_BOX (mainvbox), vpaned1, TRUE, TRUE, UI_PACKFACTOR);
@@ -935,15 +943,15 @@ GtkWidget* create_tea_main_window (void)
   notebook1 = gtk_notebook_new ();
   gtk_widget_set_name (notebook1, "notebook1");
   gtk_widget_show (notebook1);
-   
+
   gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook1), TRUE);
 
   gtk_paned_pack1 (GTK_PANED (vpaned1), notebook1, TRUE, TRUE);
-  
+
   vbox2 = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (vbox2);
   gtk_paned_pack2 (GTK_PANED (vpaned1), vbox2, TRUE, TRUE);
-  
+
   scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
   gtk_widget_show (scrolledwindow3);
 
@@ -962,11 +970,11 @@ GtkWidget* create_tea_main_window (void)
   gtk_box_pack_start (GTK_BOX (vbox2), fam_hbox, FALSE, FALSE, UI_PACKFACTOR);
 
 
-  ent_search = gtk_entry_new ();   
+  ent_search = gtk_entry_new ();
   gtk_widget_show (ent_search);
   gtk_box_pack_start (GTK_BOX (fam_hbox), ent_search, TRUE, TRUE, UI_PACKFACTOR);
 
-  GtkWidget* bt_fam = tea_stock_button (GTK_STOCK_EXECUTE);
+  GtkWidget* bt_fam = tea_stock_button (GTK_STOCK_FIND);
   gtk_widget_show (bt_fam);
   gtk_box_pack_start (GTK_BOX (fam_hbox), bt_fam, FALSE, FALSE, UI_PACKFACTOR);
 
@@ -1008,7 +1016,6 @@ GtkWidget* create_tea_main_window (void)
                     G_CALLBACK (on_notebook1_switch_page),
                     NULL);
 
-
   gtk_window_add_accel_group (GTK_WINDOW (tea_main_window), accel_group);
 
   set_dnd_accept (tea_main_window);
@@ -1046,11 +1053,11 @@ GtkWidget* create_tea_main_window (void)
 
 #ifdef CHAI_MODE
 
-log_to_memo ("TEA 12.0 @ http://tea.linux.kiev.ua", NULL, LM_GREET);
+log_to_memo ("TEA 14.2.3 @ http://tea-editor.sourceforge.net", NULL, LM_GREET);
 
 #else
 
-log_to_memo ("TEA (original) 12.0 @ http://tea.linux.kiev.ua", NULL, LM_GREET);
+log_to_memo ("TEA (original) 14.2.3 @ http://tea-editor.sourceforge.net", NULL, LM_GREET);
 
 #endif
 
@@ -1058,7 +1065,7 @@ log_to_memo ("TEA (original) 12.0 @ http://tea.linux.kiev.ua", NULL, LM_GREET);
   mni_last = NULL;
   ui_init ();
 
-  gtk_entry_set_text (ent_search, "");  
+  gtk_entry_set_text (ent_search, "");
 
   gtk_tooltips_set_tip (tooltips,
                         ent_search,
@@ -1069,15 +1076,15 @@ log_to_memo ("TEA (original) 12.0 @ http://tea.linux.kiev.ua", NULL, LM_GREET);
      gtk_window_set_default_icon_from_file (TEA_PIX_MAINICON, NULL);
 
   //g_object_set (G_OBJECT (tag_type),  "font", confile.tag_type_font, NULL);
- 
+
  // g_object_set (G_OBJECT (menubar1), "font-name", "FreeSans 18", NULL);
-  
+
   //GtkStyle *stl = gtk_style_new ();
  // PangoFontDescription *font_desc = pango_font_description_new ();
   //pango_font_description_set_family
     //                                        (menubar1->style->font_desc,
       //                                       "FreeSans 8");
-                                            
+
 //  gtk_style_attach (stl, tea_main_window);
 
   return tea_main_window;
