@@ -2,7 +2,7 @@
                           tea_about.c  -  description
                              -------------------
     begin                : august 2004
-    copyright            : (C) 2004, 2005 by Peter 'Roxton' Semiletov
+    copyright            : (C) 2004-2006 by Peter 'Roxton' Semiletov
     email                : peter.semiletov@gmail.com
  **************************************************************************/
 
@@ -28,10 +28,14 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
+#include "tea_text_document.h" // Needed for log_to_memo
 #include "callbacks.h"
 #include "interface.h"
 #include "tea_defs.h"
 #include "tea_gtk_utils.h"
+#include "tea_funx.h" // Needed for get_tea_doc_compose_name
+#include "rox_strings.h" // Needed for get_gchar_array_size
+#include "tea_about.h"
 
 
 static gboolean cb_wnd_destroy_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
@@ -61,14 +65,14 @@ GtkWidget* create_wnd_about (void)
   GtkWidget *bt_close;
 
   gchar *f = get_tea_doc_compose_name ("AUTHORS");
-  gchar *s = str_file_read (f);  
+  gchar *s = str_file_read (f);
 
   if (! s)
-    {
-     log_to_memo (_("%s is missing! I cannot show you the About window :("), f, LM_ERROR);
-     g_free (f);
-     return NULL;  
-    } 
+     {
+      log_to_memo (_("%s is missing! I cannot show you the About window :("), f, LM_ERROR);
+      g_free (f);
+      return NULL;
+     }
 
   gchar **ad;
   if (s)
@@ -104,7 +108,7 @@ GtkWidget* create_wnd_about (void)
   gtk_container_add (GTK_CONTAINER (scrolledwindow1), tv_code);
   gtk_text_view_set_editable (GTK_TEXT_VIEW (tv_code), FALSE);
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (tv_code), GTK_WRAP_WORD);
- 
+
   l_code = gtk_label_new (_("Code"));
   gtk_widget_show (l_code);
   gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), l_code);
@@ -154,10 +158,10 @@ GtkWidget* create_wnd_about (void)
   gtk_widget_show (l_rpm);
   gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 3), l_rpm);
 
-  bt_close = gtk_button_new_from_stock (GTK_STOCK_CLOSE); 
+  bt_close = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
   gtk_widget_show (bt_close);
   gtk_box_pack_start (GTK_BOX (vbox1), bt_close, FALSE, FALSE, UI_PACKFACTOR);
-  
+
   GTK_WIDGET_SET_FLAGS (bt_close, GTK_CAN_DEFAULT);
 
   g_signal_connect (bt_close, "clicked", G_CALLBACK (cb_on_bt_close), wnd_about);
@@ -168,7 +172,6 @@ GtkWidget* create_wnd_about (void)
   gtk_window_set_modal (wnd_about, TRUE);
 
   if (get_gchar_array_size (ad) == 4)
-  //2.6 - if (g_strv_length (ad) == 4)
      {
       gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (tv_code)), g_strchug (ad[0]), -1);
       gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (tv_rpms)), g_strchug (ad[2]), -1);
@@ -181,7 +184,7 @@ GtkWidget* create_wnd_about (void)
   g_signal_connect ((gpointer) wnd_about, "destroy",
                     G_CALLBACK (cb_wnd_destroy_event),
                     NULL);
-    
+
   g_strfreev (ad);
 
   g_signal_connect (G_OBJECT (wnd_about), "key_press_event", G_CALLBACK (win_key_handler), wnd_about);
