@@ -2,7 +2,7 @@
                           tea_calendar.c  -  description
                              -------------------
     begin                : 04.02.2005
-    copyright            : (C) 2005 by Peter 'Roxton' Semiletov
+    copyright            : (C) 2005-2006 by Peter 'Roxton' Semiletov
     email                : peter.semiletov@gmail.com
  ***************************************************************************/
 
@@ -34,18 +34,18 @@
 
 #include "tea_defs.h"
 #include "tea_gtk_utils.h"
+#include "tea_text_document.h" // Needs to be above for t_note_page struct
 #include "tea_config.h"
 #include "tea_calendar.h"
-#include "tea_text_document.h"
 
 
-static GDate *date1;      
+static GDate *date1;
 static GDate *date2;
 
 
 void calendar_init (void)
 {
-  date1 = NULL;      
+  date1 = NULL;
   date2 = NULL;
   wnd_calendar = NULL;
 }
@@ -70,15 +70,15 @@ static void cb_on_dateinsert (GtkButton *button,
   GDate *dt;
 
   gtk_calendar_get_date (GTK_CALENDAR (user_data), &year, &month, &day);
- 
+
   dt = g_date_new_dmy (day, month, year);
-  
+
   g_date_strftime (d1, sizeof (d1), "%x", dt);
 
   if (get_page_text ())
      doc_insert_at_cursor (cur_text_doc, d1);
 
-  g_date_free (dt); 
+  g_date_free (dt);
 }
 
 
@@ -88,17 +88,17 @@ static void cb_on_insert_julian (GtkButton *button,
   guint year;
   guint month;
   guint day;
-     
+
   gtk_calendar_get_date (GTK_CALENDAR (user_data), &year, &month, &day);
- 
+
   GDate *dt = g_date_new_dmy (day, month, year);
   gchar *d = g_strdup_printf ("%d", g_date_get_julian (dt));
 
   if (get_page_text())
      doc_insert_at_cursor (cur_text_doc, d);
-  
+
   g_free (d);
-  g_date_free (dt); 
+  g_date_free (dt);
 }
 
 
@@ -108,17 +108,17 @@ static void cb_on_insert_day_of_year (GtkButton *button,
   guint year;
   guint month;
   guint day;
-     
+
   gtk_calendar_get_date (GTK_CALENDAR (user_data), &year, &month, &day);
- 
+
   GDate *dt = g_date_new_dmy (day, month, year);
   gchar *d = g_strdup_printf ("%d", g_date_get_day_of_year (dt));
 
   if (get_page_text())
      doc_insert_at_cursor (cur_text_doc, d);
-  
+
   g_free (d);
-  g_date_free (dt); 
+  g_date_free (dt);
 }
 
 
@@ -158,7 +158,7 @@ static void cb_on_calc_button_click (GtkButton *button,
 
   gchar d1[256];
   gchar d2[256];
-  
+
   g_date_strftime (d1, sizeof (d1), "%x", date1);
   g_date_strftime (d2, sizeof (d2), "%x", date2);
 
@@ -182,9 +182,9 @@ GtkWidget* make_calendar_window (void)
   GtkWidget *wnd = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
   g_signal_connect (GTK_OBJECT (wnd), "destroy",
-                    GTK_SIGNAL_FUNC (destroy_event), wnd); 
+                    GTK_SIGNAL_FUNC (destroy_event), wnd);
 
-  gtk_window_set_position (wnd, GTK_WIN_POS_CENTER);  
+  gtk_window_set_position (wnd, GTK_WIN_POS_CENTER);
 
   gtk_window_set_title (GTK_WINDOW (wnd), _("Calendar"));
 
@@ -200,7 +200,7 @@ GtkWidget* make_calendar_window (void)
   gtk_widget_show (hbox1);
   gtk_container_add (GTK_CONTAINER (fr_diff), hbox1);
 
-  GtkWidget *cal = gtk_calendar_new ();        
+  GtkWidget *cal = gtk_calendar_new ();
   gtk_widget_show (cal);
 
   GtkWidget *bt_date1 = gtk_button_new_with_label (_("date1"));
@@ -240,4 +240,3 @@ GtkWidget* make_calendar_window (void)
   gtk_widget_show (wnd);
   return wnd;
 }
- 
\ No newline at end of file
