@@ -28,7 +28,7 @@ typedef struct
   gchar *cm_dvi_view;
   gchar *cm_pdf_view;
   gchar *cm_ps_view;
-  gchar *cm_run_latex;  
+  gchar *cm_run_latex;
 
   gchar *dir_thumbs;
   gchar *date_time;
@@ -39,15 +39,15 @@ typedef struct
 
   gboolean add_def_file_ext;
   gchar *def_file_ext;
-  
+
 
   //confirmations
-  gboolean prompt_on_not_saved; 
-  gboolean prompt_on_file_exists; 
+  gboolean prompt_on_not_saved;
+  gboolean prompt_on_file_exists;
   //paths
 
-  gboolean common_closing_button_visible; 
-
+  gboolean common_closing_button_visible;
+  gboolean show_line_markers;
 
   gchar *ext_pic_editor;
   gchar *ext_pic_viewer;
@@ -55,19 +55,18 @@ typedef struct
   gchar *tea_main_dir;
   gchar *templates_dir;
   gchar *snippets_dir;
+  gchar *scripts_dir;
 
   gchar *crapbook_file;
   gchar *tmp_dir;
   gchar *tmp_file;
-  //gchar *user_menu_file;
   gchar *ext_programs;
-  //gchar *file_gtkrc;
 
   gboolean homeend;
 
   gchar *current_movie;
   gchar *cm_mplayer;
-  
+
   gchar *bmx_file;
   gchar *sessions_dir;
   gchar *tea_rc;
@@ -100,8 +99,7 @@ typedef struct
   gchar *cmd_def_doc_browser;
 
   gint logmemo_lines_max;
-  //gint famous_history_max;
-  
+
   gint saveopen_w;
   gint saveopen_h;
 
@@ -120,7 +118,7 @@ typedef struct
 
   gchar *tag_color_paint_fg;
   gchar *tag_color_paint_bg;
-  
+
   gchar *color_cursor;
 
   gchar *tag_comment;
@@ -159,14 +157,16 @@ typedef struct
   gboolean fullscreen;
 
   gboolean do_save_all_on_termination;
+  gboolean show_cursor_pos;
 
 
   gint autosave_interval; //minutes
   gint screen_h;
   gint screen_w;
 
+  gint max_undo_levels;
+
   gboolean start_with_blank_file;
-  //gboolean start_with_recent_file;
 
   gboolean use_def_save_dir;
   gboolean use_def_open_dir;
@@ -176,10 +176,9 @@ typedef struct
   gboolean det_charset_by_meta;
 
 
-  gboolean use_snippets;
+  //gboolean use_snippets;
   gboolean use_ext_image_viewer;
 
-
   gboolean do_autosave;
   gboolean do_hl_on_fileopen;
   gboolean do_backup;
@@ -210,7 +209,7 @@ typedef struct
   gboolean use_def_doc_browser;
  //---------------
 
-  gboolean enc_det_cp1251; 
+  gboolean enc_det_cp1251;
   gboolean enc_det_koi8r;
   gboolean enc_det_koi8u;
   gboolean enc_det_866;
@@ -222,16 +221,16 @@ typedef struct
   gboolean enc_det_portuguese;
   gboolean enc_det_slovak;
   gboolean enc_det_slovenian;
-  gboolean enc_det_spanish;  
-  gboolean enc_det_turkish;  
-  gboolean enc_det_czech;  
+  gboolean enc_det_spanish;
+  gboolean enc_det_turkish;
+  gboolean enc_det_czech;
 
   gchar *rtf_enc;
 
   GList *gl_save_charsets;
   GList *iconv_encs;
   gint msg_counter;
-  
+
 } tea_confile;
 
 
@@ -246,7 +245,9 @@ typedef struct
   gpointer b;
   gboolean c;
 } t_ppair;
- 
+
+
+
 
 gchar* enc_guess_value;
 tea_confile confile; //global config
@@ -262,6 +263,7 @@ GList *gl_supported_hlangs;
 GList *gl_notetab_positions;
 GList *bookmarks;
 GList *recent_list;
+GList *gl_scripts;
 GList *gl_tea_kwas_bookmarks;
 
 GList *gl_autosave;
@@ -275,6 +277,8 @@ GtkTooltips* tooltips;
 GList *gl_found_files;
 
 gboolean b_exit_flag;
+gboolean ui_done_flag;
+
 
 gint def_mm;
 gint g_argc;
@@ -283,29 +287,39 @@ char **g_argv;
 void confile_reload (void);
 void confile_free (void);
 
-GList* parse_conf_file_to_list (gchar *filename);
-gchar* conf_get_char_value (GList *confdata, gchar *key, gchar *def);
-gint conf_get_int_value (GList *confdata, gchar *key, gint def);
+//GList* parse_conf_file_to_list (const gchar *filename);
+gchar* conf_get_char_value (GList *confdata, const gchar *key, const gchar *def);
+gint conf_get_int_value (GList *confdata, const gchar *key, gint def);
+GtkWidget* lookup_widget (GtkContainer *widget, const gchar *widget_name);
 
 void ui_init (void);
 void ui_done (void);
 
 void get_iconv_sup (void);
 
-void execute_recent_item (gchar *item);
+void execute_recent_item (const gchar *item);
 
 void update_enc_menu (void);
 void reload_snippets (void);
-GtkWidget* lookup_widget (GtkContainer *widget, gchar *widget_name);
 
 void  hash_remove_cb (gpointer data);
-gchar* get_tmp_file_string (void);
 
 void reload_usermenu (void);
 void reload_sessions (void);
 void reload_autoreplace (void);
 void reload_autosave (void);
-gchar* get_tea_config_dir (void);
-gchar* get_tea_config_file (gchar *f);
-void write_gtkrc (void);
 void reload_ext_programs (void);
+void reload_scripts (void);
+void put_tmp_file_string (const gchar *s);
+gchar* get_tmp_file_string (void);
+gchar* ht_get_char_value (const gchar *key, const gchar *def);
+gint ht_get_int_value (const gchar *key, gint def);
+void tea_start (void);
+void bmx_reload (void);
+void reload_dicts (void);
+void reload_templates (void);
+void reload_hotkeys (void);
+void update_recent_list_menu (gboolean);
+void tea_done (void);
+void fill_entities (void);
+void tabs_reload (void);
