@@ -2,13 +2,13 @@
                           tea_enc.c  -  description
                              -------------------
     begin                : Oct 1 2004
-    copyright            : (C) 2004, 2005 by Peter 'Roxton' Semiletov
+    copyright            : (C) 2004-2006 by Peter 'Roxton' Semiletov
     email                : peter.semiletov@gmail.com
 ***************************************************************************/
 
 /*
-Tarot Osuji <tarot@sdf.lonestar.org> //Leafcode editor, 
-http://tarot.freeshell.org/leafpad/ 
+Tarot Osuji <tarot@sdf.lonestar.org> //Leafcode editor,
+http://tarot.freeshell.org/leafpad/
 */
 
 /***************************************************************************
@@ -25,7 +25,10 @@ http://tarot.freeshell.org/leafpad/
 #endif
 
 #include <gtk/gtk.h>
+#include <string.h>
 
+
+#include "tea_text_document.h"
 #include "tea_config.h"
 #include "rox_strings.h"
 
@@ -48,7 +51,7 @@ gboolean enc_is_finnish (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -73,12 +76,12 @@ gboolean enc_is_koi8r (const gchar *s)
                  };
 
   gint i;
-  
+
   for (i = 0; i <= 14; i++)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -91,7 +94,7 @@ gboolean enc_is_koi8u (const gchar *s)
                   "  ",
                   " Φ ",
                   " צ ",
-                  "  ",  
+                  "  ",
                   "  ",
                   "  ",
                   " ͦ ",
@@ -103,12 +106,12 @@ gboolean enc_is_koi8u (const gchar *s)
                   };
 
   gint i;
-  
+
   for (i = 0; i <= 14; i++)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -137,13 +140,14 @@ gboolean enc_is_cp1251 (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
 gboolean enc_is_dos866 (const gchar *s)
 {
-  gchar *a[14] = {  
+  gchar *a[15] = {
+                  "  ",
                   "  ",
                   "  ",
                   "  ",
@@ -152,7 +156,6 @@ gboolean enc_is_dos866 (const gchar *s)
                   "  ",
                   "  ",
                   "  ",
-                  "  ",  
                   "  ",
                   "  ",
                   "  ",
@@ -166,7 +169,7 @@ gboolean enc_is_dos866 (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -187,7 +190,7 @@ gboolean enc_is_serbian (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -209,7 +212,7 @@ gboolean enc_is_german (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -230,7 +233,7 @@ gboolean enc_is_polish (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -250,7 +253,7 @@ gboolean enc_is_latvian (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -270,7 +273,7 @@ gboolean enc_is_slovak (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -290,7 +293,7 @@ gboolean enc_is_slovenian (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -310,7 +313,7 @@ gboolean enc_is_spanish (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -330,7 +333,7 @@ gboolean enc_is_czech (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -348,7 +351,7 @@ gboolean enc_is_turkish (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -366,12 +369,12 @@ gboolean enc_is_portuguese (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
 //JIS X 0208-1990
-gboolean enc_is_iso_2022_jp (const gchar *s) 
+gboolean enc_is_iso_2022_jp (const gchar *s)
 {
   gchar *a[4] = {
                  "\x1b\x24\x40",
@@ -385,7 +388,7 @@ gboolean enc_is_iso_2022_jp (const gchar *s)
       if (strstr (s, a[i]))
          return TRUE;
 
-  return FALSE; 
+  return FALSE;
 }
 
 
@@ -403,98 +406,124 @@ Big5 = Chinese Traditional
 
 //Leafpad code by Tarot Osuji <tarot@sdf.lonestar.org>
 //modified by roxton
-gchar *detect_charset_zh(const gchar *text)
+gchar* detect_charset_zh (const gchar *text)
 {
-	guint8 c = *text;
-	gchar *charset = g_strdup ("GB2312");
-	
-	while ((c = *text++) != '\0') {
-		if (c >= 0x81 && c <= 0x87) {
-			charset = ch_str (charset, "GB18030");
-			break;
-		}
-		else if (c >= 0x88 && c <= 0xA0) {
-			c = *text++;
-			if ((c >= 0x30 && c <= 0x39) || (c >= 0x80 && c <= 0xA0)) {
-				charset = ch_str (charset, "GB18030");
-				break;
-			} //else GBK/Big5-HKSCS cannot determine
-		}
-		else if ((c >= 0xA1 && c <= 0xC6) || (c >= 0xC9 && c <= 0xF9)) {
-			c = *text++;
-			if (c >= 0x40 && c <= 0x7E)
-				charset = ch_str ("Big5", charset);
-			else if ((c >= 0x30 && c <= 0x39) || (c >= 0x80 && c <= 0xA0)) {
-				charset = ch_str (charset, "GB18030");
-				break;
-			}
-		}
-		else if (c >= 0xC7) {
-			c = *text++;
-			if ((c >= 0x30 && c <= 0x39) || (c >= 0x80 && c <= 0xA0)) {
-				charset = ch_str (charset, "GB18030");
-				break;
-			}
-		}
-	}
-	
-	return charset;
+  guint8 c = *text;
+  gchar *charset = g_strdup ("GB2312");
+
+  while ((c = *text++) != '\0')
+       {
+        if (c >= 0x81 && c <= 0x87)
+           {
+            charset = ch_str (charset, "GB18030");
+            break;
+            }
+        else
+            if (c >= 0x88 && c <= 0xA0)
+               {
+                c = *text++;
+                if ((c >= 0x30 && c <= 0x39) || (c >= 0x80 && c <= 0xA0))
+                   {
+                    charset = ch_str (charset, "GB18030");
+                    break;
+                    } //else GBK/Big5-HKSCS cannot determine
+               }
+             else
+                 if ((c >= 0xA1 && c <= 0xC6) || (c >= 0xC9 && c <= 0xF9))
+                    {
+                     c = *text++;
+                     if (c >= 0x40 && c <= 0x7E)
+                        charset = ch_str ("Big5", charset);
+                     else
+                         if ((c >= 0x30 && c <= 0x39) || (c >= 0x80 && c <= 0xA0))
+                            {
+                             charset = ch_str (charset, "GB18030");
+                             break;
+                            }
+                    }
+                  else
+                      if (c >= 0xC7)
+                         {
+                          c = *text++;
+                          if ((c >= 0x30 && c <= 0x39) || (c >= 0x80 && c <= 0xA0))
+                             {
+                              charset = ch_str (charset, "GB18030");
+                              break;
+                             }
+                         }
+              }
+
+  return charset;
 }
 
-gchar *detect_charset_ja(const gchar *text)
+
+gchar *detect_charset_ja (const gchar *text)
 {
-   if (enc_is_iso_2022_jp (text))
-      return g_strdup ("ISO-2022-JP-2");
+  if (enc_is_iso_2022_jp (text))
+     return g_strdup ("ISO-2022-JP-2");
 
-   if ( g_utf8_validate (text, -1, 0) )
-      return g_strdup ("UTF-8");
+  if ( g_utf8_validate (text, -1, 0) )
+     return g_strdup ("UTF-8");
 
-	guint8 c = *text;
-	gchar *charset = NULL;
-	
-	while (charset == NULL && (c = *text++) != '\0') {
-		if (c >= 0x81 && c <= 0x9F) {
-			if (c == 0x8E) /* SS2 */ {
-				c = *text++;
-				if ((c >= 0x40 && c <= 0xA0) || (c >= 0xE0 && c <= 0xFC))
-					charset = ch_str (charset, "Shift_JIS");
-			}
-			else if (c == 0x8F) /* SS3 */ {
-				c = *text++;
-				if (c >= 0x40 && c <= 0xA0)
-					charset = ch_str (charset, "Shift_JIS");
-				else if (c >= 0xFD)
-					break;
-			}
-			else
-				charset = ch_str (charset, "Shift_JIS");
-		}
-		else if (c >= 0xA1 && c <= 0xDF) {
-			c = *text++;
-			if (c <= 0x9F)
-				charset = ch_str (charset, "Shift_JIS");
-			else if (c >= 0xFD)
-				break;
-		}
-		else if (c >= 0xE0 && c <= 0xEF) {
-			c = *text++;
-			if (c >= 0x40 && c <= 0xA0)
-				charset = ch_str (charset, "Shift_JIS");
-			else if (c >= 0xFD)
-				break;
-		}
-		else if (c >= 0xF0)
-			break;
-	}
-	
-	if (! charset)
-		charset = ch_str (charset, "EUC-JP");
-	
-	return charset;
+  guint8 c = *text;
+  gchar *charset = NULL;
+
+  while (charset == NULL && (c = *text++) != '\0')
+        {
+         if (c >= 0x81 && c <= 0x9F)
+            {
+             if (c == 0x8E) /* SS2 */
+                {
+                 c = *text++;
+                 if ((c >= 0x40 && c <= 0xA0) || (c >= 0xE0 && c <= 0xFC))
+                    charset = ch_str (charset, "Shift_JIS");
+                }
+            else
+                if (c == 0x8F) /* SS3 */
+                   {
+                    c = *text++;
+                    if (c >= 0x40 && c <= 0xA0)
+                       charset = ch_str (charset, "Shift_JIS");
+                    else
+                        if (c >= 0xFD)
+                           break;
+                   }
+                else
+                    charset = ch_str (charset, "Shift_JIS");
+             }
+         else
+             if (c >= 0xA1 && c <= 0xDF)
+                {
+                 c = *text++;
+                 if (c <= 0x9F)
+                    charset = ch_str (charset, "Shift_JIS");
+                 else
+                     if (c >= 0xFD)
+                        break;
+                }
+             else
+                 if (c >= 0xE0 && c <= 0xEF)
+                    {
+                     c = *text++;
+                     if (c >= 0x40 && c <= 0xA0)
+                        charset = ch_str (charset, "Shift_JIS");
+                     else
+                         if (c >= 0xFD)
+                            break;
+                     }
+                 else
+                     if (c >= 0xF0)
+                        break;
+        }
+
+  if (! charset)
+     charset = ch_str (charset, "EUC-JP");
+
+  return charset;
 }
-
 //end of the code from Leafpad
 
+
 GList* create_enc_list (void)
 {
   GList *l = NULL;
@@ -1477,4 +1506,4 @@ l = g_list_prepend (l, "500V1");
 l = g_list_prepend (l, "500");
 l = g_list_prepend (l, "437");
   return l;
-}
\ No newline at end of file
+}
