@@ -2,13 +2,12 @@
                           tea_funx.c  -  description
                              -------------------
     begin                : Mon Dec 12 2003
-    copyright            : (C) 2003, 2004, 2005 by Peter 'Roxton' Semiletov
+    copyright            : (C) 2003-2006 by Peter 'Roxton' Semiletov
     email                : peter.semiletov@gmail.com
  ***************************************************************************/
 
 /*
 Copyright (C) 2002 Paolo Maggi
-Copyright (C) 2000 Olivier Sessink
 Copyright (C) 2000  Kh. Naba Kumar Singh
 Copyright (C) 2005 BMPx development team.
 */
@@ -33,23 +32,32 @@ Copyright (C) 2005 BMPx development team.
 #include <gdk/gdkkeysyms.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <glib/gi18n.h>
+#include <stdlib.h> // Needed for strtol
+#include <limits.h> // Needed for strtol
 #include <errno.h>
 #include <sys/stat.h>
 #include <time.h>
 #include <stdio.h>
+#include <string.h>
 
 
 #include "tea_defs.h"
+#include "tea_text_document.h" // Needs to be above for t_note_page struct
 #include "tea_config.h"
+#include "image_browser.h"
+
 #include "rox_strings.h"
 #include "interface.h"
-#include "tea_text_document.h"
 #include "tea_funx.h"
 #include "tea_proj.h"
 #include "tea_gtk_utils.h"
 #include "callbacks.h"
 
 
+/* To avoid implicit declaration warning */
+void make_stats (t_note_page *doc);
+
+
 /* from:
  * time.c
  * This file is part of gedit
@@ -59,7 +67,7 @@ Copyright (C) 2005 BMPx development team.
   changed by roxton
 */
 
-gchar* get_time (const gchar* format)
+gchar* get_time (const gchar *format)
 {
    if (! format)
       return NULL;
@@ -72,12 +80,12 @@ gchar* get_time (const gchar* format)
   gchar *locale_format;
 
   if (strlen (format) == 0)
-     return g_strdup (" ");
+     return NULL;
 
   locale_format = g_locale_from_utf8 (format, -1, NULL, NULL, NULL);
 
   if (! locale_format)
-     return g_strdup (" ");
+     return NULL;
 
   clock = time (NULL);
   now = localtime (&clock);
@@ -97,8 +105,6 @@ gchar* get_time (const gchar* format)
       {
        out_utf8 = g_locale_to_utf8 (out, -1, NULL, NULL, NULL);
        g_free (out);
-       if (! out_utf8)
-          out_utf8 = g_strdup (" ");
       }
 
   return out_utf8;
@@ -109,161 +115,63 @@ gchar* get_time (const gchar* format)
 //    utilities.c
 //  Copyright (C) 2000  Kh. Naba Kumar Singh
 #define FILE_BUFFER_SIZE 65535
-gboolean
-copy_file (gchar * src, gchar * dest)
+gboolean copy_file (const gchar *src, const gchar *dest)
 {
-	FILE *input_fp, *output_fp;
-	gchar buffer[FILE_BUFFER_SIZE];
-	gint bytes_read, bytes_written;
-	gboolean error;
+  FILE *input_fp, *output_fp;
+  gchar buffer[FILE_BUFFER_SIZE];
+  gint bytes_read, bytes_written;
+  gboolean error = TRUE;
 
-	error = TRUE;
+  input_fp = fopen (src, "rb");
+  if (! input_fp)
+     return FALSE;
 
-	input_fp = fopen (src, "rb");
-	if (input_fp == NULL)
-	{
-		return FALSE;
-	}
+  output_fp = fopen (dest, "wb");
+  if (! output_fp)
+     {
+      fclose (input_fp);
+      return TRUE;
+     }
 
-	output_fp = fopen (dest, "wb");
-	if (output_fp == NULL)
-	{
-		fclose (input_fp);
-		return TRUE;
-	}
+  for (;;)
+      {
+       bytes_read = fread (buffer, 1, FILE_BUFFER_SIZE, input_fp);
+       if (bytes_read != FILE_BUFFER_SIZE && ferror (input_fp))
+          {
+           error = FALSE;
+           break;
+          }
 
-	for (;;)
-	{
-		bytes_read = fread (buffer, 1, FILE_BUFFER_SIZE, input_fp);
-		if (bytes_read != FILE_BUFFER_SIZE && ferror (input_fp))
-		{
-			error = FALSE;
-			break;
-		}
+       if (bytes_read)
+         {
+          bytes_written = fwrite (buffer, 1, bytes_read, output_fp);
+          if (bytes_read != bytes_written)
+            {
+             error = FALSE;
+             break;
+            }
+         }
 
-		if (bytes_read)
-		{
-			bytes_written = fwrite (buffer, 1, bytes_read, output_fp);
-			if (bytes_read != bytes_written)
-			{
-				error = FALSE;
-				break;
-			}
-		}
+       if (bytes_read != FILE_BUFFER_SIZE && feof (input_fp))
+          break;
+       }
 
-		if (bytes_read != FILE_BUFFER_SIZE && feof (input_fp))
-		{
-			break;
-		}
-	}
+  fclose (input_fp);
+  fclose (output_fp);
 
-	fclose (input_fp);
-	fclose (output_fp);
-
-	return error;
+  return error;
 }
 //
 
 
-//from Bluefish:: bf_lib.c and gtk_easy
-/**
- * ending_slash:
- * @dirname: a #const gchar * with a diretory name
- *
- * makes sure the last character of the newly allocated
- * string it returns is a '/'
- *
- * Return value: a newly allocated gchar * dirname that does end on a '/'
- **/
-gchar *ending_slash(const gchar *dirname) {
-	if (!dirname) {
-		return g_strdup("");
-	}
-
-	if (dirname[strlen(dirname)-1] == G_DIR_SEPARATOR) {
-		return g_strdup(dirname);
-	} else {
-		return g_strconcat(dirname, G_DIR_SEPARATOR_S, NULL);
-	}
-}
-
-
-/**
- * most_efficient_filename:
- * @filename: a gchar * with a possibly inefficient filename like /hello/../tmp/../myfile
- *
- * tries to eliminate any dir/../ combinations in filename
- * this function could do evern better, it should also remove /./ entries
- *
- * Return value: the same gchar * as passed to the function
- **/
-gchar *most_efficient_filename(gchar *filename) {
-	gint i,j, len;
-
-	len = strlen(filename);
-	for (i=0; i < len-4; i++) {
-		if (strncmp(&filename[i], "/../", 4) == 0) {
-			for (j=1; j < i; j++) {
-				if ((filename[i - j] == G_DIR_SEPARATOR) || (i==j)) {
-					memmove(&filename[i-j], &filename[i+3], len-i);
-					j=i;
-					i--;
-				}
-			}
-		}
-	}
-	return filename;
-}
-
-
-/**
- * create_full_path:
- * @filename: a gchar * with the (relative or not) filename
- * @basedir: a gchar * with a basedir or NULL for current dir
- *
- * if filename is already absolute, it returns it
- * else it will use basedir if available, else the current dir
- * to add to the filename to form the full path
- *
- * it does use most_efficient_filename() to remote unwanted dir/../ entries
- *
- * Return value: a newly allocated gchar * with the full path
- **/
-
-
-gchar *create_full_path(gchar * filename, gchar *basedir) {
-	gchar *absolute_filename;
-	gchar *tmpcdir;
-
-
-	if (g_path_is_absolute(filename)) {
-		absolute_filename = g_strdup(filename);
-	} else {
-		if (basedir) {
-			tmpcdir = ending_slash(basedir);
-		} else {
-			gchar *curdir = g_get_current_dir();
-			tmpcdir = ending_slash(curdir);
-			g_free(curdir);
-		}
-		absolute_filename = g_strconcat(tmpcdir, filename, NULL);
-		g_free(tmpcdir);
-	}
-	absolute_filename = most_efficient_filename(absolute_filename);
-	return absolute_filename;
-}
-//
-
-
-
 static gint noname_name_counter = -1;
 
-gchar* get_noname_name (void) 
+gchar* get_noname_name (void)
 {
   ++noname_name_counter;
   if (noname_name_counter >= G_MAXINT)
      noname_name_counter = 0;
-  return g_strdup_printf ("noname_%d", noname_name_counter);
+  return g_strdup_printf (_("noname_%d"), noname_name_counter);
 }
 
 
@@ -279,24 +187,24 @@ int get_percent (int total, int value)
 }
 
 
-void handle_file (gchar *filename, gint mode, gboolean kwas)
+void handle_file (const gchar *filename, gint mode, gboolean kwas)
 {
   if (! filename)
      return;
- 
+
   gchar *cmd = NULL;
-  
+
   gint i = get_n_page_by_filename (filename);
 
   if (i != -1)
      {
       gtk_notebook_set_current_page (notebook1, i);
       return;
-     } 
+     }
 
   if (! g_file_test (filename, G_FILE_TEST_EXISTS))
      return;
- 
+
   if (is_image (filename))
      {
       if (mode != 0)
@@ -312,10 +220,10 @@ void handle_file (gchar *filename, gint mode, gboolean kwas)
           cmd = g_strdup_printf (confile.ext_pic_editor, filename);
           system (cmd);
           g_free(cmd);
-          return;        
+          return;
          }
-      else  
-          create_wnd_imgviewer (filename);
+      else
+          image_viewer_create (filename);
 
       return;
      }
@@ -332,29 +240,29 @@ void handle_file (gchar *filename, gint mode, gboolean kwas)
 }
 
 
-void handle_file_enc (gchar *filename, gchar *enc)
+void handle_file_enc (const gchar *filename, const gchar *enc)
 {
   if (! filename)
      return;
- 
+
   gchar *cmd = NULL;
-  
+
   gint i = get_n_page_by_filename (filename);
 
   if (i != -1)
      {
       gtk_notebook_set_current_page (notebook1, i);
       return;
-     } 
+     }
 
   if (! g_file_test (filename, G_FILE_TEST_EXISTS))
      return;
-  
+
   cur_settings.selected_enc = ch_str (cur_settings.selected_enc, enc);
 
   open_file_std (filename);
 }
-      
+
 /*
 gboolean is_image2 (const gchar *filename)
 {
@@ -370,21 +278,21 @@ gboolean is_tex (const gchar *filename)
 
 gboolean is_po (const gchar *f)
 {
-  return is_ext (f, ".pot", ".po", NULL); 
+  return is_ext (f, ".pot", ".po", NULL);
 }
 
 
-void create_empty_file (gchar *filename, gchar *first_line)
+void create_empty_file (const gchar *filename, const gchar *first_line)
 {
   FILE *out = fopen (filename, "w");
-  
-  if (! out) 
-     return FALSE;                     
+
+  if (! out)
+     return;
 
   if (first_line)
      fprintf (out, "%s", first_line);
 
-  fclose (out);                        
+  fclose (out);
 }
 
 
@@ -396,24 +304,24 @@ gint get_file_size (const gchar *filename)
 }
 
 
-gboolean check_ext_wo_dot (gchar *filename, gchar *ext)
+//check extension without dot
+gboolean check_ext_wo_dot (const gchar *filename, const gchar *ext)
 {
   if (! filename || ! ext)
      return FALSE;
 
   gboolean r = FALSE;
   gchar *f = g_utf8_strdown (filename, -1);
-
   gchar *full_ext = g_strconcat (".", ext, NULL);
 
   r = g_str_has_suffix (f, full_ext);
   g_free (full_ext);
   g_free (f);
-  return r; 
+  return r;
 }
 
 
-gboolean check_ext (gchar *filename, gchar *ext)
+gboolean check_ext (const gchar *filename, const gchar *ext)
 {
   if (! filename || ! ext)
      return FALSE;
@@ -422,13 +330,13 @@ gboolean check_ext (gchar *filename, gchar *ext)
   gchar *f = g_utf8_strdown (filename, -1);
   r = g_str_has_suffix (f, ext);
   g_free (f);
-  return r; 
+  return r;
 }
 
 
 gboolean is_markup (const gchar *f)
 {
-  return is_ext (f, ".html", ".htm", ".shtml", ".xml", ".sgm", ".xhtml", ".docbook", NULL);
+  return is_ext (f, ".html", ".htm", ".xml", ".xhtml", ".sgm", ".shtml", ".sgml", ".docbook", NULL);
 }
 
 
@@ -444,47 +352,50 @@ gboolean is_pascal (const gchar *f)
 }
 
 
-gchar* get_hl_name (gchar *file_name)
+gchar* get_hl_name (const gchar *file_name)
 {
+  if (! file_name)
+     return NULL;
+
   gchar *b;
 
   if (is_markup (file_name))
-     return g_strdup (HL_MARKUP); 
-       
+     return g_strdup (HL_MARKUP);
+
   if (is_c (file_name))
-     return g_strdup (HL_C); 
- 
+     return g_strdup (HL_C);
+
   if (is_pascal (file_name))
-     return g_strdup (HL_PASCAL); 
+     return g_strdup (HL_PASCAL);
 
-  if (check_ext (file_name, ".py")) 
-     return g_strdup (HL_PYTHON); 
- 
-  if (check_ext (file_name, ".php")) 
-     return g_strdup (HL_PHP); 
+  if (check_ext (file_name, ".py"))
+     return g_strdup (HL_PYTHON);
 
-  if (check_ext (file_name, ".sh")) 
-     return g_strdup (HL_BASH); 
+  if (check_ext (file_name, ".php"))
+     return g_strdup (HL_PHP);
+
+  if (check_ext (file_name, ".sh"))
+     return g_strdup (HL_BASH);
 
   if (is_po (file_name))
-     return g_strdup (HL_PO); 
+     return g_strdup (HL_PO);
 
   if (is_tex (file_name))
-     return g_strdup (HL_TEX); 
+     return g_strdup (HL_TEX);
 
   if (confile.do_det_scripts_by_content)
      {
       b = str_file_read (file_name);
       if (b)
-         { 
-          if (strstr (b, "#!/bin/bash") || strstr (b, "#!/bin/sh"))
+         {
+          if ((strstr (b, "#!/bin/bash")) || (strstr (b, "#!/bin/sh")))
              {
               g_free (b);
-              return g_strdup (HL_BASH); 
+              return g_strdup (HL_BASH);
              }
           g_free (b);
          }
-     } 
+     }
 
   return g_strdup (HL_NONE);
 }
@@ -497,7 +408,7 @@ gchar* get_l_filename (const gchar *filename)
 
   gsize bytes_read;
   gsize bytes_written;
-  return g_filename_from_utf8 (filename, -1, &bytes_read, &bytes_written, NULL);  
+  return g_filename_from_utf8 (filename, -1, &bytes_read, &bytes_written, NULL);
 }
 
 
@@ -508,8 +419,8 @@ gchar* get_8_filename (const gchar *filename)
 
   gsize bytes_read;
   gsize bytes_written;
-  
-  return g_filename_to_utf8 (filename, -1, &bytes_read, &bytes_written, NULL);  
+
+  return g_filename_to_utf8 (filename, -1, &bytes_read, &bytes_written, NULL);
 }
 
 /*
@@ -530,26 +441,25 @@ void make_stats2 (t_note_page *doc)
      gtk_text_buffer_get_selection_bounds (doc->text_buffer, &start, &end);
   else
       {
-       gtk_text_buffer_get_start_iter (doc->text_buffer, &start); 
-       gtk_text_buffer_get_end_iter (doc->text_buffer, &end); 
+       gtk_text_buffer_get_bounds (doc->text_buffer, &start, &end);
        selected = FALSE;
-      }  
+      }
 
   lines = gtk_text_buffer_get_line_count (doc->text_buffer);
   text =  gtk_text_buffer_get_text (doc->text_buffer, &start, &end, FALSE);
   bytes = strlen (text);
 
-  do 
+  do
     {
      ch = gtk_text_iter_get_char (&start);
      if (g_unichar_isspace (ch))
         white_chars++;
      if (gtk_text_iter_starts_word (&start))
-        words++; 
+        words++;
 
      chars++;
     }
-  while ( (gtk_text_iter_forward_char (&start)) && ( ! gtk_text_iter_equal (&start, &end)) ); 
+  while ( (gtk_text_iter_forward_char (&start)) && ( ! gtk_text_iter_equal (&start, &end)) );
 
   gchar *s_bytes = g_strdup_printf(_("bytes: %d\n"), bytes);
   gchar *s_lines = g_strdup_printf(_("lines: %d\n"), lines);
@@ -621,7 +531,7 @@ void make_stats (t_note_page *doc)
   gint i;
   gchar *tmp_str;
 
-  selected = doc_is_sel (doc->text_buffer);
+  selected = doc_has_selection (doc);
 
   if (! selected)
       text = doc_get_buf (doc->text_buffer);
@@ -629,7 +539,7 @@ void make_stats (t_note_page *doc)
       text = doc_get_sel (doc);
 
   if (! text)
-     return; 
+     return;
 
   if (! g_utf8_validate (text, -1, NULL))
      return;
@@ -707,10 +617,10 @@ gchar* get_lang_name (void)
 {
   gchar *l = g_getenv ("LANG");
   if (! l)
-     return g_strdup ("UTF-8"); 
+     return g_strdup ("UTF-8");
 
   if (strcmp (l, "UTF-8") == 0)
-     return g_strdup ("UTF-8"); 
+     return g_strdup ("UTF-8");
 
   return g_strndup (g_getenv ("LANG"), 2);
 }
@@ -734,18 +644,18 @@ gchar* find_good_browser (void)
 }
 
 
-gchar* compose_browser_cmd (gchar *filename)
+gchar* compose_browser_cmd (const gchar *filename)
 {
   if (! filename)
      return NULL;
 
   gchar *t;
-  gchar *b; 
+  gchar *b;
 
   if (confile.use_def_doc_browser)
      {
       if (confile.cmd_def_doc_browser)
-         t = rep_all_s (confile.cmd_def_doc_browser, filename); 
+         t = rep_all_s (confile.cmd_def_doc_browser, filename);
       return t;
      }
 
@@ -761,7 +671,7 @@ gchar* compose_browser_cmd (gchar *filename)
 }
 
 
-gchar* get_tea_doc_compose_name (gchar *f)
+gchar* get_tea_doc_compose_name (const gchar *f)
 {
   return g_strconcat (TEA_DOC_DIR, f, NULL);
 }
@@ -770,7 +680,6 @@ gchar* get_tea_doc_compose_name (gchar *f)
 gchar* find_doc_index_name (void)
 {
   gchar *l = get_lang_name ();
-
   gchar *f = g_strconcat (TEA_DOC_DIR, l, G_DIR_SEPARATOR_S, "index.html", NULL);
 
   if (! g_file_test (f, G_FILE_TEST_EXISTS))
@@ -791,7 +700,7 @@ void run_doc_in_browser (void)
   gchar *f = find_doc_index_name ();
   gchar *t = compose_browser_cmd (f);
   if (t)
-     system (t); 
+     system (t);
 
   g_free (f);
   g_free (t);
@@ -802,19 +711,20 @@ gchar* get_clipboard_text (void)
 {
   GtkClipboard *c = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
   if (! c)
-     return NULL; 
+     return NULL;
 
   if (! gtk_clipboard_wait_is_text_available (c))
      return NULL;
 
-  return gtk_clipboard_wait_for_text (c); 
-}  
+  return gtk_clipboard_wait_for_text (c);
+}
 
 
-void insert_image (gchar *a_filename)
+void insert_image (const gchar *a_filename)
 {
-  if (! get_page_text()) return;
- 
+  if (! get_page_text ())
+     return;
+
   if (! cur_text_doc->b_saved)
      return;
 
@@ -830,47 +740,44 @@ void insert_image (gchar *a_filename)
 
   gchar *filename = create_relative_link (cur_text_doc->file_name_local, a_filename);
 
-
-  if (! filename) 
-     {
+  if (! filename)
       return;
-     }
 
-  gint mm = get_markup_mode ();       
+  gint mm = get_markup_mode ();
   gchar *t;
 
   gchar *temp = g_malloc (7777);
 
-  if (mm == MM_HTML) 
+  if (mm == MM_HTML)
       g_snprintf (temp,
                   7777,
                   "<img src=\"%s\" border=\"0\" alt=\"\" width=\"%d\" height=\"%d\">",
                   filename, width, height);
   else
-  if (mm == MM_XHTML) 
+  if (mm == MM_XHTML)
      g_snprintf (temp,
                  7777,
                  "<img src=\"%s\" border=\"0\" alt=\"\" width=\"%d\" height=\"%d\" />",
                  filename, width, height);
   else
-      if (mm == MM_DOCBOOK) 
+      if (mm == MM_DOCBOOK)
           g_snprintf (temp,
                       7777,
                       "<mediaobject><imageobject>\n<imagedata fileref=\"%s\"/>\n</imageobject></mediaobject>",
                       filename);
   else
-  if (mm == MM_WIKI) 
+  if (mm == MM_WIKI)
      g_snprintf (temp,
                  7777,
                  "[[Image:%s|thumb|alternative text]]",
                  filename);
   else
-  if (mm == MM_TEX) 
+  if (mm == MM_TEX)
      g_snprintf (temp,
                  7777,
                  "\\includegraphics{%s}",
                  filename);
-    
+
   doc_insert_at_cursor (cur_text_doc, temp);
 
   g_free (temp);
@@ -878,22 +785,20 @@ void insert_image (gchar *a_filename)
 }
 
 
-void insert_link (gchar *a_filename)
+void insert_link (const gchar *a_filename)
 {
-  if (! a_filename)
+  if (! a_filename || ! cur_text_doc)
      return;
 
   if (! g_file_test (a_filename, G_FILE_TEST_EXISTS))
      return;
-  
+
   gchar *filename = create_relative_link (cur_text_doc->file_name_local, a_filename);
- 
-  if (! filename) 
-     {
+
+  if (! filename)
       return;
-     }
-       
-  gchar *z =  g_strdup_printf ("<a href=\"%s\"></a>", filename); 
+
+  gchar *z =  g_strdup_printf ("<a href=\"%s\"></a>", filename);
 
   doc_insert_at_cursor (cur_text_doc, z);
   doc_move_cursor_backw_middle_tags (cur_text_doc);
@@ -903,62 +808,64 @@ void insert_link (gchar *a_filename)
 }
 
 
-
-
-void handle_file_ide (gchar *filename, gint line)
+void handle_file_ide (const gchar *filename, gint line)
 {
   if (! filename || ! cur_tea_project)
      return;
- 
-  gchar *cmd = g_strconcat (cur_tea_project->dir_source, filename, NULL);
- 
+
+  gchar *cmd = file_combine_path (cur_tea_project->dir_source, filename);
   gint i = get_n_page_by_filename (cmd);
 
   if (i != -1)
      {
       gtk_notebook_set_current_page (notebook1, i);
       if (get_page_text ())
-          doc_select_line (cur_text_doc, line); 
+          doc_select_line (cur_text_doc, line);
+      g_free (cmd);
       return;
-     } 
+     }
 
   if (! g_file_test (cmd, G_FILE_TEST_EXISTS))
-     return;
+     {
+      g_free (cmd);
+      return;
+     }
   else
       {
-       if (! get_page_text() )
+       if (! get_page_text ())
           cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "UTF-8");
        else
            cur_settings.selected_enc = ch_str (cur_settings.selected_enc, cur_text_doc->encoding);
 
        cur_text_doc = open_file_std (cmd);
        doc_select_line (cur_text_doc, line);
+       g_free (cmd);
       }
 }
 
 
 gboolean parse_error_line (const gchar *line, gchar **filename, gint *lineno)
 {
-   if (! line)
-       return FALSE;
+  if (! line)
+      return FALSE;
 
-   if (! strstr (line, ":"))
-       return FALSE;
+  if (! strstr (line, ":"))
+      return FALSE;
 
-   gchar **a = g_strsplit (line, ":", -1);
-   
-   if (a[0])
-      if (a[1])
-         if (a[2])
-            if (strstr (a[2], "error") || strstr (a[2], "warning")) 
-            {
-             *filename = g_strstrip (g_strdup (a[0]));
-             *lineno = strtol (a[1], NULL, 10);
-             g_strfreev (a);   
-             return TRUE; 
-            }
+  gchar **a = g_strsplit (line, ":", -1);
 
-   g_strfreev (a);   
+  if (a[0])
+     if (a[1])
+        if (a[2])
+           if (strstr (a[2], "error") || strstr (a[2], "warning"))
+              {
+               *filename = g_strstrip (g_strdup (a[0]));
+               *lineno = strtol (a[1], NULL, 10);
+               g_strfreev (a);
+               return TRUE;
+              }
+
+   g_strfreev (a);
    return FALSE;
 }
 
@@ -971,18 +878,18 @@ void clipboard_put_text (const gchar *s)
 }
 
 
-GList* read_dir_to_glist (gchar *path)
+GList* read_dir_to_glist (const gchar *path)
 {
   if (! path)
      return NULL;
 
   GDir *dir = g_dir_open (path, 0, NULL);
-  if (! dir) 
+  if (! dir)
      return NULL;
 
   GList *l = NULL;
   gchar *f = NULL;
-  
+
   while (f = g_dir_read_name (dir))
         l = g_list_prepend (l, g_strconcat (path, G_DIR_SEPARATOR_S, f, NULL));
 
@@ -991,23 +898,21 @@ GList* read_dir_to_glist (gchar *path)
 }
 
 
-GList* read_dir_files (gchar *path)
+GList* read_dir_files (const gchar *path)
 {
   if (! path)
       return NULL;
 
   GList *l = NULL;
 
-  GDir *d = g_dir_open (path, 0, NULL);  
+  GDir *d = g_dir_open (path, 0, NULL);
   gchar *t;
 
   while (t = g_dir_read_name (d))
-        {
          if (! g_file_test (t, G_FILE_TEST_IS_DIR))
              l = g_list_prepend (l, g_strdup (t));
-        }
 
-  g_dir_close (d);  
+  g_dir_close (d);
   return g_list_reverse (l);
 }
 
@@ -1029,7 +934,7 @@ gint get_markup_mode (void)
      return MM_DOCBOOK;
 
   return MM_HTML;
-} 
+}
 
 
 gchar* get_amarok_now_playing (void)
@@ -1037,21 +942,23 @@ gchar* get_amarok_now_playing (void)
   gchar *standard_output = NULL;
   gchar *standard_error = NULL;
   gint exit_status;
-  GError *error;
+  GError *error = NULL;
 
-  if (g_spawn_command_line_sync ("dcop amarok player nowPlaying", &standard_output,
-                                 &standard_error,
-                                 &exit_status,
-                                 &error))
+  if (g_spawn_command_line_sync ("dcop amarok player nowPlaying",
+                                 &standard_output, &standard_error,
+                                 &exit_status, &error))
      g_free (standard_error);
 
   gchar *x = locale_to_utf8 (standard_output);
   g_free (standard_output);
-  return g_strstrip (x);
+  if (x)
+     return g_strstrip (x);
+  else
+      return NULL;
 }
 
 
-gboolean is_ext (gchar *filename, gchar *ext1, ...)
+gboolean is_ext (const gchar *filename, gchar *ext1, ...)
 {
   if (! filename)
      return FALSE;
@@ -1059,15 +966,15 @@ gboolean is_ext (gchar *filename, gchar *ext1, ...)
   va_list args;
   gchar *t;
   gboolean r = FALSE;
-  
+
   va_start (args, ext1);
   t = ext1;
-  
+
   while (t)
         {
          r = check_ext (filename, t);
          if (r)
-            break;   
+            break;
 
          t = va_arg (args, gchar*);
         }
@@ -1078,7 +985,7 @@ gboolean is_ext (gchar *filename, gchar *ext1, ...)
 }
 
 
-gchar* run_process_and_wait (gchar *command) 
+gchar* run_process_and_wait (const gchar *command)
 {
   if (! command)
      return NULL;
@@ -1088,23 +995,27 @@ gchar* run_process_and_wait (gchar *command)
   gint exit_status;
   gchar *x = NULL;
 
-  if (command) 
-  if (g_spawn_command_line_sync (command, &standard_output,
-                                 &standard_error,
-                                 &exit_status,
-                                 NULL))
-     {
-      g_free (standard_error);
-      x = locale_to_utf8 (standard_output);
-      g_free (standard_output);
-     }
+  if (command)
+     if (g_spawn_command_line_sync (command, &standard_output,
+                                    &standard_error, &exit_status, NULL))
+       {
+        g_free (standard_error);
+        x = locale_to_utf8 (standard_output);
+        g_free (standard_output);
+       }
 
   return x;
 }
 
 
-void delete_dir (gchar *path)
+void delete_dir (const gchar *path)
 {
+  if (! path)
+     return;
+
+  if (g_utf8_collate (path, G_DIR_SEPARATOR_S) == 0)
+     return;
+
   gchar *cm = g_strconcat ("rm -r -f ", path, NULL);
   system (cm);
   g_free (cm);
@@ -1113,7 +1024,10 @@ void delete_dir (gchar *path)
 
 gchar* correct_relative_filename (const gchar *path, const gchar *filename)
 {
-  gchar *x = filename_from_xuri (filename); 
+  if (! filename)
+     return NULL;
+
+  gchar *x = filename_from_xuri (filename);
   gchar *r;
 
   if (g_utf8_get_char (x) == '/')
@@ -1122,8 +1036,8 @@ gchar* correct_relative_filename (const gchar *path, const gchar *filename)
       r = g_strconcat (path, G_DIR_SEPARATOR_S, filename, NULL);
 
   g_free (x);
-  return r;  
-} 
+  return r;
+}
 
 
 gchar* filename_from_xuri (const gchar *uri)
@@ -1150,50 +1064,52 @@ gchar* filename_from_xuri (const gchar *uri)
   if (mode == 0)
      return g_strdup (uri);
 
-  if (mode == 1) 
+  if (mode == 1)
      p+=5;
   else
       p+=7;
- 
+
   return g_strdup (p);
 }
 
 
 gchar *create_relative_link (const gchar *doc_filename, const gchar *img_filename)
 {
+  if (! doc_filename || ! img_filename)
+     return NULL;
+
   gchar *dir_doc_filename = g_path_get_dirname (doc_filename);
   gchar *dir_img_filename = g_path_get_dirname (img_filename);
-  
+
   gint dir_doc_filename_len = strlen (dir_doc_filename);
-  gint dir_img_filename_len = strlen (dir_img_filename);;
+  gint dir_img_filename_len = strlen (dir_img_filename);
 
   //if the directories are the same, return the pure filename
-  
+
   if (strcmp (dir_doc_filename, dir_img_filename) == 0)
      {
-      g_free (dir_doc_filename); 
+      g_free (dir_doc_filename);
       g_free (dir_img_filename);
       return g_path_get_basename (img_filename);
-     } 
+     }
 
-  //if img_filename is in the upper dir than doc_filename, we return the pure filename 
+  //if img_filename is in the upper dir than doc_filename, we return the pure filename
   if (strcmp (dir_img_filename, dir_doc_filename) < 0)
      {
-      g_free (dir_doc_filename); 
+      g_free (dir_doc_filename);
       g_free (dir_img_filename);
       return g_path_get_basename (img_filename);
-     } 
+     }
 
-  //if img_filename is in the lower dir than doc_filename, we return the relative filename 
+  //if img_filename is in the lower dir than doc_filename, we return the relative filename
   if (strcmp (dir_doc_filename, dir_img_filename) < 0)
   if (strstr (dir_img_filename, dir_doc_filename))
      {
-      
-      gchar *rel_path = dir_img_filename + dir_doc_filename_len;      
+      gchar *rel_path = dir_img_filename + dir_doc_filename_len;
       gchar *pure_filename = g_path_get_basename (img_filename);
-      gchar *x = g_strconcat (".", rel_path, "/", pure_filename, NULL);
+      gchar *x = g_strconcat (".", rel_path, G_DIR_SEPARATOR_S, pure_filename, NULL);
       g_free (pure_filename);
-      g_free (dir_doc_filename); 
+      g_free (dir_doc_filename);
       g_free (dir_img_filename);
 
       return g_strdup (x);
@@ -1202,19 +1118,32 @@ gchar *create_relative_link (const gchar *doc_filename, const gchar *img_filenam
    return g_path_get_basename (img_filename);
 }
 
-///////////////////////WRITE ME
-gchar* create_full_path2 (gchar *filename, gchar *basedir) 
+
+gchar* create_full_path (const gchar *filename, const gchar *basedir)
 {
+  if (! filename && ! basedir)
+     return NULL;
 
+  if (g_path_is_absolute (filename))
+     return g_strdup (filename);
+
+  gchar *x = filename;
+  if (x[0] == '.')
+     x++;
+
+  if (x[0] == G_DIR_SEPARATOR)
+     x++;
+
+  gchar *p = g_strconcat (basedir, G_DIR_SEPARATOR_S, x, NULL);
+  return p;
 }
-//////////////////////
 
 
-gboolean is_ext_arr (gchar *filename, gchar *exts)
+gboolean is_ext_arr (const gchar *filename, const gchar *exts)
 {
   if (! filename || ! exts)
      return FALSE;
- 
+
   gboolean r = FALSE;
 
   gchar **arr = g_strsplit (exts, ";", -1);
@@ -1230,7 +1159,7 @@ gboolean is_ext_arr (gchar *filename, gchar *exts)
        if (r)
           break;
       }
-            
+
   g_strfreev (arr);
 
   return r;
@@ -1239,10 +1168,13 @@ gboolean is_ext_arr (gchar *filename, gchar *exts)
 
 void add_image_format (GdkPixbufFormat *data, GList **list)
 {
-  gchar** arr = gdk_pixbuf_format_get_extensions (data); 
+  if (! data)
+     return;
+
+  gchar** arr = gdk_pixbuf_format_get_extensions (data);
   if (! arr)
      return;
- 
+
   gchar *extensions = g_strjoinv (";", arr);
   *list = g_list_prepend (*list, extensions);
   g_strfreev (arr);
@@ -1261,7 +1193,7 @@ GList* image_formats_fill (void)
 
 gboolean is_image (const gchar *filename)
 {
-  if (! gl_image_formats)
+  if (! gl_image_formats || ! filename)
      return FALSE;
 
   gboolean r = FALSE;
@@ -1271,7 +1203,7 @@ gboolean is_image (const gchar *filename)
         {
          r = is_ext_arr (filename, t->data);
          if (r)
-             break;           
+             break;
 
          t = g_list_next (t);
         }
@@ -1280,7 +1212,7 @@ gboolean is_image (const gchar *filename)
 }
 
 
-gboolean is_writable (gchar *filename)
+gboolean is_writable (const gchar *filename)
 {
   if (access (filename, W_OK) != 0)
      return FALSE;
@@ -1289,7 +1221,7 @@ gboolean is_writable (gchar *filename)
 }
 
 
-gboolean is_readable (gchar *filename)
+gboolean is_readable (const gchar *filename)
 {
   if (access (filename, R_OK) != 0)
      return FALSE;
@@ -1298,78 +1230,122 @@ gboolean is_readable (gchar *filename)
 }
 
 
-gboolean has_ext (gchar *filename)
+gboolean has_ext (const gchar *filename)
 {
   return (strstr (++filename, "."));
 }
 
 /*
-
-from 
-/*  BMPx - The Dumb Music Player
- *  Copyright (C) 2005 BMPx development team.
+from:
+  BMPx - The Dumb Music Player
+  Copyright (C) 2005 BMPx development team.
 */
 
 
-gboolean
-dir_foreach (const gchar    *path,
-             DirForeachFunc  function,
-	     gpointer        user_data,
-             GError        **error)
+gboolean dir_foreach (const gchar *path, DirForeachFunc function,
+                      gpointer user_data, GError **error)
 {
   GDir *dir;
   const gchar *entry;
   gchar *entry_fullpath;
 
-  if (!(dir = g_dir_open (path, 0, error)))
+  if (! (dir = g_dir_open (path, 0, error)))
     return FALSE;
 
   while ((entry = g_dir_read_name (dir)))
-    {
-      entry_fullpath = g_build_filename (path, entry, NULL);
+       {
+        entry_fullpath = g_build_filename (path, entry, NULL);
 
-      if ((*function) (entry_fullpath, entry, user_data))
-        {
-          g_free (entry_fullpath);
-          break;
-        }
+        if ((*function) (entry_fullpath, entry, user_data))
+           {
+            g_free (entry_fullpath);
+            break;
+           }
 
-      g_free (entry_fullpath);
-    }
+         g_free (entry_fullpath);
+       }
 
   g_dir_close (dir);
-
   return TRUE;
 }
 
 
-gboolean cbf_listdir (const gchar *path, const gchar *basename, gpointer  user_data)
+gboolean cbf_listdir (const gchar *path, const gchar *basename, gpointer user_data)
 {
   GError *error = NULL;
 
   if (g_file_test (path, G_FILE_TEST_IS_DIR))
-      {
-       dir_foreach (path, cbf_listdir, user_data, &error);
-       return FALSE;
-      }
+     {
+      dir_foreach (path, cbf_listdir, user_data, &error);
+      return FALSE;
+     }
 
   if (g_pattern_match_simple (user_data, path))
       gl_found_files = g_list_prepend (gl_found_files, g_strdup (path));
 
- /*
-  while (gtk_events_pending ())
-          gtk_main_iteration ();
-
-*/
 
   return FALSE;
 }
 
 
-void read_dir_files_recurse (gchar *path, gchar *pattern)
+void read_dir_files_recurse (const gchar *path, const gchar *pattern)
 {
+  if (! path || ! pattern)
+     return;
+
   glist_strings_free (gl_found_files);
   gl_found_files = NULL;
   GError *error = NULL;
   dir_foreach (path, cbf_listdir, pattern, &error);
 }
+
+
+gchar* get_build_info (void)
+{
+   return g_strdup_printf (_("(built on %s with GTK %d.%d.%d and GLib %d.%d.%d)"),
+                          __DATE__, GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION,
+                          GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
+}
+
+
+gchar* file_replace_path (const gchar *filename, const gchar *new_dir)
+{
+  if (! filename || ! new_dir)
+     return NULL;
+
+  gchar *name = g_strrstr (filename, G_DIR_SEPARATOR_S);
+  if (! name)
+     return NULL;
+
+  if (g_str_has_suffix  (new_dir, G_DIR_SEPARATOR_S))
+     return g_strconcat (new_dir, name, NULL);
+  else
+     return g_strconcat (new_dir, name, G_DIR_SEPARATOR_S, NULL);
+
+}
+
+
+gchar* file_combine_path (const gchar *dir, const gchar *filename)
+{
+  if (! filename || ! dir)
+     return NULL;
+
+  if (g_str_has_suffix  (dir, G_DIR_SEPARATOR_S))
+     return g_strconcat (dir, filename, NULL);
+  else
+     return g_strconcat (dir, filename, G_DIR_SEPARATOR_S, NULL);
+}
+
+
+gchar* add_slash_if_need (const gchar *dir)
+{
+  if (! dir)
+     return NULL;
+
+  if (g_str_has_suffix  (dir, G_DIR_SEPARATOR_S))
+     return g_strdup (dir);
+  else
+     return g_strconcat (dir, G_DIR_SEPARATOR_S, NULL);
+}
+
+
