@@ -2,7 +2,7 @@
                           tea_gtk_utils.c  -  description
                              -------------------
     begin                : aug. 2004
-    copyright            : (C) 2004, 2005 by Peter 'Roxton' Semiletov
+    copyright            : (C) 2004-2006 by Peter 'Roxton' Semiletov
     email                : peter.semiletov@gmail.com
  ***************************************************************************/
 //  Copyright (C) 1998, 1999 Alex Roberts, Evan Lawrence
@@ -22,62 +22,61 @@
 
 #include <glib.h>
 #include <gtk/gtk.h>
-#include <unistd.h>
 #include <sys/stat.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 #include <stddef.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 
 #include "tea_defs.h"
+#include "tea_text_document.h" // Needed for t_note_page struct
 #include "tea_config.h"
 #include "tea_gtk_utils.h"
 
 
-GtkWidget* tea_text_entry (GtkWidget *container, gchar *caption, gchar *value)
+GtkWidget* tea_text_entry (GtkWidget *container, const gchar *caption, const gchar *value)
 {
   GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (b);
   gtk_box_pack_start (container, b, FALSE, TRUE, UI_PACKFACTOR);
- 
-  GtkWidget *x = gtk_label_new (caption);  
-  //gtk_label_set_markup (x, value); 
-  gtk_label_set_use_markup (x, TRUE); 
 
+  GtkWidget *x = gtk_label_new (caption);
+  gtk_label_set_use_markup (x, TRUE);
 
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, FALSE, FALSE, UI_PACKFACTOR);
-  x = gtk_entry_new ();   
+  x = gtk_entry_new ();
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, TRUE, TRUE, UI_PACKFACTOR);
 
   if (value)
      gtk_entry_set_text (x, value);
-    
+
   return x;
 }
 
 
-static void cb_select_color (GtkColorButton *widget, gpointer user_data) 
+static void cb_select_color (GtkColorButton *widget, gpointer user_data)
 {
   GdkColor color;
-  gtk_color_button_get_color (widget, &color);  
+  gtk_color_button_get_color (widget, &color);
   gchar *t = g_strdup_printf ("#%02x%02x%02x", color.red / 256, color.green / 256, color.blue / 256);
-  gtk_entry_set_text (GTK_ENTRY (user_data), t);  
+  gtk_entry_set_text (GTK_ENTRY (user_data), t);
   g_free (t);
 }
 
 
-GtkWidget* tea_color_selector (GtkWidget *container, gchar *caption, gchar *value)
+GtkWidget* tea_color_selector (GtkWidget *container, const gchar *caption, const gchar *value)
 {
   GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (b);
   gtk_box_pack_start (container, b, FALSE, TRUE, UI_PACKFACTOR);
 
-  GtkWidget *x = gtk_label_new (caption);  
+  GtkWidget *x = gtk_label_new (caption);
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, FALSE, FALSE, UI_PACKFACTOR);
 
@@ -87,7 +86,7 @@ GtkWidget* tea_color_selector (GtkWidget *container, gchar *caption, gchar *valu
 
   if (value)
      gtk_entry_set_text (x, value);
-  
+
   GdkColor color;
   gdk_color_parse (value, &color);
 
@@ -96,12 +95,12 @@ GtkWidget* tea_color_selector (GtkWidget *container, gchar *caption, gchar *valu
   gtk_box_pack_start (b, bt, FALSE, FALSE, UI_PACKFACTOR);
 
   g_signal_connect (bt, "color-set", G_CALLBACK (cb_select_color), x);
-  
+
   return x;
 }
 
 
-GtkWidget* tea_checkbox (GtkWidget *container, gchar *caption, gboolean *value)
+GtkWidget* tea_checkbox (GtkWidget *container, const gchar *caption, gboolean value)
 {
   GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (b);
@@ -111,29 +110,29 @@ GtkWidget* tea_checkbox (GtkWidget *container, gchar *caption, gboolean *value)
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, TRUE, TRUE, UI_PACKFACTOR);
   gtk_toggle_button_set_active (&x->toggle_button, value);
-    
+
   return x;
 }
 
 
-GtkWidget* tea_spinbutton (GtkWidget *container, gchar *caption, gdouble value)
+GtkWidget* tea_spinbutton (GtkWidget *container, const gchar *caption, gdouble value)
 {
   GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (b);
   gtk_box_pack_start (container, b, FALSE, TRUE, UI_PACKFACTOR);
 
-  GtkWidget *l = gtk_label_new (caption);  
+  GtkWidget *l = gtk_label_new (caption);
   gtk_widget_show (l);
   gtk_box_pack_start (b, l, FALSE, FALSE, UI_PACKFACTOR);
- 
+
   GtkCheckButton *x = gtk_spin_button_new (gtk_adjustment_new (value,
                                                                0.0, G_MAXDOUBLE,
                                                                1.0, 10.0, 0),
-                                           1, 0); 
+                                           1, 0);
 
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, TRUE, TRUE, UI_PACKFACTOR);
-    
+
   return x;
 }
 
@@ -149,7 +148,7 @@ void logmemo_set_pos (GtkTextView *v, guint pos)
 }
 
 
-static void cb_select_dir (GtkObject *object, GtkWidget *w) 
+static void cb_select_dir (GtkObject *object, GtkWidget *w)
 {
   gchar *dir;
   gchar *t;
@@ -161,16 +160,14 @@ static void cb_select_dir (GtkObject *object, GtkWidget *w)
                                                    GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                                    NULL);
 
-  if (g_file_test (gtk_entry_get_text (e), G_FILE_TEST_EXISTS)) 
+  if (g_file_test (gtk_entry_get_text (e), G_FILE_TEST_EXISTS))
       gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
                                            gtk_entry_get_text (e));
 
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
     {
      dir = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog));
-     t = g_strconcat (dir, G_DIR_SEPARATOR_S, NULL);
-     gtk_entry_set_text (e, t);  
-     g_free (t);
+     gtk_entry_set_text (e, dir);
      g_free (dir);
     }
 
@@ -178,68 +175,13 @@ static void cb_select_dir (GtkObject *object, GtkWidget *w)
 }
 
 
-static void cb_select_dir2 (GtkObject *object, GtkWidget *w) 
-{
-  gchar *dir;
-  gchar *t;
-  GtkEntry *e = w;
-  GtkWidget *dialog = gtk_file_chooser_dialog_new (_("Select a directory"),
-                                                   NULL,
-                                                   GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
-                                                   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                                   GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-                                                   NULL);
-
-  if (g_file_test (gtk_entry_get_text (e), G_FILE_TEST_EXISTS)) 
-      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
-                                           gtk_entry_get_text (e));
-
-  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
-    {
-     dir = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog));
-     gtk_entry_set_text (e, dir);  
-     g_free (dir);
-    }
-
-  gtk_widget_destroy (dialog);
-}
-
-
-GtkWidget* tea_dir_selector (GtkWidget *container, gchar *caption, gchar *value)
+GtkWidget* tea_dir_selector (GtkWidget *container, const gchar *caption, const gchar *value)
 {
   GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (b);
   gtk_box_pack_start (container, b, FALSE, TRUE, UI_PACKFACTOR);
 
-  GtkWidget *x = gtk_label_new (caption);  
-  gtk_widget_show (x);
-  gtk_box_pack_start (b, x, FALSE, FALSE, UI_PACKFACTOR);
-
-  x = gtk_entry_new ();   
-  gtk_widget_show (x);
-  gtk_box_pack_start (b, x, TRUE, TRUE, UI_PACKFACTOR);
-
-  if (value)
-     gtk_entry_set_text (x, value);
-
-//  gtk_entry_set_editable (x, FALSE);
-
-  GtkWidget *bt = tea_stock_button (GTK_STOCK_OPEN);
-  gtk_widget_show (bt);
-  gtk_box_pack_start (b, bt, FALSE, FALSE, UI_PACKFACTOR);
-  g_signal_connect (bt, "clicked", G_CALLBACK (cb_select_dir), x);
-
-  return x;
-}
-
-
-GtkWidget* tea_dir_selector2 (GtkWidget *container, gchar *caption, gchar *value)
-{
-  GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
-  gtk_widget_show (b);
-  gtk_box_pack_start (container, b, FALSE, TRUE, UI_PACKFACTOR);
-
-  GtkWidget *x = gtk_label_new (caption);  
+  GtkWidget *x = gtk_label_new (caption);
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, FALSE, FALSE, UI_PACKFACTOR);
 
@@ -253,45 +195,45 @@ GtkWidget* tea_dir_selector2 (GtkWidget *container, gchar *caption, gchar *value
   GtkWidget *bt = tea_stock_button (GTK_STOCK_OPEN);
   gtk_widget_show (bt);
   gtk_box_pack_start (b, bt, FALSE, FALSE, UI_PACKFACTOR);
-  g_signal_connect (bt, "clicked", G_CALLBACK (cb_select_dir2), x);
+  g_signal_connect (bt, "clicked", G_CALLBACK (cb_select_dir), x);
 
   return x;
 }
 
 
 //returns combo
-GtkWidget* tea_combo_combo (GtkWidget *container, GList *strings, gchar *caption, gint value)
+GtkWidget* tea_combo_combo (GtkWidget *container, GList *strings, const gchar *caption, gint value)
 {
   GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (b);
   gtk_box_pack_start (container, b, FALSE, TRUE, UI_PACKFACTOR);
- 
-  GtkWidget *x = gtk_label_new (caption);  
+
+  GtkWidget *x = gtk_label_new (caption);
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, FALSE, FALSE, UI_PACKFACTOR);
-  x = gtk_combo_box_new_text ();   
+  x = gtk_combo_box_new_text ();
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, TRUE, TRUE, UI_PACKFACTOR);
 
   if (strings)
     fill_combo_with_glist (x, strings);
-  
+
    gtk_combo_box_set_active (x, value);
 
   return x;
 }
 
 
-GtkWidget* tea_combo (GtkWidget *container, GList *strings, gchar *caption, gchar *value)
+GtkWidget* tea_combo (GtkWidget *container, GList *strings, const gchar *caption, const gchar *value)
 {
   GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (b);
   gtk_box_pack_start (container, b, FALSE, TRUE, UI_PACKFACTOR);
- 
-  GtkWidget *x = gtk_label_new (caption);  
+
+  GtkWidget *x = gtk_label_new (caption);
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, FALSE, FALSE, UI_PACKFACTOR);
-  x = gtk_combo_box_entry_new_text ();   
+  x = gtk_combo_box_entry_new_text ();
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, TRUE, TRUE, UI_PACKFACTOR);
 
@@ -335,21 +277,21 @@ void widget_set_colors (GtkWidget *w, GdkColor *background, GdkColor *text,
 }
 
 
-gboolean dlg_question (gchar *caption, gchar *prompt, gchar *val)
+gboolean dlg_question (const gchar *caption, const gchar *prompt, const gchar *val)
 {
-   gboolean result = FALSE;  
-   GtkWidget *dialog = gtk_dialog_new_with_buttons (caption, 
+   gboolean result = FALSE;
+   GtkWidget *dialog = gtk_dialog_new_with_buttons (caption,
                                                     NULL,
                                                     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                                     GTK_STOCK_NO,
-                                                    GTK_RESPONSE_REJECT, 
+                                                    GTK_RESPONSE_REJECT,
                                                     GTK_STOCK_YES,
                                                     GTK_RESPONSE_ACCEPT,
                                                     NULL);
-  gchar *s; 
+  gchar *s;
   if (val)
      s = g_strdup_printf (prompt, val);
-  else    
+  else
       s = g_strdup (prompt);
 
   GtkWidget *hbox = gtk_hbox_new (FALSE, UI_PACKFACTOR);
@@ -361,10 +303,10 @@ gboolean dlg_question (gchar *caption, gchar *prompt, gchar *val)
   gtk_container_add (hbox, label);
   gtk_widget_show_all (hbox);
   gtk_container_add (GTK_DIALOG (dialog)->vbox, hbox);
- 
+
   if (gtk_dialog_run (dialog) == GTK_RESPONSE_ACCEPT)
      result = TRUE;
-   
+
   gtk_widget_destroy (dialog);
   g_free (s);
 
@@ -372,18 +314,18 @@ gboolean dlg_question (gchar *caption, gchar *prompt, gchar *val)
 }
 
 
-void dlg_info (gchar *caption, gchar *prompt, gchar *val)
+void dlg_info (const gchar *caption, const gchar *prompt, const gchar *val)
 {
-  GtkWidget *dialog = gtk_dialog_new_with_buttons (caption, 
+  GtkWidget *dialog = gtk_dialog_new_with_buttons (caption,
                                                    NULL,
                                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                                    GTK_STOCK_OK,
                                                    GTK_RESPONSE_ACCEPT,
                                                    NULL);
-  gchar *s; 
+  gchar *s;
   if (val)
      s = g_strdup_printf (prompt, val);
-  else    
+  else
       s = g_strdup (prompt);
 
   GtkWidget *hbox = gtk_hbox_new (FALSE, UI_PACKFACTOR);
@@ -401,7 +343,7 @@ void dlg_info (gchar *caption, gchar *prompt, gchar *val)
 }
 
 
-GtkWidget* tea_button_at_box (GtkWidget *container, gchar *caption, gpointer callback, gpointer user_data)
+GtkWidget* tea_button_at_box (GtkWidget *container, const gchar *caption, gpointer callback, gpointer user_data)
 {
   GtkWidget *b = gtk_button_new_with_label (caption);
   gtk_widget_show (b);
@@ -420,7 +362,7 @@ GtkWidget* tea_button_at_box (GtkWidget *container, gchar *caption, gpointer cal
 }
 
 
-void cb_on_bt_close (GtkObject *object, GtkWidget *w) 
+void cb_on_bt_close (GtkObject *object, GtkWidget *w)
 {
   window_destroy (w);
 }
@@ -435,17 +377,17 @@ static void find_menuitem_cb (GtkWidget *widget, gpointer data)
 
   if (g_utf8_collate (data, s) == 0)
      {
-      mni_temp = widget;       
+      mni_temp = widget;
       return;
      }
 
   if (GTK_IS_MENU_ITEM (widget))
-     find_menuitem (GTK_CONTAINER (gtk_menu_item_get_submenu (widget)), data);      
-        
+     find_menuitem (GTK_CONTAINER (gtk_menu_item_get_submenu (widget)), data);
+
 }
 
 
-GtkWidget* find_menuitem (GtkContainer *widget, gchar *widget_name)
+GtkWidget* find_menuitem (GtkContainer *widget, const gchar *widget_name)
 {
   if (widget)
      gtk_container_foreach (widget, find_menuitem_cb, widget_name);
@@ -454,7 +396,7 @@ GtkWidget* find_menuitem (GtkContainer *widget, gchar *widget_name)
 }
 
 
-GtkFileFilter* tea_file_filter_with_patterns (gchar *name, gchar *pattern1, ...)
+GtkFileFilter* tea_file_filter_with_patterns (const gchar *name, gchar *pattern1, ...)
 {
   va_list args;
   gchar *t;
@@ -475,7 +417,7 @@ GtkFileFilter* tea_file_filter_with_patterns (gchar *name, gchar *pattern1, ...)
 }
 
 
-GtkFileFilter* find_ffilter_by_name (GtkFileChooser *chooser, gchar *name)
+GtkFileFilter* find_ffilter_by_name (GtkFileChooser *chooser, const gchar *name)
 {
   GtkFileFilter *f = NULL;
   GSList *l = gtk_file_chooser_list_filters (chooser);
@@ -490,7 +432,7 @@ GtkFileFilter* find_ffilter_by_name (GtkFileChooser *chooser, gchar *name)
 
   g_slist_free (l);
   return f;
-} 
+}
 
 
 GtkWidget* daisho_add_del (gpointer *f1, gpointer f2)
@@ -538,6 +480,7 @@ GtkWidget* new_menu_item (const gchar *label, GtkWidget *parent, gpointer f)
   GtkWidget *item;
 
   item = gtk_menu_item_new_with_label (label);
+
   gtk_widget_show (item);
   gtk_container_add (GTK_CONTAINER (parent), item);
   gtk_widget_set_name (item, label);
@@ -550,8 +493,8 @@ GtkWidget* new_menu_item (const gchar *label, GtkWidget *parent, gpointer f)
 
 
 //new_action_item returns menu item
-GtkWidget* new_action_item (GtkActionGroup *ag, 
-                            GtkWidget *parent, 
+GtkWidget* new_action_item (GtkActionGroup *ag,
+                            GtkWidget *parent,
                             const gchar *name,
                             const gchar *label,
                             const gchar *tooltip,
@@ -607,7 +550,7 @@ GtkWidget* new_menu_submenu (GtkWidget *parent)
 }
 
 
-gboolean win_key_handler (GtkWidget *widget, GdkEventKey *event, GtkWidget *w) 
+gboolean win_key_handler (GtkWidget *widget, GdkEventKey *event, GtkWidget *w)
 {
   if (event->keyval == GDK_Escape)
     {
@@ -618,11 +561,11 @@ gboolean win_key_handler (GtkWidget *widget, GdkEventKey *event, GtkWidget *w)
 }
 
 
-gboolean dlg_key_handler (GtkWidget *widget, GdkEventKey *event, GtkWidget *w) 
+gboolean dlg_key_handler (GtkWidget *widget, GdkEventKey *event, GtkWidget *w)
 {
   if (event->keyval == GDK_Escape)
     {
-     gtk_dialog_response (GTK_DIALOG (widget), GTK_RESPONSE_REJECT);   
+     gtk_dialog_response (GTK_DIALOG (widget), GTK_RESPONSE_REJECT);
      return TRUE;
     }
   return FALSE;
@@ -632,9 +575,9 @@ gboolean dlg_key_handler (GtkWidget *widget, GdkEventKey *event, GtkWidget *w)
 /**
  * window_destroy:
  * @windowname: a #GtkWidget pointer to the window
- * 
+ *
  * Remove grab and signals and then destroy window
- * 
+ *
  * Return value: void
  **/
 void window_destroy(GtkWidget * windowname) {
@@ -654,7 +597,7 @@ void set_cursor_blink (gboolean value)
 
 
 GtkWidget* tea_gtk_toolbar_prepend_space (GtkToolbar *toolbar)
-{ 
+{
   return gtk_toolbar_prepend_element (toolbar,
                                       GTK_TOOLBAR_CHILD_SPACE,
                                       NULL,
@@ -675,24 +618,24 @@ void fill_combo_with_glist (GtkComboBox *combo, GList *l)
   GList *t = g_list_first (l);
   while (t && t->data)
         {
-         //if (strlen (t->data) > 1) 
+         //if (strlen (t->data) > 1)
             gtk_combo_box_append_text (combo, t->data);
-         t = g_list_next (t); 
-        }   
+         t = g_list_next (t);
+        }
 }
 
 
-GtkWidget* tea_font_button (GtkWidget *container, gchar *caption, gchar *value)
+GtkWidget* tea_font_button (GtkWidget *container, const gchar *caption, const gchar *value)
 {
   GtkWidget *b = gtk_hbox_new (TRUE, UI_PACKFACTOR);
   gtk_widget_show (b);
   gtk_box_pack_start (container, b, FALSE, TRUE, UI_PACKFACTOR);
 
-  GtkWidget *x = gtk_label_new (caption);  
+  GtkWidget *x = gtk_label_new (caption);
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, FALSE, FALSE, UI_PACKFACTOR);
 
-  x = gtk_font_button_new ();   
+  x = gtk_font_button_new ();
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, FALSE, TRUE, UI_PACKFACTOR);
 
@@ -702,27 +645,6 @@ GtkWidget* tea_font_button (GtkWidget *container, gchar *caption, gchar *value)
   return x;
 }
 
-/*
-GtkWidget* tea_font_button (GtkWidget *container, gchar *caption, gchar *value)
-{
-  GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
-  gtk_widget_show (b);
-  gtk_box_pack_start (container, b, FALSE, FALSE, UI_PACKFACTOR);
-
-  GtkWidget *x = gtk_label_new (caption);  
-  gtk_widget_show (x);
-  gtk_box_pack_start (b, x, FALSE, FALSE, UI_PACKFACTOR);
-
-  x = gtk_font_button_new ();   
-  gtk_widget_show (x);
-  gtk_box_pack_start (b, x, FALSE, FALSE, UI_PACKFACTOR);
-
-  if (value)
-     gtk_font_button_set_font_name (x, value);
-
-  return x;
-}
-*/
 
 GtkWidget* tea_stock_button (const gchar *stock_id)
 {
@@ -734,32 +656,32 @@ GtkWidget* tea_stock_button (const gchar *stock_id)
 }
 
 
-gchar* dlg_question_char (gchar *prompt, gchar *defval)
+gchar* dlg_question_char (const gchar *prompt, const gchar *defval)
 {
-  gchar *result = NULL;  
-  GtkWidget *dialog = gtk_dialog_new_with_buttons (NULL, 
+  gchar *result = NULL;
+  GtkWidget *dialog = gtk_dialog_new_with_buttons (NULL,
                                                    NULL,
                                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                                    GTK_STOCK_CANCEL,
-                                                   GTK_RESPONSE_REJECT, 
+                                                   GTK_RESPONSE_REJECT,
                                                    GTK_STOCK_OK,
                                                    GTK_RESPONSE_ACCEPT,
                                                    NULL);
-   
+
   GtkWidget *hbox = gtk_hbox_new (FALSE, UI_PACKFACTOR);
 
   GtkWidget *im = gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_DIALOG);
   gtk_container_add (GTK_DIALOG (dialog)->vbox, im);
   gtk_widget_show (im);
-  
+
   GtkWidget *ent = tea_text_entry (GTK_DIALOG (dialog)->vbox, prompt, defval);
 
   if (gtk_dialog_run (dialog) == GTK_RESPONSE_ACCEPT)
   if (gtk_entry_get_text (ent))
       result = g_strdup (gtk_entry_get_text (ent));
-   
+
   gtk_widget_destroy (dialog);
-  
+
   return result;
 }
 
@@ -776,11 +698,11 @@ static build_menu_from_ht_cb (gpointer key,
 void build_menu_from_ht (GHashTable *hash_table, gpointer menu, gpointer callback)
 {
    if (! ht_entities && ! hash_table)
-       return; 
- 
+       return;
+
    t_ppair t;
    t.a = menu;
-   t.b = callback; 
+   t.b = callback;
 
    g_hash_table_foreach (hash_table,
                          build_menu_from_ht_cb,
@@ -791,8 +713,8 @@ void build_menu_from_ht (GHashTable *hash_table, gpointer menu, gpointer callbac
 void build_menu_from_glist (GList *list, gpointer menu, gpointer callback)
 {
    if (! list)
-       return; 
- 
+       return;
+
    GList *p = g_list_first (list);
    while (p)
         {
@@ -807,7 +729,7 @@ void build_menu_from_glist (GList *list, gpointer menu, gpointer callback)
 void build_menu_wudata_from_glist (GList *list, gpointer menu, gpointer callback, gpointer udata)
 {
   if (! list)
-     return; 
+     return;
   GList *p = g_list_first (list);
   while (p)
        {
@@ -823,31 +745,43 @@ void tea_widget_show (GtkWidget *w, gboolean visible)
 {
   if (visible)
      gtk_widget_show (w);
-   else 
+   else
        gtk_widget_hide (w);
 }
 
 
-GtkWidget* tea_color_selector_simple (GtkWidget *container, gchar *caption)
+GtkWidget* tea_color_selector_simple (GtkWidget *container, const gchar *caption)
 {
   GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (b);
   gtk_box_pack_start (container, b, FALSE, TRUE, UI_PACKFACTOR);
 
-  GtkWidget *x = gtk_label_new (caption);  
+  GtkWidget *x = gtk_label_new (caption);
   gtk_widget_show (x);
   gtk_box_pack_start (b, x, FALSE, FALSE, UI_PACKFACTOR);
 
   GtkWidget *bt = gtk_color_button_new ();
   gtk_widget_show (bt);
   gtk_box_pack_start (b, bt, FALSE, FALSE, UI_PACKFACTOR);
-  
+
   return bt;
 }
 
 gchar* get_gtk_ver (void)
 {
-   return g_strdup_printf ("%d.%d.%d", gtk_major_version, gtk_minor_version, gtk_micro_version);
-} 
+  return g_strdup_printf ("%d.%d.%d", gtk_major_version, gtk_minor_version, gtk_micro_version);
+}
 
 
+gchar* get_text_buf (gpointer text_buffer)
+{
+  if (! text_buffer)
+     return NULL;
+
+  GtkTextIter itstart, itend;
+
+  gtk_text_buffer_get_iter_at_offset (text_buffer, &itstart, 0);
+  gtk_text_buffer_get_iter_at_offset(text_buffer, &itend, gtk_text_buffer_get_char_count (text_buffer));
+
+  return gtk_text_buffer_get_text (text_buffer, &itstart, &itend, FALSE);
+}
