@@ -2,11 +2,13 @@
                           tea_kwas.c  -  description
                              -------------------
     begin                : 11.06.2004 - it was a really good day
-    copyright            : (C) 2004, 2005 by Peter 'Roxton' Semiletov
+    copyright            : (C) 2004-2006 by Peter 'Roxton' Semiletov
     email                : peter.semiletov@gmail.com
  ***************************************************************************/
 /*
 portions of code from:
+Michele Garoche <michele.garoche@easyconnect.fr>
+
 ** Beaver's an Early AdVanced EditoR
 ** (C) 1999-2000 Marc Bevand, Damien Terrier and Emmanuel Turquin
 **
@@ -40,36 +42,44 @@ portions of code from:
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <string.h>
+
 
 #include "tea_defs.h"
+#include "tea_text_document.h" // Needs to be above for t_note_page struct
 #include "tea_funx.h"
 #include "rox_strings.h"
-#include "tea_text_document.h"
 #include "interface.h"
 #include "tea_config.h"
 #include "tea_kwas.h"
 #include "tea_gtk_utils.h"
 #include "image_browser.h"
 
- 
+#ifdef CHAI_MODE
+
+#include <libgnomevfs/gnome-vfs.h>
+
+#endif
+
+
 enum
 {
   COL_TEXT = 0,
   COL_ACCESS,
   COL_SIZE,
-  COL_DATETIME, 
+  COL_DATETIME,
   NUM_COLS
 };
 
 
-typedef struct 
+typedef struct
               {
                struct stat *fstat;
                gchar *path;
                gchar *access;
                gchar *file_size;
                gchar *file_time;
-              } 
+              }
                t_flist_item;
 
 
@@ -78,7 +88,7 @@ typedef struct
 static gchar* get_file_mode (struct stat *Stats)
 {
   static gchar Mode[10];
-  
+
   g_snprintf (Mode, 10, "%c%c%c%c%c%c%c%c%c",
 	      (Stats->st_mode & S_IRUSR) ? 'r' : '-',
 	      (Stats->st_mode & S_IWUSR) ? 'w' : '-',
@@ -93,39 +103,23 @@ static gchar* get_file_mode (struct stat *Stats)
 }
 
 
-static void kwas_navigate (t_kwas_win *w, gchar *path)
+static void kwas_navigate (t_kwas_win *w, const gchar *path)
 {
   if (w->focused_panel)
-     { 
+     {
       gchar *dir = filename_from_xuri (path);
       g_free (w->focused_panel->path);
-      w->focused_panel->path = dir; 
+      w->focused_panel->path = dir;
       fill_filelist (w->focused_panel, dir);
-     } 
+     }
 }
 
-/*
-static void on_mni_kwas_cin_filelist (GtkButton *button,
-                                       gpointer user_data)
-{
-  t_kwas_win *w = user_data;
-  cinelerra_gen_filelins_dialog (w->focused_panel->path);
-}
-*/
 
 static void on_mni_kwas_imageplane (GtkButton *button,
                                     gpointer user_data)
 {
   t_kwas_win *w = user_data;
-
-  if (! wnd_imageplane)
-     wnd_imageplane = create_image_browser (w->focused_panel->path);
-  else 
-     {
-      window_destroy (wnd_imageplane);
-      wnd_imageplane = NULL;
-      wnd_imageplane = create_image_browser (w->focused_panel->path);
-     }
+  t_image_plane *plane = create_image_plane (w->focused_panel->path);
 }
 
 
@@ -140,7 +134,7 @@ static void kwas_bookmarks_menu_update (t_kwas_win *w)
 {
   if (! gl_tea_kwas_bookmarks)
      return;
- 
+
   glist_strings_free (gl_tea_kwas_bookmarks);
   gl_tea_kwas_bookmarks = load_file_to_glist (confile.tea_kwas_bookmarks);
 
@@ -151,10 +145,10 @@ static void kwas_bookmarks_menu_update (t_kwas_win *w)
 
   while (p)
        {
-        if (p->data)                 
-        if (strlen (p->data) > 1)
-            mni_temp = new_menu_item_with_udata ((gchar *)p->data, w->mni_bookmarks_menu, on_mni_kwas_open_bookmark, w);
-        
+        if (p->data)
+           if (strlen (p->data) > 1)
+              mni_temp = new_menu_item_with_udata (p->data, w->mni_bookmarks_menu, on_mni_kwas_open_bookmark, w);
+
         p = g_list_next (p);
        }
 }
@@ -166,14 +160,14 @@ static gboolean on_ent_nav_key_press_event (GtkWidget *widget,
 {
   t_kwas_win *w = user_data;
   if (! w)
-      return;
+      return FALSE;
 
   if (event->keyval != GDK_Return)
      return FALSE;
 
   gchar *s = gtk_entry_get_text (w->cm_entry);
 
-  if (access (s, R_OK) == 0) 
+  if (access (s, R_OK) == 0)
     {
      g_free (w->focused_panel->path);
      w->focused_panel->path = g_strdup (s);
@@ -193,26 +187,27 @@ static void on_execute_button (GtkButton *button,
 
   gchar *s = gtk_entry_get_text (w->cm_entry);
   if (! s)
-     return; 
+     return;
 
-  if (access (s, R_OK) == 0) 
+  if (access (s, R_OK) == 0)
       kwas_navigate (w, s);
 }
 
 
 static void on_mni_kwas_insert_link (GtkMenuItem *menuitem,
-                                      gpointer user_data)
+                                     gpointer user_data)
 {
-  if (! get_page_text()) return;
+  if (! get_page_text ())
+     return;
 
   if (! cur_text_doc->b_saved)
      return;
 
-  gchar *f = kwas_get_current_filename ((t_kwas_win*)user_data);
+  gchar *f = kwas_get_current_filename (user_data);
   if (! f)
-     return; 
+     return;
 
-  insert_link (f); 
+  insert_link (f);
   g_free (f);
 }
 
@@ -220,16 +215,17 @@ static void on_mni_kwas_insert_link (GtkMenuItem *menuitem,
 static void on_mni_kwas_insert_image (GtkMenuItem *menuitem,
                                       gpointer user_data)
 {
-  if (! get_page_text()) return;
+  if (! get_page_text ())
+     return;
 
   if (! cur_text_doc->b_saved)
      return;
 
-  gchar *f = kwas_get_current_filename ((t_kwas_win*)user_data);
+  gchar *f = kwas_get_current_filename (user_data);
   if (! f)
-     return; 
+     return;
 
-  insert_image (f); 
+  insert_image (f);
   g_free (f);
 }
 
@@ -241,11 +237,11 @@ static void on_mni_kwas_run_with (GtkMenuItem *menuitem,
   gchar *f = kwas_get_current_filename (w);
 
   if (! f)
-     return; 
+     return;
 
   gchar *t = g_strconcat (gtk_entry_get_text (w->cm_entry), " ", f, "&", NULL);
   system (t);
-  
+
   g_free (f);
   g_free (t);
 }
@@ -256,19 +252,20 @@ static void on_mni_kwas_copy_filename (GtkMenuItem *menuitem,
 {
   GtkClipboard *c = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
   if (! c)
-     return; 
+     return;
 
   gchar *f = kwas_get_current_filename (user_data);
 
   if (! f)
-     return; 
+     return;
 
   gtk_clipboard_set_text (c, f, -1);
-  
+
   g_free (f);
 }
 
 
+/*
 static void on_mni_kwas_add_bookmark (GtkMenuItem *menuitem,
                                       gpointer user_data)
 {
@@ -280,6 +277,8 @@ static void on_mni_kwas_add_bookmark (GtkMenuItem *menuitem,
   gl_tea_kwas_bookmarks = load_file_to_glist (confile.tea_kwas_bookmarks);
   kwas_bookmarks_menu_update (w);
 }
+*/
+
 
 //I am not sure - maybe better to move it into destroy_event?
 static gboolean on_kwas_window_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
@@ -289,7 +288,7 @@ static gboolean on_kwas_window_delete_event (GtkWidget *widget, GdkEvent *event,
 }
 
 
-static void kwas_log_to_memo (t_kwas_win *win, gchar *m1, gchar* m2, gint mode)
+static void kwas_log_to_memo (t_kwas_win *win, const gchar *m1, const gchar* m2, gint mode)
 {
   if (! win->log_memo)
      return;
@@ -301,7 +300,7 @@ static void kwas_log_to_memo (t_kwas_win *win, gchar *m1, gchar* m2, gint mode)
      {
       win->log_to_memo_counter = 0;
       GtkTextIter itstart, itend;
-      gtk_text_buffer_get_bounds (b, &itstart, &itend);   
+      gtk_text_buffer_get_bounds (b, &itstart, &itend);
       gtk_text_buffer_delete (b, &itstart, &itend);
      }
 
@@ -333,23 +332,23 @@ static void kwas_log_to_memo (t_kwas_win *win, gchar *m1, gchar* m2, gint mode)
       g_free (st);
       g_free (prefix);
       logmemo_set_pos (win->log_memo, 0);
- 
+
       return;
      }
- 
+
   gulong len = strlen (m1) + strlen (m2) + 2024;
   gchar *message = g_malloc (len);
 
   g_snprintf (message, len, m1, m2);
   gchar *s = g_strconcat (prefix, message, "\n", NULL);
-  
+
   gtk_text_buffer_insert_at_cursor (b, s, -1);
 
   g_free (message);
   g_free (s);
   g_free (prefix);
   logmemo_set_pos (win->log_memo, 0);
-  
+
   return;
 }
 
@@ -411,27 +410,28 @@ static struct stat* get_stat (const gchar *filename)
   return s;
 }
 
+
 //n.p. Led Zeppelin "Immigrants Song"
-static gchar* path_up (gchar *filename) 
+static gchar* path_up (const gchar *filename)
 {
    gint c = 0;
    gint i = 0;
-   
+
    while (filename[i++])
-         if (filename[i] == '/')
+         if (filename[i] == G_DIR_SEPARATOR)
             c = i;
-  
-   if (c == 0) 
-      return g_strdup ("/");
-   else  
-       return g_strndup (filename, c); 
+
+   if (c == 0)
+      return g_strdup (G_DIR_SEPARATOR_S);
+   else
+       return g_strndup (filename, c);
 }
 
 
-static void statusbar_message (t_kwas_win *w, gchar *s)
+static void statusbar_message (t_kwas_win *w, const gchar *s)
 {
   if (! w->status_bar)
-     return;  
+     return;
 
   if (s)
      gtk_statusbar_push (w->status_bar,
@@ -446,7 +446,7 @@ static void wnd_caption (t_kwas_win *w, gchar *s)
      return;
 
   if (s)
-     gtk_window_set_title (w->window, s);   
+     gtk_window_set_title (w->window, s);
 }
 
 
@@ -454,7 +454,7 @@ static void filelist_add_new_item (t_kwas_panel *p, const gchar *s)
 {
   if (! s)
      return;
-  
+
   GtkTreeModel *model;
   GtkTreeIter newrow;
   t_flist_item *t = g_hash_table_lookup (p->hash_filelist, s);
@@ -468,19 +468,47 @@ static void filelist_add_new_item (t_kwas_panel *p, const gchar *s)
 }
 
 
-static t_flist_item* fli_new (gchar *path, gchar *filename)
+static t_flist_item* fli_new (const gchar *path, const gchar *filename)
 {
   t_flist_item* tfl = g_malloc (sizeof (t_flist_item));
-  
-  tfl->path = g_strconcat (path, "/", filename, NULL);  
+
+  tfl->path = g_strconcat (path, G_DIR_SEPARATOR_S, filename, NULL);
   tfl->fstat = get_stat (tfl->path);
 
+#ifndef CHAI_MODE
+
+
+#if defined(DARWIN)
+  if (sizeof (off_t) == 8)
+      tfl->file_size = g_strdup_printf ("%-10llu", tfl->fstat->st_size);
+  else
+      tfl->file_size = g_strdup_printf ("%-10lu"), (tfl->fstat->st_size);
+#else
+
   gint size = tfl->fstat->st_size;
-  
+
   if (size < 1024)
-      tfl->file_size = g_strdup_printf ("%-10d", tfl->fstat->st_size);
-   else
-        tfl->file_size = g_strdup_printf ("%-10d Kb", (tfl->fstat->st_size / 1024)); 
+     tfl->file_size = g_strdup_printf ("%-10d", tfl->fstat->st_size);
+  else
+     tfl->file_size = g_strdup_printf (_("%-10d Kb"), (tfl->fstat->st_size / 1024));
+#endif
+
+
+#else
+
+  GnomeVFSFileInfo *info;
+  GnomeVFSResult res;
+  info = gnome_vfs_file_info_new ();
+  res = gnome_vfs_get_file_info (tfl->path, info, (GNOME_VFS_FILE_INFO_DEFAULT | GNOME_VFS_FILE_INFO_FOLLOW_LINKS));
+
+  if (res == GNOME_VFS_OK)
+     tfl->file_size = gnome_vfs_format_file_size_for_display (info->size);
+  else
+      tfl->file_size = g_strdup ("?");
+
+  gnome_vfs_file_info_unref (info);
+
+#endif
 
   struct tm *lt = localtime (&tfl->fstat->st_mtime);
 
@@ -502,12 +530,12 @@ static void fli_free (t_flist_item* fli)
 }
 
 
-void fill_filelist (t_kwas_panel *p, gchar *path)
+void fill_filelist (t_kwas_panel *p, const gchar *path)
 {
   if (! p->tree_view)
      return;
 
-  GtkListStore *liststore = gtk_tree_view_get_model (GTK_TREE_VIEW (p->tree_view));    
+  GtkListStore *liststore = gtk_tree_view_get_model (GTK_TREE_VIEW (p->tree_view));
 
   if (liststore)
      gtk_list_store_clear (liststore);
@@ -515,7 +543,7 @@ void fill_filelist (t_kwas_panel *p, gchar *path)
   GDir *dir = g_dir_open (path, 0, NULL);
 
   if (! dir)
-     return; 
+     return;
 
   GList *l_dirs = NULL;
   GList *l_files = NULL;
@@ -530,22 +558,22 @@ void fill_filelist (t_kwas_panel *p, gchar *path)
   if (p->hash_filelist)
      g_hash_table_destroy (p->hash_filelist);
 
-  p->hash_filelist = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, fli_free);  
-  
+  p->hash_filelist = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, fli_free);
+
   while (t = g_dir_read_name (dir))
         {
-         if (! confile.show_hidden_files) 
+         if (! confile.show_hidden_files)
          if (t[0] == '.')
-            continue; 
-       
-         z = g_strconcat (path, "/", t, NULL); //full localized path
-         
-         y = get_8_filename (t); //filename in UTF-8        
-  
-         if (g_file_test (z, G_FILE_TEST_IS_DIR))              
+            continue;
+
+         z = g_strconcat (path, G_DIR_SEPARATOR_S, t, NULL); //full localized path
+
+         y = get_8_filename (t); //filename in UTF-8
+
+         if (g_file_test (z, G_FILE_TEST_IS_DIR))
             {
-             x = g_strconcat ("/", y, NULL); //dirname in utf-8 
-         
+             x = g_strconcat (G_DIR_SEPARATOR_S, y, NULL); //dirname in utf-8
+
              l_dirs = g_list_prepend (l_dirs, x);
              g_hash_table_insert (p->hash_filelist,
                                   g_strdup (x),
@@ -553,17 +581,16 @@ void fill_filelist (t_kwas_panel *p, gchar *path)
 
              g_free (y);
             }
-         else 
+         else
              {
               l_files = g_list_prepend (l_files, y);
               g_hash_table_insert  (p->hash_filelist,
                                     g_strdup (y),
                                     fli_new (path, t));
-              } 
+              }
 
         g_free (z);
-       }   
-
+       }
 
   l_dirs = g_list_prepend (sort_list_case_insensetive (l_dirs), g_strdup (".."));
   g_hash_table_insert (p->hash_filelist,
@@ -571,17 +598,17 @@ void fill_filelist (t_kwas_panel *p, gchar *path)
                        fli_new (path, ".."));
 
   GList *tl = g_list_first (l_dirs);
-  
+
   while (tl)
         {
-         filelist_add_new_item (p, tl->data);	
+         filelist_add_new_item (p, tl->data);
          tl = g_list_next (tl);
         }
 
   tl = g_list_first (sort_list_case_insensetive (l_files));
   while (tl)
         {
-         filelist_add_new_item (p, tl->data);	
+         filelist_add_new_item (p, tl->data);
          tl = g_list_next (tl);
         }
 
@@ -593,7 +620,7 @@ void fill_filelist (t_kwas_panel *p, gchar *path)
   wnd_caption (p->kwas_win, p->path);
   statusbar_message (p->kwas_win, p->path);
 
-  gtk_widget_grab_focus (p->tree_view);    
+  gtk_widget_grab_focus (p->tree_view);
 }
 
 
@@ -606,32 +633,32 @@ static void dir_up (t_kwas_panel *p)
 }
 
 
-static void handle_enter (t_kwas_panel *p, gchar *f)
+static void handle_enter (t_kwas_panel *p, const gchar *f)
 {
   t_flist_item *ti = g_hash_table_lookup (p->hash_filelist, f);
-  t_kwas_win *w = p->kwas_win;  
+  t_kwas_win *w = p->kwas_win;
 
   if (! ti)
-     return;  
+     return;
 
-  if (strstr (f, ".."))  
+  if (strstr (f, ".."))
      {
       dir_up (p);
       return;
      }
-    
+
   if (S_ISDIR (ti->fstat->st_mode))
      {
-      if (access (ti->path, R_OK) == -1) 
-         return;         
+      if (access (ti->path, R_OK) == -1)
+         return;
 
       p->path = g_strdup (ti->path);
-      fill_filelist (p, p->path);       
+      fill_filelist (p, p->path);
       return;
      }
 
   if (S_ISREG (ti->fstat->st_mode))
-     { 
+     {
       g_free (cur_settings.selected_enc);
       cur_settings.selected_enc = g_strdup (g_list_nth_data (confile.iconv_encs, gtk_combo_box_get_active (w->cb_charset)));
       handle_file (ti->path, 0, TRUE);
@@ -647,33 +674,33 @@ static gboolean on_treeview_key_press_event (GtkWidget *widget, GdkEventKey *eve
 
   if (event->keyval == GDK_Tab)
      {
-      gtk_widget_grab_focus (op->tree_view);     
+      gtk_widget_grab_focus (op->tree_view);
       return TRUE;
-     } 
+     }
 
   if (event->keyval == GDK_BackSpace)
      {
       dir_up (user_data);
       return TRUE;
-     } 
+     }
 
   if (event->keyval == GDK_Return)
      {
       GtkTreeSelection *sl = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
       GtkTreeModel *model;
-      GtkTreeIter iter; 
+      GtkTreeIter iter;
 
       if (gtk_tree_selection_get_selected (sl, &model, &iter))
          {
           gchar *name;
-          
+
           gtk_tree_model_get (model, &iter, COL_TEXT, &name, -1);
 
-          if (! name)            
+          if (! name)
              return FALSE;
- 
+
           handle_enter (user_data, name);
-    
+
           g_free (name);
           return TRUE;
          }
@@ -695,13 +722,12 @@ static void cell_data_func (GtkTreeViewColumn *col,
      return;
 
   if (! p->hash_filelist)
-     return; 
+     return;
 
   gchar *c1;
   gchar *c2;
   gchar *c3;
   gchar *c4;
-  
   gchar *st;
 
   gtk_tree_model_get (model, iter, COL_TEXT, &c1, -1);
@@ -709,17 +735,17 @@ static void cell_data_func (GtkTreeViewColumn *col,
   t_flist_item *t = g_hash_table_lookup (p->hash_filelist, c1);
 
   if (! t)
-     return; 
+     return;
 
   if (S_ISDIR (t->fstat->st_mode))
       g_object_set (renderer, "weight", PANGO_WEIGHT_BOLD, "weight-set", TRUE, NULL);
   else
-      g_object_set (renderer, "weight", PANGO_WEIGHT_NORMAL, "weight-set", TRUE, NULL); 
+      g_object_set (renderer, "weight", PANGO_WEIGHT_NORMAL, "weight-set", TRUE, NULL);
 
 }
 
 
-static gint widget_get_y (GtkWidget *widget, gchar *s) 
+static gint widget_get_y (GtkWidget *widget, const gchar *s)
 {
   PangoLayout *l;
   gint result = -1;
@@ -735,30 +761,29 @@ static gint widget_get_y (GtkWidget *widget, gchar *s)
 
 static gint button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
-  if (event->button == 1) 
+  if (event->button == 1)
      if (event->type == GDK_2BUTTON_PRESS)
         {
          GtkTreeSelection *sl = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
          GtkTreeModel *model;
-         GtkTreeIter iter; 
+         GtkTreeIter iter;
 
          if (gtk_tree_selection_get_selected (sl, &model, &iter))
             {
              gchar *name;
-             
              gtk_tree_model_get (model, &iter, COL_TEXT, &name, -1);
 
-             if (! name)            
+             if (! name)
                return FALSE;
- 
+
              handle_enter (user_data, name);
 
              g_free (name);
              return TRUE;
             }
-         
-      }  
-  return FALSE; 
+
+        }
+  return FALSE;
 }
 
 
@@ -767,7 +792,7 @@ void free_kwas_panel (t_kwas_panel *p)
    g_hash_table_destroy (p->hash_filelist);
    g_free (p->path);
    g_free (p);
-   p = NULL; 
+   p = NULL;
 }
 
 
@@ -794,16 +819,16 @@ static gboolean on_panel_focus (GtkWidget *widget,
 t_kwas_panel* create_filenav_panel (t_kwas_win *w)
 {
   t_kwas_panel *p = g_malloc (sizeof (t_kwas_panel));
-  p->hash_filelist = NULL; 
+  p->hash_filelist = NULL;
   p->kwas_win = w;
   p->path = NULL;
 
-  gchar *fn = g_malloc (4096); 
+  gchar *fn = g_malloc (4096);
   gchar *cur_dir = g_get_current_dir ();
 
   if (cur_dir)
   if (realpath (cur_dir, fn))
-     p->path = fn;	
+     p->path = fn;
 
   g_free (cur_dir);
 
@@ -813,11 +838,10 @@ t_kwas_panel* create_filenav_panel (t_kwas_win *w)
   GtkListStore *list_store;
 
   list_store = gtk_list_store_new (NUM_COLS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING); /* NUM_COLS = 1 */
-  
   p->tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list_store));
-
   renderer = gtk_cell_renderer_text_new ();
-/*      
+
+/*
   g_object_set(renderer,
                "cell-background", "navy",
                "cell-background-set", TRUE,
@@ -830,8 +854,8 @@ t_kwas_panel* create_filenav_panel (t_kwas_win *w)
 */
 
   g_object_set (renderer, "height", widget_get_y (GTK_WIDGET (p->tree_view), "ZORROTEST") + 5, NULL);
-   
-  col = gtk_tree_view_column_new();
+
+  col = gtk_tree_view_column_new ();
 
   gtk_tree_view_column_pack_start (col, renderer, TRUE);
   gtk_tree_view_column_add_attribute (col, renderer, "text", COL_TEXT);
@@ -860,15 +884,16 @@ t_kwas_panel* create_filenav_panel (t_kwas_win *w)
   sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (p->tree_view));
 
   gtk_tree_selection_set_mode (sel, GTK_SELECTION_SINGLE);
+  //gtk_tree_selection_set_mode (sel, GTK_SELECTION_MULTIPLE);
 
 //  g_object_set(renderer, "foreground", "white", "foreground-set", TRUE, NULL);
-//  g_object_set(renderer, "background", "navy", "background-set", TRUE, NULL); 
-  
+//  g_object_set(renderer, "background", "navy", "background-set", TRUE, NULL);
+
   g_signal_connect ((gpointer) p->tree_view, "key_press_event",
                     G_CALLBACK (on_treeview_key_press_event),
                     p);
 
- g_signal_connect (GTK_OBJECT (p->tree_view), "button_press_event",
+  g_signal_connect (GTK_OBJECT (p->tree_view), "button_press_event",
                      (GtkSignalFunc) button_press_event, p);
 
   g_signal_connect ((gpointer) p->tree_view, "focus-in-event",
@@ -885,32 +910,32 @@ gchar* kwas_get_current_filename (t_kwas_win *win)
 {
   t_kwas_panel *p = win->focused_panel;
   if (! p)
-     return NULL;  
+     return NULL;
 
   GtkTreeSelection *sl = gtk_tree_view_get_selection (GTK_TREE_VIEW (p->tree_view));
   GtkTreeModel *model;
-  GtkTreeIter iter; 
+  GtkTreeIter iter;
 
   if (gtk_tree_selection_get_selected (sl, &model, &iter))
      {
       gchar *name;
-  
+
       gtk_tree_model_get (model, &iter, COL_TEXT, &name, -1);
 
-      if (! name)            
+      if (! name)
          return FALSE;
- 
+
       t_flist_item *ti = g_hash_table_lookup (p->hash_filelist, name);
-   
+
       if (! ti)
-         return NULL;  
- 
+         return NULL;
+
       if (S_ISREG (ti->fstat->st_mode))
-         { 
-          g_free (name); 
+         {
+          g_free (name);
           return g_strdup (ti->path);
-         }   
-    
+         }
+
       g_free (name);
       return NULL;
      }
@@ -928,7 +953,7 @@ t_kwas_win* create_kwas_window (void)
   GtkWidget *mn_kwas_menu;
   GtkWidget *mn_kwas_nav_menu;
   GtkWidget *mn_kwas_ins_menu;
-  
+
   GtkWidget *item1;
   GtkWidget *item2;
   GtkWidget *tb_toolbar;
@@ -954,7 +979,7 @@ t_kwas_win* create_kwas_window (void)
 
   g_signal_connect (G_OBJECT (w->window), "key_press_event", G_CALLBACK (win_key_handler), w->window);
 
-  w->status_bar = NULL; 
+  w->status_bar = NULL;
 
   vpaned1 = gtk_vpaned_new ();
   gtk_widget_show (vpaned1);
@@ -976,7 +1001,7 @@ t_kwas_win* create_kwas_window (void)
   gtk_widget_add_accelerator (mni_temp, "activate", accel_group,
                               GDK_R, GDK_CONTROL_MASK,
                               GTK_ACCEL_VISIBLE);
-   
+
   mni_temp = new_menu_item_with_udata (_("Get file info"), kwas_mn_functions, on_mni_kwas_get_file_info, w);
 
   mni_temp = new_menu_item_with_udata (_("Run with..."), kwas_mn_functions, on_mni_kwas_run_with, w);
@@ -991,12 +1016,12 @@ t_kwas_win* create_kwas_window (void)
   mni_temp = new_menu_tof (mn_kwas_nav_menu);
 
   mni_temp = new_menu_item_with_udata (_("Go to the home directory"), mn_kwas_nav_menu, on_mni_kwas_goto_homedir, w);
-  
+
 
   w->mni_bookmarks = new_menu_item (_("Bookmarks"), mn_kwas_menu, NULL);
   w->mni_bookmarks_menu = new_menu_submenu (w->mni_bookmarks);
 
-  kwas_bookmarks_menu_update (w);  
+  kwas_bookmarks_menu_update (w);
 
   mni_temp = new_menu_item (_("Ins to editor"), mn_kwas_menu, NULL);
   mn_kwas_ins_menu = new_menu_submenu (mni_temp);
@@ -1007,7 +1032,7 @@ t_kwas_win* create_kwas_window (void)
   GtkWidget *hbox_top = gtk_hbox_new (FALSE, 0);
   gtk_widget_show (hbox_top);
   gtk_box_pack_start (GTK_BOX (vbox1), hbox_top, FALSE, FALSE, UI_PACKFACTOR);
-   
+
   w->cb_charset = gtk_combo_box_new_text ();
   fill_combo_with_glist (w->cb_charset, confile.iconv_encs);
 
@@ -1015,7 +1040,7 @@ t_kwas_win* create_kwas_window (void)
   if (i == -1)
      i = 0;
 
-  gtk_combo_box_set_active (w->cb_charset, i); 
+  gtk_combo_box_set_active (w->cb_charset, i);
   gtk_widget_show (w->cb_charset);
   gtk_box_pack_start (GTK_BOX (hbox_top), w->cb_charset, FALSE, FALSE, UI_PACKFACTOR);
 
@@ -1057,7 +1082,7 @@ t_kwas_win* create_kwas_window (void)
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (w->log_memo), GTK_WRAP_WORD);
   gtk_widget_show (w->log_memo);
   gtk_container_add (GTK_CONTAINER (scrolledwindow3), w->log_memo);
-  
+
   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_widget_show (hbox1);
   gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, TRUE, UI_PACKFACTOR);
@@ -1087,15 +1112,15 @@ t_kwas_win* create_kwas_window (void)
 
   w->p1->opposite_panel = w->p2;
   w->p2->opposite_panel = w->p1;
-  w->focused_panel = w->p1;  
+  w->focused_panel = w->p1;
 
   width = get_value (confile.screen_w, 80);
   height = get_value (confile.screen_h, 80);
 
-  gtk_window_resize (GTK_WINDOW (w->window), width, height);     
-  
-  gtk_paned_set_position (vpaned1, get_value (height, 85)); 
-  gtk_paned_set_position (hpaned1, get_value (width, 50)); 
+  gtk_window_resize (GTK_WINDOW (w->window), width, height);
+
+  gtk_paned_set_position (vpaned1, get_value (height, 85));
+  gtk_paned_set_position (hpaned1, get_value (width, 50));
 
   GtkTextBuffer *buf = gtk_text_view_get_buffer (w->log_memo);
 
@@ -1118,15 +1143,14 @@ t_kwas_win* create_kwas_window (void)
   g_signal_connect ((gpointer) w->cm_entry, "key_press_event",
                     G_CALLBACK (on_ent_nav_key_press_event),
                     w);
-  
+
   gtk_window_add_accel_group (GTK_WINDOW (w->window), a);
   gtk_widget_show (w->window);
 
-  gtk_widget_grab_focus (w->p1->tree_view);    
+  gtk_widget_grab_focus (w->p1->tree_view);
 
   w->log_to_memo_counter = 0;
   w->msg_counter = 0;
   return w;
 }
 
- 
