@@ -1,20 +1,20 @@
-typedef struct 
+typedef struct
               {
                GHashTable* hash_filelist;
                gchar *path;
                GtkWidget *tree_view;
                gpointer *opposite_panel;
                gpointer *kwas_win;
-              } 
+              }
                t_kwas_panel;
 
 
-typedef struct 
+typedef struct
               {
                t_kwas_panel *p1;
                t_kwas_panel *p2;
                t_kwas_panel *focused_panel;
-               GtkWindow *window;   
+               GtkWindow *window;
                GtkWidget *cm_entry;
                GtkComboBox *cb_charset;
                GtkWidget *status_bar;
@@ -23,11 +23,10 @@ typedef struct
                GtkWidget *mni_bookmarks_menu;
                gint log_to_memo_counter;
                gint msg_counter;
-              } 
+              }
                t_kwas_win;
 
 
-
 GtkWidget *kwas_mn_functions;
 GtkWidget *kwas_mn_boomarks;
 GtkAccelGroup *kwas_accel_group;
@@ -37,5 +36,5 @@ gchar* kwas_get_current_filename (t_kwas_win *win);
 void free_kwas_panel (t_kwas_panel *p);
 void free_kwas_window (t_kwas_win *w);
 t_kwas_panel* get_current_panel (t_kwas_win *win);
-void fill_filelist (t_kwas_panel *p, gchar *path);
+void fill_filelist (t_kwas_panel *p, const gchar *path);
 
