@@ -2,7 +2,8 @@
                           tea_options.c  -  description
                              -------------------
     begin                : aug. 2004
-    copyright            : (C) 2004, 2005 by Peter 'Roxton' Semiletov
+    copyright            : (C) 2004-2006 by Peter 'Roxton' Semiletov,
+                          Michele Garoche
     email                : peter.semiletov@gmail.com
  ***************************************************************************/
 
@@ -46,8 +47,13 @@
 #include "tea_defs.h"
 #include "tea_options.h"
 #include "tea_gtk_utils.h"
+#include "tea_text_document.h" // Needs to be above for t_note_page struct
 #include "tea_config.h"
-#include "tea_text_document.h"
+#include "rox_strings.h" // Needed for glist_save_to_file
+#include "free_desktop_support.h" // Needed for free_desktop_delete_shortcut
+#include "tea_tree_view.h" // Needed for tv_fill_with_glist
+#include "interface.h" // Needed for ui_update
+#include "tea_funx.h" // Needed for copy_file
 
 
 
@@ -82,15 +88,15 @@ static void cb_tv_available_hlangs_change (GtkComboBox *combobox,
   gint index = gtk_combo_box_get_active (combobox);
   if (index == -1)
      return;
-    
+
   GSList *list = g_slist_nth (avail_langs, index);
   if (! list)
-    return;  
+    return;
 
   cur_language = list->data;
 
  // dbm (gtk_source_language_get_name (cur_language));
- 
+
   gchar *name;
 
   if (gsl_tags_list)
@@ -98,7 +104,7 @@ static void cb_tv_available_hlangs_change (GtkComboBox *combobox,
       g_slist_foreach (gsl_tags_list, (GFunc)g_object_unref, NULL);
       g_slist_free (gsl_tags_list);
      }
-  
+
   gsl_tags_list = gtk_source_language_get_tags (GTK_SOURCE_LANGUAGE (cur_language));
 
   cur_tag = gsl_tags_list->data;
@@ -112,7 +118,7 @@ static void cb_tv_available_hlangs_change (GtkComboBox *combobox,
          g_object_get (gsl->data, "name", &name, NULL);
          gl_tags_list = g_list_append (gl_tags_list, name);
          gsl = g_slist_next (gsl);
-        } 
+        }
 
   gtk_list_store_clear (gtk_combo_box_get_model (tv_hlang_elements));
   fill_combo_with_glist (tv_hlang_elements, gl_tags_list);
@@ -148,13 +154,13 @@ static void cb_tv_hlang_elements_change (GtkComboBox *combobox,
 }
 
 
-static void cb_select_tag_color (GtkColorButton *widget, gpointer user_data) 
+static void cb_select_tag_color (GtkColorButton *widget, gpointer user_data)
 {
   GdkColor color;
-  gtk_color_button_get_color (widget, &color);  
+  gtk_color_button_get_color (widget, &color);
   cur_tag_style->foreground = color;
 
-  gchar *id;   
+  gchar *id;
   g_object_get (cur_tag, "id", &id, NULL);
 
   gtk_source_language_set_tag_style (cur_language, id, cur_tag_style);
@@ -167,8 +173,8 @@ static void cb_select_tag_color (GtkColorButton *widget, gpointer user_data)
 static void cb_hl_italic_toggle (GtkToggleButton *togglebutton, gpointer user_data)
 {
   cur_tag_style->italic = gtk_toggle_button_get_active (togglebutton);
-  
-  gchar *id;   
+
+  gchar *id;
   g_object_get (cur_tag, "id", &id, NULL);
 
   gtk_source_language_set_tag_style (cur_language, id, cur_tag_style);
@@ -181,8 +187,8 @@ static void cb_hl_italic_toggle (GtkToggleButton *togglebutton, gpointer user_da
 static void cb_hl_bold_toggle (GtkToggleButton *togglebutton, gpointer user_data)
 {
   cur_tag_style->bold = gtk_toggle_button_get_active (togglebutton);
-  
-  gchar *id;   
+
+  gchar *id;
   g_object_get (cur_tag, "id", &id, NULL);
 
   gtk_source_language_set_tag_style (cur_language, id, cur_tag_style);
@@ -196,7 +202,7 @@ static void cb_hl_underline_toggle (GtkToggleButton *togglebutton, gpointer user
 {
   cur_tag_style->underline = gtk_toggle_button_get_active (togglebutton);
 
-  gchar *id;   
+  gchar *id;
   g_object_get (cur_tag, "id", &id, NULL);
 
   gtk_source_language_set_tag_style (cur_language, id, cur_tag_style);
@@ -211,7 +217,7 @@ static void cb_hl_strikethrough_toggle (GtkToggleButton *togglebutton, gpointer
   cur_tag_style->strikethrough = gtk_toggle_button_get_active (togglebutton);
   gtk_source_tag_set_style (cur_tag, cur_tag_style);
 
-  gchar *id;   
+  gchar *id;
   g_object_get (cur_tag, "id", &id, NULL);
 
   gtk_source_language_set_tag_style (cur_language, id, cur_tag_style);
@@ -225,7 +231,7 @@ static void cb_hl_strikethrough_toggle (GtkToggleButton *togglebutton, gpointer
 static GtkWidget* wnd_options_hl_chai (void)
 {
   avail_langs = gtk_source_languages_manager_get_available_languages (source_languages_manager);
-  
+
   cur_tag = NULL;
   gsl_tags_list = NULL;
   gl_tags_list = NULL;
@@ -236,36 +242,34 @@ static GtkWidget* wnd_options_hl_chai (void)
   gtk_widget_show (page);
 
   GtkWidget *hb = gtk_hbox_new (FALSE, UI_PACKFACTOR);
-  
+
   gtk_widget_show (hb);
   gtk_box_pack_start (page, hb, FALSE, FALSE, UI_PACKFACTOR);
 
 
-  tv_available_hlangs = tea_combo_combo (page, gl_supported_hlangs, _("Available highlighting modes"), 0); 
-  
+  tv_available_hlangs = tea_combo_combo (page, gl_supported_hlangs, _("Available highlighting modes"), 0);
+
   g_signal_connect ((gpointer) tv_available_hlangs, "changed",
                     G_CALLBACK (cb_tv_available_hlangs_change),
                     NULL);
 
-
-  tv_hlang_elements = tea_combo_combo (page, NULL, _("Elements"), 0); 
+  tv_hlang_elements = tea_combo_combo (page, NULL, _("Elements"), 0);
   g_signal_connect ((gpointer) tv_hlang_elements, "changed",
                     G_CALLBACK (cb_tv_hlang_elements_change),
                     NULL);
-  
 
-  bt_tag_foreground = tea_color_selector_simple (page, _("Element foreground"));  
+  bt_tag_foreground = tea_color_selector_simple (page, _("Element foreground"));
   g_signal_connect (bt_tag_foreground, "color-set", G_CALLBACK (cb_select_tag_color), NULL);
 
-  cb_hl_italic = tea_checkbox (page, _("Italic"), FALSE); 
-  cb_hl_bold = tea_checkbox (page, _("Bold"), FALSE); 
-  cb_hl_underline = tea_checkbox (page, _("Underline"), FALSE); 
-  cb_hl_strikethrough = tea_checkbox (page, _("Strikethrough"), FALSE); 
+  cb_hl_italic = tea_checkbox (page, _("Italic"), FALSE);
+  cb_hl_bold = tea_checkbox (page, _("Bold"), FALSE);
+  cb_hl_underline = tea_checkbox (page, _("Underline"), FALSE);
+  cb_hl_strikethrough = tea_checkbox (page, _("Strikethrough"), FALSE);
 
   g_signal_connect ((gpointer) &cb_hl_italic->toggle_button, "clicked",
                     G_CALLBACK (cb_hl_italic_toggle),
                     NULL);
- 
+
   g_signal_connect ((gpointer) &cb_hl_bold->toggle_button, "clicked", //"toggled"
                     G_CALLBACK (cb_hl_bold_toggle),
                     NULL);
@@ -313,7 +317,7 @@ static void cb_on_choose_enc (GtkButton *button,
   gl_enc_choosed = g_list_prepend (gl_enc_choosed, g_strdup (s));
   tv_fill_with_glist (tv_choosed_charsets, gl_enc_choosed);
 
-  glist_save_to_file (gl_enc_choosed, confile.tea_choosed_charsets); 
+  glist_save_to_file (gl_enc_choosed, confile.tea_choosed_charsets);
   get_iconv_sup ();
 }
 
@@ -327,10 +331,14 @@ static void cb_on_del_enc (GtkButton *button,
 
   GList *l = str_remove_from_list (gl_enc_choosed, s);
 
-  glist_strings_free (gl_enc_choosed);
-  gl_enc_choosed = l;
+  // Need to test whether the list is empty or not
+  // If it is empty, remove the charsets file in home/.config/tea
+  // If it is not empty, save the new charsets set
+  if (g_list_length (l) == 0)
+     unlink (confile.tea_choosed_charsets);
+  else
+    glist_save_to_file (l, confile.tea_choosed_charsets);
 
-  glist_save_to_file (gl_enc_choosed, confile.tea_choosed_charsets); 
   get_iconv_sup ();
   tv_fill_with_glist (tv_choosed_charsets, gl_enc_choosed);
 }
@@ -341,16 +349,16 @@ static void on_kwas_notebook_switch_page (GtkNotebook *notebook,
                                           guint page_num,
                                           gpointer user_data)
 {
-  confile.last_prefs_tab = page_num;      
+  confile.last_prefs_tab = page_num;
 }
 
 
-static void cb_on_bt_apply (GtkObject *object, GtkWidget *w) 
+static void cb_on_bt_apply (GtkObject *object, GtkWidget *w)
 {
    GList *l = NULL;
 
    l = add_to_glist_combined_str (l, "fmt_color_function", gtk_entry_get_text (GTK_ENTRY (ent_fmt_color_function)));
-   l = add_to_glist_combined_str (l, "default_markup_mode", gtk_entry_get_text (GTK_ENTRY (ent_def_markup_mode)));
+   //l = add_to_glist_combined_str (l, "default_markup_mode", gtk_entry_get_text (GTK_ENTRY (ent_def_markup_mode)));
 
    l = add_to_glist_combined_str (l, "def_save_dir", gtk_entry_get_text (GTK_ENTRY (ent_def_save_dir)));
    l = add_to_glist_combined_str (l, "def_open_dir", gtk_entry_get_text (GTK_ENTRY (ent_def_open_dir)));
@@ -377,9 +385,12 @@ static void cb_on_bt_apply (GtkObject *object, GtkWidget *w)
    l = add_to_glist_combined_str (l, "tag_current_line_bg", gtk_entry_get_text (GTK_ENTRY (ent_tag_current_line_bg)));
 
    l = add_to_glist_combined_int (l, "margin_visible", gtk_toggle_button_get_active (&cb_margin_visible->toggle_button));
-   l = add_to_glist_combined_int (l, "margin_width", gtk_spin_button_get_value_as_int (sb_margin_width)); 
+   l = add_to_glist_combined_int (l, "margin_width", gtk_spin_button_get_value_as_int (sb_margin_width));
+   l = add_to_glist_combined_int (l, "max_undo_levels", gtk_spin_button_get_value_as_int (sb_max_undo_levels));
 
-   l = add_to_glist_combined_str (l, "color_cursor", gtk_entry_get_text (GTK_ENTRY (ent_color_cursor)));
+
+
+   //l = add_to_glist_combined_str (l, "color_cursor", gtk_entry_get_text (GTK_ENTRY (ent_color_cursor)));
 
    l = add_to_glist_combined_str (l, "tag_color_paint_fg", gtk_entry_get_text (GTK_ENTRY (ent_tag_color_paint_fg)));
    l = add_to_glist_combined_str (l, "tag_color_paint_bg", gtk_entry_get_text (GTK_ENTRY (ent_tag_color_paint_bg)));
@@ -397,11 +408,10 @@ static void cb_on_bt_apply (GtkObject *object, GtkWidget *w)
 
    l = add_to_glist_combined_str (l, "text_foreground", gtk_entry_get_text (GTK_ENTRY (ent_text_foreground)));
    l = add_to_glist_combined_str (l, "text_background", gtk_entry_get_text (GTK_ENTRY (ent_text_background)));
- 
+
    l = add_to_glist_combined_str (l, "text_sel_foreground", gtk_entry_get_text (GTK_ENTRY (ent_text_sel_foreground)));
    l = add_to_glist_combined_str (l, "text_sel_background", gtk_entry_get_text (GTK_ENTRY (ent_text_sel_background)));
-   l = add_to_glist_combined_str (l, "", gtk_entry_get_text (GTK_ENTRY (ent_color_cursor)));
-
+   l = add_to_glist_combined_str (l, "color_cursor", gtk_entry_get_text (GTK_ENTRY (ent_color_cursor)));
 
 
    l = add_to_glist_combined_str (l, "def_filesave_charset", gtk_entry_get_text (GTK_ENTRY (ent_def_filesave_charset)));
@@ -419,7 +429,7 @@ static void cb_on_bt_apply (GtkObject *object, GtkWidget *w)
 
    l = add_to_glist_combined_str (l, "ext_pic_editor", gtk_entry_get_text (GTK_ENTRY (ent_ext_pic_editor)));
    l = add_to_glist_combined_str (l, "ext_pic_viewer", gtk_entry_get_text (GTK_ENTRY (ent_ext_pic_viewer)));
-   
+
    l = add_to_glist_combined_str (l, "def_file_ext", gtk_entry_get_text (GTK_ENTRY (ent_def_file_ext)));
 
 
@@ -441,18 +451,16 @@ static void cb_on_bt_apply (GtkObject *object, GtkWidget *w)
    l = add_to_glist_combined_int (l, "implane_ins_image", gtk_toggle_button_get_active (&cb_implane_ins_image->toggle_button));
    l = add_to_glist_combined_int (l, "use_cursor_blink", gtk_toggle_button_get_active (&cb_use_cursor_blink->toggle_button));
 
-   l = add_to_glist_combined_int (l, "use_snippets", gtk_toggle_button_get_active (&cb_use_snippets->toggle_button));
    l = add_to_glist_combined_int (l, "use_ext_image_viewer", gtk_toggle_button_get_active (&cb_use_ext_image_viewer->toggle_button));
 
 
-
    l = add_to_glist_combined_int (l, "add_def_file_ext", gtk_toggle_button_get_active (&cb_add_def_file_ext->toggle_button));
-   
 
 
+   l = add_to_glist_combined_int (l, "homeend", gtk_toggle_button_get_active (&cb_homeend->toggle_button));
+   l = add_to_glist_combined_int (l, "show_cursor_pos", gtk_toggle_button_get_active (&cb_show_cursor_pos->toggle_button));
+
 
-    l = add_to_glist_combined_int (l, "homeend", gtk_toggle_button_get_active (&cb_homeend->toggle_button));
-   
    l = add_to_glist_combined_int (l, "do_hl_on_fileopen", gtk_toggle_button_get_active (&cb_do_hl_on_fileopen->toggle_button));
    l = add_to_glist_combined_int (l, "do_backup", gtk_toggle_button_get_active (&cb_do_backup->toggle_button));
 
@@ -491,76 +499,76 @@ static void cb_on_bt_apply (GtkObject *object, GtkWidget *w)
 
 
 
-   l = add_to_glist_combined_int (l, "autosave_interval", gtk_spin_button_get_value_as_int (sb_autosave_interval));  
-   l = add_to_glist_combined_int (l, "logmemo_lines_max", gtk_spin_button_get_value_as_int (sb_logmemo_lines_max));  
-   //l = add_to_glist_combined_int (l, "famous_history_max", gtk_spin_button_get_value_as_int (sb_famous_history_max));  
-   l = add_to_glist_combined_int (l, "max_recent_items", gtk_spin_button_get_value_as_int (sb_max_recent_items));  
+   l = add_to_glist_combined_int (l, "autosave_interval", gtk_spin_button_get_value_as_int (sb_autosave_interval));
+   l = add_to_glist_combined_int (l, "logmemo_lines_max", gtk_spin_button_get_value_as_int (sb_logmemo_lines_max));
+   //l = add_to_glist_combined_int (l, "famous_history_max", gtk_spin_button_get_value_as_int (sb_famous_history_max));
+   l = add_to_glist_combined_int (l, "max_recent_items", gtk_spin_button_get_value_as_int (sb_max_recent_items));
 
-   l = add_to_glist_combined_int (l, "thumb_width", gtk_spin_button_get_value_as_int (sb_thumb_width));  
-   l = add_to_glist_combined_int (l, "thumb_height", gtk_spin_button_get_value_as_int (sb_thumb_height));  
+   l = add_to_glist_combined_int (l, "thumb_width", gtk_spin_button_get_value_as_int (sb_thumb_width));
+   l = add_to_glist_combined_int (l, "thumb_height", gtk_spin_button_get_value_as_int (sb_thumb_height));
+
+   l = add_to_glist_combined_int (l, "saveopen_w", gtk_spin_button_get_value_as_int (sb_saveopen_w));
+   l = add_to_glist_combined_int (l, "saveopen_h", gtk_spin_button_get_value_as_int (sb_saveopen_h));
+   l = add_to_glist_combined_int (l, "tab_size", gtk_spin_button_get_value_as_int (sb_tab_size));
 
-   l = add_to_glist_combined_int (l, "saveopen_w", gtk_spin_button_get_value_as_int (sb_saveopen_w));  
-   l = add_to_glist_combined_int (l, "saveopen_h", gtk_spin_button_get_value_as_int (sb_saveopen_h));  
-   l = add_to_glist_combined_int (l, "tab_size", gtk_spin_button_get_value_as_int (sb_tab_size));  
- 
    glist_save_to_file (l, confile.tea_rc);
    glist_strings_free (l);
+
    confile_reload ();
-//   write_gtkrc (); 
-//   gtk_rc_parse (confile.file_gtkrc);
-//   gtk_rc_reset_styles (gtk_settings_get_for_screen (gdk_screen_get_default ()));
+
    ui_update ();
    doc_update_all ();
+
    update_enc_menu();
 }
 
 
 static GtkWidget* wnd_options_encoding (void)
 {
-  GtkWidget *enc_tabs = gtk_notebook_new ();    
+  GtkWidget *enc_tabs = gtk_notebook_new ();
   gtk_widget_show (enc_tabs);
-  
+
   GtkWidget *wnd = gtk_scrolled_window_new (NULL, NULL);
   gtk_widget_show (wnd);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (wnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
- 
+
   GtkWidget *page = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (page);
   gtk_scrolled_window_add_with_viewport (wnd, page);
 
-  GtkWidget *fr1 = gtk_frame_new (_("Defaults"));   
+  GtkWidget *fr1 = gtk_frame_new (_("Defaults"));
   gtk_widget_show (fr1);
   gtk_box_pack_start (page, fr1, FALSE, FALSE, UI_PACKFACTOR);
 
   GtkWidget *box = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (box);
-  gtk_container_add (fr1, box); 
+  gtk_container_add (fr1, box);
 
   ent_default_charset = tea_combo (box, confile.iconv_encs, _("Default charset for File open"), confile.default_charset);
   ent_def_filesave_charset = tea_combo (box, confile.gl_save_charsets, _("Default charset for File save as"), confile.def_filesave_charset);
-  ent_rtf_enc = tea_text_entry (box, _("Default charset for RTF"), confile.rtf_enc);  
- 
+  ent_rtf_enc = tea_text_entry (box, _("Default charset for RTF"), confile.rtf_enc);
 
-  fr1 = gtk_frame_new (_("Misc"));   
+
+  fr1 = gtk_frame_new (_("Misc"));
   gtk_widget_show (fr1);
   gtk_box_pack_start (box, fr1, FALSE, FALSE, UI_PACKFACTOR);
- 
+
   box = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (box);
-  gtk_container_add (fr1, box); 
+  gtk_container_add (fr1, box);
 
-  cb_det_charset_by_meta = tea_checkbox (box, _("Detect a charset by the [X]HTML meta header"), confile.det_charset_by_meta); 
- 
-  GtkWidget *fr2 = gtk_frame_new (_("Autodetect"));   
+  cb_det_charset_by_meta = tea_checkbox (box, _("Detect a charset by the [X]HTML meta header"), confile.det_charset_by_meta);
+
+  GtkWidget *fr2 = gtk_frame_new (_("Autodetect"));
   gtk_widget_show (fr2);
- 
+
   gtk_box_pack_start (page, fr2, FALSE, FALSE, UI_PACKFACTOR);
-  
+
   box = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (box);
-  gtk_container_add (fr2, box); 
+  gtk_container_add (fr2, box);
 
-  cb_enc_det_cp1251 = tea_checkbox (box, _("CP 1251"), confile.enc_det_cp1251); 
+  cb_enc_det_cp1251 = tea_checkbox (box, _("CP 1251"), confile.enc_det_cp1251);
   cb_enc_det_koi8r = tea_checkbox (box, _("KOI8-R"), confile.enc_det_koi8r);
   cb_enc_det_koi8u = tea_checkbox (box, _("KOI8-U"), confile.enc_det_koi8u);
   cb_enc_det_866 = tea_checkbox (box, _("DOS 866"), confile.enc_det_866);
@@ -573,14 +581,14 @@ static GtkWidget* wnd_options_encoding (void)
   cb_enc_det_portuguese = tea_checkbox (box, _("Portuguese"), confile.enc_det_portuguese);
   cb_enc_det_slovak = tea_checkbox (box, _("Slovak"), confile.enc_det_slovak);
   cb_enc_det_slovenian = tea_checkbox (box, _("Slovenian"), confile.enc_det_slovenian);
-  cb_enc_det_spanish = tea_checkbox (box, _("Spanish"), confile.enc_det_spanish);  
-  cb_enc_det_turkish = tea_checkbox (box, _("Turkish"), confile.enc_det_turkish);  
+  cb_enc_det_spanish = tea_checkbox (box, _("Spanish"), confile.enc_det_spanish);
+  cb_enc_det_turkish = tea_checkbox (box, _("Turkish"), confile.enc_det_turkish);
 
   //current music: Radiohead - Banana Co.
   gtk_notebook_append_page (enc_tabs, wnd, gtk_label_new (_("Management")));
 
   GtkWidget *hb = gtk_hbox_new (FALSE, UI_PACKFACTOR);
-  
+
   gtk_widget_show (hb);
 
   tv_available_charsets = tv_create_framed (hb, _("Available"), GTK_SELECTION_SINGLE);
@@ -588,12 +596,12 @@ static GtkWidget* wnd_options_encoding (void)
 
   GtkVBox *buttons = daisho_add_del (cb_on_choose_enc, cb_on_del_enc);
   gtk_box_pack_start (hb, buttons, FALSE, FALSE, UI_PACKFACTOR);
-  
+
   tv_choosed_charsets = tv_create_framed (hb, _("Selected"), GTK_SELECTION_SINGLE);
   tv_fill_with_glist (tv_choosed_charsets, gl_enc_choosed);
- 
+
   gtk_notebook_append_page (enc_tabs, hb, gtk_label_new (_("Configure charsets")));
-  
+
   return enc_tabs;
 }
 
@@ -607,7 +615,7 @@ static GtkWidget* wnd_options_commands (void)
   gtk_widget_show (fr_misc);
   gtk_container_add (page, fr_misc);
 
-  GtkWidget *b = gtk_vbox_new (FALSE, UI_PACKFACTOR); 
+  GtkWidget *b = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (b);
   gtk_container_add (fr_misc, b);
 
@@ -616,24 +624,24 @@ static GtkWidget* wnd_options_commands (void)
   ent_cm_mplayer =  tea_text_entry (b, _("Run Mplayer with subtitles"), confile.cm_mplayer);
 
   ent_cm_manual_browser = tea_text_entry (b, _("Browser for the manual"), confile.cmd_def_doc_browser);
-  cb_use_def_doc_browser = tea_checkbox (b, _("Use this browser"), confile.use_def_doc_browser); 
+  cb_use_def_doc_browser = tea_checkbox (b, _("Use this browser"), confile.use_def_doc_browser);
 
   GtkWidget *fr_latex = gtk_frame_new (_("LaTeX"));
   gtk_widget_show (fr_latex);
   gtk_container_add (page, fr_latex);
 
-  b = gtk_vbox_new (FALSE, UI_PACKFACTOR); 
+  b = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (b);
   gtk_container_add (fr_latex, b);
-  
-  ent_cm_run_latex = tea_text_entry (b, _("Process with LaTeX"), confile.cm_run_latex); 
-  ent_cm_pdflatex = tea_text_entry (b, _("Process with pdflatex"), confile.cm_pdflatex); 
-  ent_cm_dvi_to_pdf = tea_text_entry (b, _("dvi to PDF"), confile.cm_dvi_to_pdf); 
-  ent_cm_dvi_to_ps = tea_text_entry (b, _("dvi to PS"), confile.cm_dvi_to_ps); 
-  ent_cm_dvi_view = tea_text_entry (b, _("View dvi"), confile.cm_dvi_view); 
-  ent_cm_pdf_view = tea_text_entry (b, _("View PDF"), confile.cm_pdf_view); 
-  ent_cm_ps_view = tea_text_entry (b, _("View PS"), confile.cm_ps_view); 
-  
+
+  ent_cm_run_latex = tea_text_entry (b, _("Process with LaTeX"), confile.cm_run_latex);
+  ent_cm_pdflatex = tea_text_entry (b, _("Process with pdflatex"), confile.cm_pdflatex);
+  ent_cm_dvi_to_pdf = tea_text_entry (b, _("dvi to PDF"), confile.cm_dvi_to_pdf);
+  ent_cm_dvi_to_ps = tea_text_entry (b, _("dvi to PS"), confile.cm_dvi_to_ps);
+  ent_cm_dvi_view = tea_text_entry (b, _("View dvi"), confile.cm_dvi_view);
+  ent_cm_pdf_view = tea_text_entry (b, _("View PDF"), confile.cm_pdf_view);
+  ent_cm_ps_view = tea_text_entry (b, _("View PS"), confile.cm_ps_view);
+
   return page;
 }
 
@@ -644,9 +652,9 @@ static GtkWidget* wnd_options_limits (void)
   gtk_widget_show (page);
 
   sb_famous_history_max = tea_spinbutton (page, _("Text entry history limit"), confile.famous_history_max);
-  sb_max_recent_items = tea_spinbutton (page, _("Max. recent files"), confile.max_recent_items); 
+  sb_max_recent_items = tea_spinbutton (page, _("Max. recent files"), confile.max_recent_items);
  sb_logmemo_lines_max = tea_spinbutton (page, _("Logmemo lines limit"), confile.logmemo_lines_max);
- 
+
   return page;
 }
 */
@@ -667,21 +675,23 @@ static GtkWidget* wnd_options_interface (void)
 {
   GtkWidget *page = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (page);
-  
+
   sb_saveopen_w = tea_spinbutton (page, _("Save/Open dialogs width per cents"), confile.saveopen_w);
   sb_saveopen_h = tea_spinbutton (page, _("Save/Open dialogs height per cents"), confile.saveopen_h);
   sb_thumb_width = tea_spinbutton (page, _("Image thumbnail width"), confile.thumb_width);
   sb_thumb_height = tea_spinbutton (page, _("Image thumbnail height"), confile.thumb_height);
-  cb_main_wnd_show_full_path = tea_checkbox (page, _("Show the full path in the window caption"), confile.main_wnd_show_full_path); 
-  cb_do_show_main_toolbar = tea_checkbox (page, _("Show the main toolbar"), confile.do_show_main_toolbar); 
-  cb_common_closing_button_visible = tea_checkbox (page, _("Show the common close button"), confile.common_closing_button_visible); 
-  cb_fullscreen = tea_checkbox (page, _("Start in the full-screen mode"), confile.fullscreen); 
+  cb_main_wnd_show_full_path = tea_checkbox (page, _("Show the full path in the window caption"), confile.main_wnd_show_full_path);
+  cb_do_show_main_toolbar = tea_checkbox (page, _("Show the main toolbar"), confile.do_show_main_toolbar);
+  cb_common_closing_button_visible = tea_checkbox (page, _("Show the common close button"), confile.common_closing_button_visible);
+  cb_show_cursor_pos = tea_checkbox (page, _("Show the cursor position at statusbar"), confile.show_cursor_pos);
 
-  tv_notetab_positions = tea_combo_combo (page, gl_notetab_positions, _("Tabs position"), confile.notebook_tab_position); 
+  cb_fullscreen = tea_checkbox (page, _("Start in the full-screen mode"), confile.fullscreen);
 
-  sb_max_recent_items = tea_spinbutton (page, _("Max. recent files"), confile.max_recent_items); 
+  tv_notetab_positions = tea_combo_combo (page, gl_notetab_positions, _("Tabs position"), confile.notebook_tab_position);
+
+  sb_max_recent_items = tea_spinbutton (page, _("Max. recent files"), confile.max_recent_items);
   sb_logmemo_lines_max = tea_spinbutton (page, _("Logmemo lines limit"), confile.logmemo_lines_max);
- 
+
 
   return page;
 }
@@ -692,22 +702,21 @@ static GtkWidget* wnd_options_switches (void)
   GtkWidget *page = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (page);
 
-  cb_do_crash_file = tea_checkbox (page, _("Save the crashfile on terminate"), confile.do_crash_file); 
-  cb_do_save_all_on_termination = tea_checkbox (page, _("Save all on the abnormal termination (SIGTERM, etc)"), confile.do_save_all_on_termination); 
-  
-  cb_autorestore_last_session = tea_checkbox (page, _("Restore the last session on start"), confile.do_last_session); 
-  cb_start_with_blank_file = tea_checkbox (page, _("Start TEA with a blank file"), confile.start_with_blank_file); 
+  cb_do_crash_file = tea_checkbox (page, _("Save the crashfile on terminate"), confile.do_crash_file);
+  cb_do_save_all_on_termination = tea_checkbox (page, _("Save all on the abnormal termination (SIGTERM, etc)"), confile.do_save_all_on_termination);
 
-  cb_do_det_scripts_by_content = tea_checkbox (page, _("Determine a scripts highlighting by the content"), confile.do_det_scripts_by_content); 
+  cb_autorestore_last_session = tea_checkbox (page, _("Restore the last session on start"), confile.do_last_session);
+  cb_start_with_blank_file = tea_checkbox (page, _("Start TEA with a blank file"), confile.start_with_blank_file);
+
+  cb_do_det_scripts_by_content = tea_checkbox (page, _("Determine a scripts highlighting by the content"), confile.do_det_scripts_by_content);
 
 #ifndef CHAI_MODE
 
-gtk_widget_hide (cb_do_det_scripts_by_content);
-  
+  gtk_widget_hide (cb_do_det_scripts_by_content);
+
 #endif
 
-  cb_show_hidden_files = tea_checkbox (page, _("Show hidden files in Kwas"), confile.show_hidden_files); 
-  cb_use_snippets = tea_checkbox (page, _("Use snippets"), confile.use_snippets); 
+  cb_show_hidden_files = tea_checkbox (page, _("Show hidden files in Kwas"), confile.show_hidden_files);
   cb_use_ext_image_viewer = tea_checkbox (page, _("Use external image viewer"), confile.use_ext_image_viewer);
   cb_do_hl_on_fileopen = tea_checkbox (page, _("Highlight on file open"), confile.do_hl_on_fileopen);
   cb_do_backup = tea_checkbox (page, _("Do backup"), confile.do_backup);
@@ -724,10 +733,10 @@ static GtkWidget* wnd_options_confirmations (void)
 {
   GtkWidget *page = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (page);
-  
-  cb_prompt_on_not_saved = tea_checkbox (page, _("Warn before closing a modified file"), confile.prompt_on_not_saved); 
-  cb_prompt_on_file_exists = tea_checkbox (page, _("Overwrite the existing file"), confile.prompt_on_file_exists); 
- 
+
+  cb_prompt_on_not_saved = tea_checkbox (page, _("Warn before closing a modified file"), confile.prompt_on_not_saved);
+  cb_prompt_on_file_exists = tea_checkbox (page, _("Overwrite the existing file"), confile.prompt_on_file_exists);
+
   return page;
 }
 
@@ -776,7 +785,7 @@ static void cb_restore_config (GtkButton *button,
   confile_reload ();
   doc_update_all();
   update_enc_menu();
-  dlg_info (wnd_options, _("Configuration restored, please close and reopen the Preferences window."), NULL);  
+  dlg_info (wnd_options, _("Configuration restored, please close and reopen the Preferences window."), NULL);
 }
 
 
@@ -788,8 +797,8 @@ static void cb_config_reset (GtkButton *button,
        confile_reload ();
        doc_update_all();
        update_enc_menu();
-       dlg_info (wnd_options, _("Configuration is reseted to default, please close and reopen the Preferences window."), NULL);  
-      }  
+       dlg_info (wnd_options, _("Configuration is reseted to default, please close and reopen the Preferences window."), NULL);
+      }
 }
 
 
@@ -797,20 +806,20 @@ static GtkWidget* wnd_options_maintenance (void)
 {
   GtkWidget *page = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (page);
-  
+
   GtkWidget *b = tea_button_at_box (page, _("Backup the current config" ), cb_backup_config, NULL);
   b = tea_button_at_box (page, _("Config restore" ), cb_restore_config, NULL);
   b = tea_button_at_box (page, _("Reset config to default" ), cb_config_reset, NULL);
   b = tea_button_at_box (page, _("Put the launcher to the desktop" ), cb_put_launcher, NULL);
   b = tea_button_at_box (page, _("Remove the launcher from the desktop" ), cb_del_launcher, NULL);
-  
+
   return page;
 }
 
 
 static GtkWidget* wnd_options_colors (void)
 {
-  GtkWidget *notebook = gtk_notebook_new ();         
+  GtkWidget *notebook = gtk_notebook_new ();
   gtk_widget_show (notebook);
 
   GtkWidget *page = gtk_vbox_new (FALSE, UI_PACKFACTOR);
@@ -831,14 +840,14 @@ static GtkWidget* wnd_options_colors (void)
   ent_tag_current_line_bg = tea_color_selector (page, _("Current line background"), confile.tag_current_line_bg);
 
   ent_tag_spellcheck = tea_color_selector (page, _("Spellchecker marks"), confile.tag_spellcheck);
-  
+
   page = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (page);
   gtk_notebook_append_page (notebook, page, gtk_label_new (_("Highlighting")));
- 
+
   ent_tag_comment = tea_color_selector (page, _("Comments"), confile.tag_comment);
   ent_tag_identifier = tea_color_selector (page, _("Identifiers"), confile.tag_identifier);
-  ent_tag_digit = tea_color_selector (page, _("Digits"), confile.tag_digit); 
+  ent_tag_digit = tea_color_selector (page, _("Digits"), confile.tag_digit);
   ent_tag_string = tea_color_selector (page, _("Strings"), confile.tag_string);
   ent_tag_html_tag = tea_color_selector (page, _("HTML tags"), confile.tag_html_tag);
   ent_tag_preprocessor = tea_color_selector (page, _("Preprocessor"), confile.tag_preprocessor);
@@ -847,13 +856,13 @@ static GtkWidget* wnd_options_colors (void)
   #ifdef CHAI_MODE
 
   gtk_widget_hide (page);
-  
+
   #endif
 
   page = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (page);
   gtk_notebook_append_page (notebook, page, gtk_label_new (_("IDE")));
- 
+
   ent_tag_color_lm_error_fg = tea_color_selector (page, _("Errors foreground (logmemo)"), confile.tag_color_lm_error_fg);
   ent_tag_color_lm_error_bg = tea_color_selector (page, _("Errors background (logmemo)"), confile.tag_color_lm_error_bg);
 
@@ -868,20 +877,20 @@ static GtkWidget* wnd_options_fonts (void)
 
 #ifndef CHAI_MODE
 
-  fb_editor_font = tea_font_button (page, _("Editor font"), confile.editor_font); 
+  fb_editor_font = tea_font_button (page, _("Editor font"), confile.editor_font);
 
 #endif
 
-  fb_tag_comment_font = tea_font_button (page, _("Comments font"), confile.tag_comment_font); 
-  fb_tag_identifier_font = tea_font_button (page, _("Identifiers font"), confile.tag_identifier_font); 
-  fb_tag_digit_font = tea_font_button (page, _("Digits font"), confile.tag_digit_font); 
-  fb_tag_string_font = tea_font_button (page, _("Strings font"), confile.tag_string_font); 
-  fb_tag_html_tag_font = tea_font_button (page, _("HTML tags font"), confile.tag_html_tag_font); 
-  fb_tag_preprocessor_font = tea_font_button (page, _("Preprocessor font"), confile.tag_preprocessor_font); 
-  fb_tag_type_font = tea_font_button (page, _("Types font"), confile.tag_type_font); 
+  fb_tag_comment_font = tea_font_button (page, _("Comments font"), confile.tag_comment_font);
+  fb_tag_identifier_font = tea_font_button (page, _("Identifiers font"), confile.tag_identifier_font);
+  fb_tag_digit_font = tea_font_button (page, _("Digits font"), confile.tag_digit_font);
+  fb_tag_string_font = tea_font_button (page, _("Strings font"), confile.tag_string_font);
+  fb_tag_html_tag_font = tea_font_button (page, _("HTML tags font"), confile.tag_html_tag_font);
+  fb_tag_preprocessor_font = tea_font_button (page, _("Preprocessor font"), confile.tag_preprocessor_font);
+  fb_tag_type_font = tea_font_button (page, _("Types font"), confile.tag_type_font);
 
   GtkButton *bt_change_all_fonts = tea_button_at_box (page, _("Change all fonts"), cb_change_all_fonts, NULL);
- 
+
   return page;
 }
 
@@ -890,33 +899,36 @@ static GtkWidget* wnd_options_editor (void)
 {
   GtkWidget *page = gtk_vbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (page);
- 
+
 #ifdef CHAI_MODE
 
-  fb_editor_font = tea_font_button (page, _("Font"), confile.editor_font); 
+  fb_editor_font = tea_font_button (page, _("Font"), confile.editor_font);
 
 #endif
 
-  ent_def_markup_mode = tea_combo (page, gl_markup_modes, _("Default markup mode"), confile.default_markup_mode);
+  sb_max_undo_levels = tea_spinbutton (page, _("Max undo levels"), confile.max_undo_levels);
+
+  //ent_def_markup_mode = tea_combo (page, gl_markup_modes, _("Default markup mode"), confile.default_markup_mode);
   sb_tab_size = tea_spinbutton (page, _("Tab size in spaces"), confile.tab_size);
   cb_homeend = tea_checkbox (page, _("Smart Home/End"), confile.homeend);
 
   cb_ins_spaces_on_tab_press = tea_checkbox (page, _("Insert spaces on TAB key"), confile.ins_spaces_on_tab_press);
   cb_use_auto_indent = tea_checkbox (page, _("Automatic indent"), confile.use_auto_indent);
   cb_do_autorep = tea_checkbox (page, _("Autoreplace"), confile.do_autorep);
-  cb_autoclose_tags = tea_checkbox (page, _("Autoclose tags"), confile.autoclose_tags);  
-  cb_use_cursor_blink = tea_checkbox (page, _("Cursor blink"), confile.use_cursor_blink);  
+  cb_autoclose_tags = tea_checkbox (page, _("Autoclose tags"), confile.autoclose_tags);
+  cb_use_cursor_blink = tea_checkbox (page, _("Cursor blink"), confile.use_cursor_blink);
   sb_autosave_interval = tea_spinbutton (page, _("Autosaving interval, minutes"), confile.autosave_interval);
   cb_do_autosave = tea_checkbox (page, _("Enable the autosaving"), confile.do_autosave);
   cb_hl_current_line = tea_checkbox (page, _("Highlight the current line"), confile.hl_current_line);
 
-  cb_margin_visible = tea_checkbox (page, _("Show the margin"), confile.margin_visible);  
+  cb_margin_visible = tea_checkbox (page, _("Show the margin"), confile.margin_visible);
   sb_margin_width = tea_spinbutton (page, _("Position of the margin (GtkSourceView-only)"), confile.margin_width);
 
 #ifndef CHAI_MODE
 
   gtk_widget_hide (cb_margin_visible);
-  
+  gtk_widget_hide (sb_max_undo_levels);
+
 #endif
 
 
