@@ -32,10 +32,10 @@
 #include "tea_config.h"
 
 
-static void cb_proj_on_bt_apply (GtkObject *object, gpointer user_data) 
+static void cb_proj_on_bt_apply (GtkObject *object, gpointer user_data)
 {
   t_ppair *t = user_data;
-  t_tea_project *p = t->b;   
+  t_tea_project *p = t->b;
 
    if (p)
       {
@@ -44,7 +44,7 @@ static void cb_proj_on_bt_apply (GtkObject *object, gpointer user_data)
        p->dir_source = ch_str (p->dir_source, gtk_entry_get_text (GTK_ENTRY (ent_dir_source)));
        p->file_executable = ch_str (p->file_executable, gtk_entry_get_text (GTK_ENTRY (ent_file_executable)));
       }
-  
+
   window_destroy (t->a);
   g_free (t);
 }
@@ -59,14 +59,14 @@ static GtkWidget* mk_page_main (t_tea_project *p)
   ent_dir_makefile = tea_dir_selector (page, _("Makefile directory"), p->dir_makefile);
   ent_dir_source = tea_dir_selector (page, _("Source directory"), p->dir_source);
   ent_file_executable = tea_text_entry (page, _("Target executable"), p->file_executable);
-  
+
   return page;
 }
 
 
 t_tea_project* tea_proj_new (void)
 {
-  t_tea_project *p = g_malloc (sizeof (t_tea_project)); 
+  t_tea_project *p = g_malloc (sizeof (t_tea_project));
 
   p->project_name = g_strdup ("noname");
   p->project_file_name = NULL;
@@ -75,17 +75,17 @@ t_tea_project* tea_proj_new (void)
   p->file_executable = NULL;
 
   return p;
-} 
+}
 
 
 t_tea_project* tea_proj_clear_new (t_tea_project *p)
 {
   tea_proj_free (p);
   return tea_proj_new ();
-} 
+}
 
 
-t_tea_project* tea_proj_open (gchar *name)
+t_tea_project* tea_proj_open (const gchar *name)
 {
   t_tea_project *r = tea_proj_new ();
   GHashTable *ht = load_file_to_hashtable (name);
@@ -96,7 +96,7 @@ t_tea_project* tea_proj_open (gchar *name)
   r->dir_source = ht_get_gchar_value (ht, "dir_source", "noname");
   r->file_executable = ht_get_gchar_value (ht, "file_executable", "noname");
 
-  g_hash_table_destroy (ht); 
+  g_hash_table_destroy (ht);
   return r;
 }
 
@@ -107,10 +107,10 @@ void tea_proj_save (t_tea_project *p)
      return;
 
   if (! p->project_file_name)
-     return; 
+     return;
 
   GList *l = NULL;
-  
+
   l = add_to_glist_combined_str (l, "file_executable", p->file_executable);
   l = add_to_glist_combined_str (l, "project_name", p->project_name);
   l = add_to_glist_combined_str (l, "project_file_name", p->project_file_name);
@@ -118,22 +118,19 @@ void tea_proj_save (t_tea_project *p)
   l = add_to_glist_combined_str (l, "dir_source", p->dir_source);
 
   glist_save_to_file (l, p->project_file_name);
-  g_list_free (l); 
+  g_list_free (l);
 }
 
 
-void tea_proj_save_as (t_tea_project *p, gchar *name)
+void tea_proj_save_as (t_tea_project *p, const gchar *name)
 {
-  if (! p)
-     return;
-
-  if (! name)
+  if (! p || ! name)
      return;
 
   p->project_file_name = ch_str (p->project_file_name, name);
 
   GList *l = NULL;
-  
+
   l = add_to_glist_combined_str (l, "file_executable", p->file_executable);
   l = add_to_glist_combined_str (l, "project_name", p->project_name);
   l = add_to_glist_combined_str (l, "project_file_name", p->project_file_name);
@@ -141,7 +138,7 @@ void tea_proj_save_as (t_tea_project *p, gchar *name)
   l = add_to_glist_combined_str (l, "dir_source", p->dir_source);
 
   glist_save_to_file (l, p->project_file_name);
-  g_list_free (l); 
+  g_list_free (l);
 }
 
 
@@ -165,7 +162,7 @@ void tea_proj_run (t_tea_project *p)
      return;
 
   if (! p->file_executable)
-     return; 
+     return;
 
   gchar *f = g_strconcat (p->dir_source, p->file_executable, NULL);
   g_spawn_command_line_async (f, NULL);
@@ -176,7 +173,7 @@ void tea_proj_run (t_tea_project *p)
 GtkWidget* create_proj_props_window (t_tea_project *p)
 {
   if (! p)
-    return NULL; 
+    return NULL;
 
   wnd_proj_props = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
@@ -186,7 +183,7 @@ GtkWidget* create_proj_props_window (t_tea_project *p)
   gtk_widget_show (vbox1);
   gtk_container_add (GTK_CONTAINER (wnd_proj_props), vbox1);
 
-  GtkWidget *notebook = gtk_notebook_new ();         
+  GtkWidget *notebook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (notebook, GTK_POS_LEFT);
 
   gtk_widget_show (notebook);
@@ -194,7 +191,7 @@ GtkWidget* create_proj_props_window (t_tea_project *p)
   gtk_container_add (GTK_CONTAINER (vbox1), notebook);
 
   gtk_notebook_append_page (notebook, mk_page_main (p), gtk_label_new (_("Main")));
-  
+
   GtkWidget *hbox1 = gtk_hbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (hbox1);
   gtk_container_add (GTK_CONTAINER (vbox1), hbox1);
@@ -204,7 +201,7 @@ GtkWidget* create_proj_props_window (t_tea_project *p)
   gtk_container_add (GTK_CONTAINER (hbox1), bt_close);
   g_signal_connect (bt_close, "clicked", G_CALLBACK (cb_on_bt_close), wnd_proj_props);
 
-  t_ppair *pair = g_malloc (sizeof (t_ppair)); 
+  t_ppair *pair = g_malloc (sizeof (t_ppair));
   pair->a = wnd_proj_props;
   pair->b = p;
 
@@ -213,11 +210,11 @@ GtkWidget* create_proj_props_window (t_tea_project *p)
   gtk_container_add (GTK_CONTAINER (hbox1), bt_apply);
   g_signal_connect (bt_apply, "clicked", G_CALLBACK (cb_proj_on_bt_apply), pair);
 
-  gtk_window_set_position (wnd_proj_props, GTK_WIN_POS_CENTER);   
+  gtk_window_set_position (wnd_proj_props, GTK_WIN_POS_CENTER);
   gtk_window_set_modal (wnd_proj_props, TRUE);
 
   g_signal_connect (G_OBJECT (wnd_proj_props), "key_press_event", G_CALLBACK (win_key_handler), wnd_proj_props);
- 
+
   gtk_widget_show (wnd_proj_props);
   return wnd_proj_props;
 }
