@@ -2,7 +2,7 @@
                           tea_gtk_utils.c  -  description
                              -------------------
     begin                : aug. 2004
-    copyright            : (C) 2004, 2005 by Peter 'Roxton' Semiletov
+    copyright            : (C) 2004-2006 by Peter 'Roxton' Semiletov
     email                : peter.semiletov@gmail.com
  ***************************************************************************/
 
@@ -25,23 +25,23 @@
 #include "aspell.h"
 
 
-GList* get_speller_modules_list (gchar *default_lang)
+GList* get_speller_modules_list (const gchar *default_lang)
 {
   gchar *lang;
   if (! default_lang)
      lang = g_strdup ("C");
-  else 
-      lang = g_strdup (default_lang); 
+  else
+      lang = g_strdup (default_lang);
 
-  GList *l = NULL; 
+  GList *l = NULL;
   AspellDictInfoList *dlist;
   AspellDictInfoEnumeration *dels;
   const AspellDictInfo *entry;
   AspellConfig *config;
 
   config = new_aspell_config ();
-  aspell_config_replace (config, "lang", lang); 
-  aspell_config_replace (config, "encoding", "UTF-8"); 
+  aspell_config_replace (config, "lang", lang);
+  aspell_config_replace (config, "encoding", "UTF-8");
 
   dlist = get_aspell_dict_info_list (config);
 
@@ -49,34 +49,35 @@ GList* get_speller_modules_list (gchar *default_lang)
 
   dels = aspell_dict_info_list_elements(dlist);
 
-  while ((entry = aspell_dict_info_enumeration_next(dels)) != 0) 
+  while ((entry = aspell_dict_info_enumeration_next(dels)) != 0)
         if (entry)
-           l = g_list_prepend (l, g_strdup (entry->name)); 	 
+           l = g_list_prepend (l, g_strdup (entry->name));
 
   delete_aspell_dict_info_enumeration(dels);
   g_free (lang);
 
-  return l;  
+  return l;
 }
 
-GList* get_suggestions_list (gchar *lang, gchar *s)
+
+GList* get_suggestions_list (const gchar *lang, const gchar *s)
 {
   GList *result = NULL;
 
-  AspellConfig *spell_config = new_aspell_config(); 
+  AspellConfig *spell_config = new_aspell_config ();
 
   AspellCanHaveError *ret;
   AspellSpeller *speller;
-  
-  aspell_config_replace (spell_config, "lang", lang); 
-  aspell_config_replace (spell_config, "encoding", "UTF-8"); 
+
+  aspell_config_replace (spell_config, "lang", lang);
+  aspell_config_replace (spell_config, "encoding", "UTF-8");
 
   ret = new_aspell_speller (spell_config);
 
   if (aspell_error (ret) != 0)
      {
       g_print ("Error: %s\n", aspell_error_message (ret));
-      delete_aspell_config (spell_config); 
+      delete_aspell_config (spell_config);
       delete_aspell_can_have_error (ret);
       return NULL;
      }
@@ -94,17 +95,14 @@ GList* get_suggestions_list (gchar *lang, gchar *s)
   const char *word;
 
   while (word = aspell_string_enumeration_next (elements))
-        {
-         result = g_list_prepend (result, g_strdup (word));
-        }
+        result = g_list_prepend (result, g_strdup (word));
 
   delete_aspell_string_enumeration (elements);
-  
+
   delete_aspell_speller (speller);
   delete_aspell_config (spell_config);
 
   return g_list_reverse (result);
 }
- 
 
 #endif
